@@ -21,6 +21,7 @@ describe('Gerbera Autoscan', () => {
     let autoscanPersistent;
     let autoscanRecursive;
     let autoscanHidden;
+    let autoscanFollowSymlinks;
     let autoscanInterval;
     let autoscanPersistentMsg;
     let autoscanSave;
@@ -45,6 +46,7 @@ describe('Gerbera Autoscan', () => {
       autoscanPersistent = $('#autoscanPersistent');
       autoscanRecursive = $('#autoscanRecursive');
       autoscanHidden = $('#autoscanHidden');
+      autoscanFollowSymlinks = $('#autoscanSymlinks');
       autoscanInterval = $('#autoscanInterval');
       autoscanPersistentMsg = $('#autoscan-persistent-msg');
       autoscanSave = $('#autoscanSave');
@@ -75,6 +77,7 @@ describe('Gerbera Autoscan', () => {
       expect(autoscanPersistent.is(':checked')).toBeFalsy();
       expect(autoscanRecursive.is(':checked')).toBeFalsy();
       expect(autoscanHidden.is(':checked')).toBeFalsy();
+      expect(autoscanFollowSymlinks.is(':checked')).toBeFalsy();
       expect(autoscanInterval.val()).toBe('');
       expect(autoscanPersistentMsg.css('display')).toBe('none');
       expect(autoscanSave.is(':disabled')).toBeFalsy();
@@ -103,6 +106,9 @@ describe('Gerbera Autoscan', () => {
       event = {
         data: {
           id: '26fd6',
+          retryCount: '',
+          dirTypes: false,
+          forceRescan: false,
           audio: true,
           audioMusic: true,
           audioBook: true,
@@ -112,7 +118,10 @@ describe('Gerbera Autoscan', () => {
           video: true,
           videoMovie: true,
           videoTV: true,
-          videoMusicVideo: true
+          videoMusicVideo: true,
+          ctAudio: 'object.container.audioMusic',
+          ctImage: 'object.container.photoAlbum',
+          ctVideo: 'object.container.videoContainer',
         }
       };
       spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
@@ -121,6 +130,7 @@ describe('Gerbera Autoscan', () => {
     afterEach(() => {
       fixture.cleanup();
       ajaxSpy.and.callThrough();
+      $('#autoscanModal').remove();
     });
 
     it('calls the server to obtain autoscan edit load', () => {
@@ -129,6 +139,10 @@ describe('Gerbera Autoscan', () => {
         req_type: 'autoscan',
         action: 'as_edit_load',
         object_id: '26fd6',
+        interval: undefined,
+        retryCount: '',
+        dirTypes: false,
+        forceRescan: false,
         audio: true,
         audioMusic: true,
         audioBook: true,
@@ -139,7 +153,10 @@ describe('Gerbera Autoscan', () => {
         videoMovie: true,
         videoTV: true,
         videoMusicVideo: true,
-        from_fs: true
+        from_fs: true,
+        ctAudio: 'object.container.audioMusic',
+        ctImage: 'object.container.photoAlbum',
+        ctVideo: 'object.container.videoContainer',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -156,6 +173,9 @@ describe('Gerbera Autoscan', () => {
         action: 'as_edit_load',
         object_id: '26fd6',
         from_fs: false,
+        retryCount: '',
+        dirTypes: false,
+        forceRescan: false,
         audio: true,
         audioMusic: true,
         audioBook: true,
@@ -166,7 +186,10 @@ describe('Gerbera Autoscan', () => {
         videoMovie: true,
         videoTV: true,
         videoMusicVideo: true,
-        updates: 'check'
+        updates: 'check',
+        ctAudio: 'object.container.audioMusic',
+        ctImage: 'object.container.photoAlbum',
+        ctVideo: 'object.container.videoContainer',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -183,6 +206,7 @@ describe('Gerbera Autoscan', () => {
     let autoscanPersistent;
     let autoscanRecursive;
     let autoscanHidden;
+    let autoscanFollowSymlinks;
     let autoscanInterval;
     let autoscanPersistentMsg;
     let autoscanSave;
@@ -207,6 +231,7 @@ describe('Gerbera Autoscan', () => {
       autoscanPersistent = $('#autoscanPersistent');
       autoscanRecursive = $('#autoscanRecursive');
       autoscanHidden = $('#autoscanHidden');
+      autoscanFollowSymlinks = $('#autoscanSymlinks');
       autoscanInterval = $('#autoscanInterval');
       autoscanPersistentMsg = $('#autoscan-persistent-msg');
       autoscanSave = $('#autoscanSave');
@@ -239,6 +264,7 @@ describe('Gerbera Autoscan', () => {
       expect(autoscanPersistent.is(':checked')).toBeFalsy();
       expect(autoscanRecursive.is(':checked')).toBeTruthy();
       expect(autoscanHidden.is(':checked')).toBeFalsy();
+      expect(autoscanFollowSymlinks.is(':checked')).toBeFalsy();
       expect(autoscanInterval.val()).toBe('1800');
       expect(autoscanPersistentMsg.css('display')).toBe('none');
       expect(autoscanSave.is(':disabled')).toBeFalsy();
@@ -291,6 +317,10 @@ describe('Gerbera Autoscan', () => {
         from_fs: true,
         scan_mode: 'timed',
         recursive: true,
+        hidden: false,
+        dirTypes: false,
+        forceRescan: false,
+        followSymlinks: false,
         audio: true,
         audioMusic: true,
         audioBook: true,
@@ -301,8 +331,10 @@ describe('Gerbera Autoscan', () => {
         videoMovie: true,
         videoTV: true,
         videoMusicVideo: true,
-        hidden: false,
-        interval: '1800'
+        interval: '1800',
+        ctAudio: '',
+        ctImage: '',
+        ctVideo: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
