@@ -1,8 +1,31 @@
-import {GerberaApp} from "../../../web/js/gerbera-app.module";
-import {Items} from '../../../web/js/gerbera-items.module';
-import {Trail} from "../../../web/js/gerbera-trail.module";
-import {Updates} from "../../../web/js/gerbera-updates.module";
-import {Auth} from "../../../web/js/gerbera-auth.module";
+/*GRB*
+
+    Gerbera - https://gerbera.io/
+
+    gerbera.items.spec.js - this file is part of Gerbera.
+
+    Copyright (C) 2016-2025 Gerbera Contributors
+
+    Gerbera is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation.
+
+    Gerbera is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Gerbera.  If not, see <http://www.gnu.org/licenses/>.
+
+    $Id$
+*/
+import { GerberaApp } from "../../../web/js/gerbera-app.module";
+import { Items, BrowseItemView, SearchItemView, FileItemView } from '../../../web/js/gerbera-items.module';
+import { Trail } from "../../../web/js/gerbera-trail.module";
+import { Updates } from "../../../web/js/gerbera-updates.module";
+import { Auth } from "../../../web/js/gerbera-auth.module";
+
 import mockConfig from './fixtures/config';
 import itemsResponse from './fixtures/parent_id-7443-start-0';
 import gerberaItems from './fixtures/gerbera-items';
@@ -17,15 +40,21 @@ import treeDataJson from './fixtures/tree-data';
 
 describe('Gerbera Items', () => {
   let lsSpy;
+  let viewFactoryData;
   beforeEach(() => {
     fixture.setBase('test/client/fixtures');
     fixture.load('index.html');
+    viewFactoryData = {
+      gerbera: {
+        id: 1235
+      }
+    };
     lsSpy = spyOn(window.localStorage, 'getItem').and.callFake((name) => {
-        return;
+      return;
     });
   });
   afterEach((done) => {
-    $("body").on('transitionend', function(event){
+    $("body").on('transitionend', function (event) {
       $('#editModal').remove();
       $('.modal-backdrop').remove();
     });
@@ -33,19 +62,19 @@ describe('Gerbera Items', () => {
     done();
   });
   beforeAll(() => {
-    spyOn(GerberaApp, 'writeLocalStorage').and.callFake(() => {});
+    spyOn(GerberaApp, 'writeLocalStorage').and.callFake(() => { });
   });
 
   describe('initialize()', () => {
-   beforeEach(() => {
-     GerberaApp.serverConfig = mockConfig.config;
-   });
+    beforeEach(() => {
+      GerberaApp.serverConfig = mockConfig.config;
+    });
 
-   it('clears the datagrid', async () => {
-     await Items.initialize();
-     expect($('#datagrid').text()).toBe('');
-   });
- });
+    it('clears the datagrid', async () => {
+      await Items.initialize();
+      expect($('#datagrid').text()).toBe('');
+    });
+  });
   describe('treeItemSelected()', () => {
     let ajaxSpy;
 
@@ -60,31 +89,23 @@ describe('Gerbera Items', () => {
     });
 
     it('updates the breadcrumb based on the selected item', async () => {
-      const data = {
-        gerbera: {
-          id: 12345
-        }
-      };
       spyOn(GerberaApp, 'getType').and.returnValue('db');
-      GerberaApp.serverConfig = {};
-      GerberaApp.pageInfo.viewItems = 25;
+      GerberaApp.serverConfig = mockConfig.config;
 
       await Items.initialize();
-      Items.treeItemSelected(data);
+      Items.treeItemSelected(viewFactoryData);
 
-      expect(GerberaApp.currentTreeItem).toEqual(data);
+      expect(GerberaApp.currentTreeItem).toEqual(viewFactoryData);
       expect(ajaxSpy.calls.count()).toBe(1);
       expect(ajaxSpy.calls.mostRecent().args[0].data.req_type).toBe('items');
-      expect(ajaxSpy.calls.mostRecent().args[0].data.parent_id).toBe(12345);
+      expect(ajaxSpy.calls.mostRecent().args[0].data.parent_id).toBe(1235);
       expect(ajaxSpy.calls.mostRecent().args[0].data.start).toBe(0);
-      expect(ajaxSpy.calls.mostRecent().args[0].data.count).toBe(25);
+      expect(ajaxSpy.calls.mostRecent().args[0].data.count).toBe(50);
     });
   });
   describe('transformItems()', () => {
-    let widgetList;
-
     beforeEach(() => {
-      GerberaApp.serverConfig = {};
+      GerberaApp.serverConfig = mockConfig.config;
     });
 
     it('converts a gerbera list of items to the widget accepted list', () => {
@@ -94,9 +115,10 @@ describe('Gerbera Items', () => {
   });
   describe('loadItems()', () => {
     beforeEach(() => {
-      GerberaApp.serverConfig = {};
+      GerberaApp.serverConfig = mockConfig.config;
       spyOn(GerberaApp, 'getType').and.returnValue('db');
       spyOn(GerberaApp, 'gridMode').and.returnValue(0);
+      Items.viewFactory(viewFactoryData);
       $('#tree').tree({
         data: treeDataJson,
         config: {
@@ -145,15 +167,14 @@ describe('Gerbera Items', () => {
 
       Items.loadItems(itemsResponse);
 
-      expect(Trail.makeTrailFromItem).toHaveBeenCalledWith(itemsResponse.items);
+      expect(Trail.makeTrailFromItem).toHaveBeenCalledWith(itemsResponse.items, undefined);
     });
   });
   describe('loadItems() for files', () => {
-    let response;
-
     beforeEach(() => {
-      GerberaApp.serverConfig = {};
+      GerberaApp.serverConfig = mockConfig.config;
       spyOn(GerberaApp, 'getType').and.returnValue('fs');
+      Items.viewFactory(viewFactoryData);
       $('#tree').tree({
         data: treeDataJson,
         config: {
@@ -200,6 +221,7 @@ describe('Gerbera Items', () => {
       ajaxSpy = spyOn($, 'ajax').and.callFake(() => {
         return Promise.resolve({});
       });
+      Items.viewFactory(viewFactoryData);
       item = { id: 913 };
     });
 
@@ -374,6 +396,7 @@ describe('Gerbera Items', () => {
         description: 'A%20description',
         updates: 'check',
         flags: 'Restricted',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -396,6 +419,7 @@ describe('Gerbera Items', () => {
         title: 'container%20title',
         updates: 'check',
         flags: 'Restricted',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -422,6 +446,7 @@ describe('Gerbera Items', () => {
         protocol: 'http-get',
         updates: 'check',
         flags: 'Restricted',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -559,7 +584,7 @@ describe('Gerbera Items', () => {
       spyOn(Updates, 'updateTreeByIds');
       spyOn(Updates, 'showMessage');
       ajaxSpy = spyOn($, 'ajax').and.callFake(() => {
-        return Promise.resolve({success: true});
+        return Promise.resolve({ success: true });
       });
       item = {
         id: 9999
@@ -572,7 +597,7 @@ describe('Gerbera Items', () => {
     });
 
     it('calls the server with the item details to add', async () => {
-      editModal.editmodal('addNewItem', {type: 'item', item: item});
+      editModal.editmodal('addNewItem', { type: 'item', item: item });
 
       await Items.addObject();
       var data = {
@@ -585,6 +610,7 @@ describe('Gerbera Items', () => {
         description: '',
         updates: 'check',
         flags: '',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -596,7 +622,7 @@ describe('Gerbera Items', () => {
       const parentId = {
         id: 0
       };
-      editModal.editmodal('addNewItem', {type: 'container', item: parentId});
+      editModal.editmodal('addNewItem', { type: 'container', item: parentId });
 
       await Items.addObject();
       var data = {
@@ -607,6 +633,7 @@ describe('Gerbera Items', () => {
         title: '',
         updates: 'check',
         flags: '',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -615,7 +642,7 @@ describe('Gerbera Items', () => {
     });
 
     it('calls the server with the `container` details', async () => {
-      editModal.editmodal('addNewItem', {type: 'container', item: item});
+      editModal.editmodal('addNewItem', { type: 'container', item: item });
 
       await Items.addObject();
       var data = {
@@ -626,6 +653,7 @@ describe('Gerbera Items', () => {
         title: '',
         updates: 'check',
         flags: '',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -634,7 +662,7 @@ describe('Gerbera Items', () => {
     });
 
     it('calls the server with the `external_url` details', async () => {
-      editModal.editmodal('addNewItem', {type: 'external_url', item: item});
+      editModal.editmodal('addNewItem', { type: 'external_url', item: item });
 
       await Items.addObject();
       var data = {
@@ -649,6 +677,7 @@ describe('Gerbera Items', () => {
         protocol: 'http-get',
         updates: 'check',
         flags: '',
+        resources: '',
       };
       data[Auth.SID] = 'SESSION_ID';
 
@@ -657,7 +686,7 @@ describe('Gerbera Items', () => {
     });
 
     it('when successful adds a GERBERA.Updates UI timer to check for updates later', async () => {
-      editModal.editmodal('addNewItem', {type: 'item', item: item});
+      editModal.editmodal('addNewItem', { type: 'item', item: item });
       GerberaApp.currentTreeItem = {};
       await Items.addObject();
 
@@ -668,6 +697,7 @@ describe('Gerbera Items', () => {
     let ajaxSpy;
 
     beforeEach(() => {
+      GerberaApp.serverConfig = mockConfig.config;
       ajaxSpy = spyOn($, 'ajax').and.callFake(() => {
         return Promise.resolve({});
       });
@@ -675,30 +705,77 @@ describe('Gerbera Items', () => {
 
     afterEach(() => {
       ajaxSpy.and.callThrough();
+      GerberaApp.serverConfig = mockConfig.config;
     });
 
     it('calls the server for items given correct parameters', async () => {
       spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
 
-      await Items.retrieveGerberaItems('db', 0, 0, 25);
+      const view = new BrowseItemView(0);
+      view.start = 0;
+      // view.count = 25;
+      await view.retrieveGerberaItems(0);
       var data = {
-        req_type: 'db',
+        req_type: 'items',
+        action: 'browse',
         parent_id: 0,
         start: 0,
-        count: 25,
+        count: 50,
+        select_it: 0,
         updates: 'check'
       };
       data[Auth.SID] = 'SESSION_ID';
       expect(ajaxSpy.calls.count()).toBe(1);
       expect(ajaxSpy.calls.mostRecent().args[0].data).toEqual(data);
     });
+
+    it('calls the server for search given correct parameters', async () => {
+      spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
+      const view = new SearchItemView(0);
+      view.start = 0;
+      view.count = 50;
+      view.searchCriteria = 'dc:title contains "music"';
+      view.sortCriteria = '-dc:title';
+      await view.retrieveGerberaItems(0);
+      var data = {
+        req_type: 'items',
+        action: 'search',
+        parent_id: 0,
+        start: 0,
+        count: 50,
+        select_it: 0,
+        searchCriteria: 'dc:title contains "music"',
+        sortCriteria: '-dc:title',
+        searchableContainers: false
+      };
+      data[Auth.SID] = 'SESSION_ID';
+      expect(ajaxSpy.calls.count()).toBe(1);
+      expect(ajaxSpy.calls.mostRecent().args[0].data).toEqual(data);
+    });
+
+    it('calls the server for files given correct parameters', async () => {
+      spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
+      const view = new FileItemView(0);
+      view.start = 0;
+      await view.retrieveGerberaItems(0);
+      var data = {
+        req_type: 'files',
+        action: 'files',
+        parent_id: 0,
+        start: 0,
+        count: 50,
+        select_it: 0
+      };
+      data[Auth.SID] = 'SESSION_ID';
+      expect(ajaxSpy.calls.count()).toBe(1);
+      expect(ajaxSpy.calls.mostRecent().args[0].data).toEqual(data);
+    });
   });
   describe('retrieveItemsForPage()', () => {
     let ajaxSpy;
 
     beforeEach(async () => {
       GerberaApp.serverConfig = mockConfig.config;
-      GerberaApp.serverConfig = {};
 
       await Items.initialize();
       ajaxSpy = spyOn($, 'ajax').and.callFake(() => {
@@ -711,23 +788,27 @@ describe('Gerbera Items', () => {
     });
 
 
-    it('sends the page number, items per page, and total items to retreive items', async () => {
+    it('sends the page number, items per page, and total items to retrieve items', async () => {
       const event = {
         data: {
+          gridMode: 0,
           pageNumber: 1,
-          itemsPerPage: 25,
+          itemsPerPage: 50,
           parentId: 1235
         }
       };
       spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
       spyOn(GerberaApp, 'getType').and.returnValue('db');
 
+      Items.viewFactory(viewFactoryData);
       await Items.retrieveItemsForPage(event);
       var data = {
         req_type: 'items',
+        action: 'browse',
         parent_id: 1235,
         start: 0,
-        count: 25,
+        count: 50,
+        select_it: 0,
         updates: 'check'
       };
       data[Auth.SID] = 'SESSION_ID';
@@ -739,8 +820,9 @@ describe('Gerbera Items', () => {
     it('properly sets the start when page number is high', async () => {
       const event = {
         data: {
+          gridMode: 0,
           pageNumber: 5,
-          itemsPerPage: 25,
+          itemsPerPage: 50,
           parentId: 1235
         }
       };
@@ -748,12 +830,15 @@ describe('Gerbera Items', () => {
       spyOn(Auth, 'getSessionId').and.returnValue('SESSION_ID');
       spyOn(GerberaApp, 'getType').and.returnValue('db');
 
+      Items.viewFactory(viewFactoryData);
       await Items.retrieveItemsForPage(event);
       var data = {
         req_type: 'items',
+        action: 'browse',
         parent_id: 1235,
-        start: 100,
-        count: 25,
+        start: 200,
+        count: 50,
+        select_it: 0,
         updates: 'check'
       };
       data[Auth.SID] = 'SESSION_ID';
@@ -780,7 +865,8 @@ describe('Gerbera Items', () => {
     it('calls the server for the next page based on current page', async () => {
       const event = {
         data: {
-          itemsPerPage: 25,
+          gridMode: 0,
+          itemsPerPage: 50,
           totalMatches: 100,
           parentId: 1235
         }
@@ -789,12 +875,15 @@ describe('Gerbera Items', () => {
       spyOn(GerberaApp, 'getType').and.returnValue('db');
       Items.setPage(1);
 
+      Items.viewFactory(viewFactoryData);
       await Items.nextPage(event);
       var data = {
         req_type: 'items',
+        action: 'browse',
         parent_id: 1235,
-        start: 25,
-        count: 25,
+        start: 50,
+        count: 50,
+        select_it: 0,
         updates: 'check'
       };
       data[Auth.SID] = 'SESSION_ID';
@@ -822,7 +911,8 @@ describe('Gerbera Items', () => {
     it('calls the server for the previous page based on current page', async () => {
       const event = {
         data: {
-          itemsPerPage: 25,
+          gridMode: 0,
+          itemsPerPage: 50,
           totalMatches: 100,
           parentId: 1235
         }
@@ -831,12 +921,15 @@ describe('Gerbera Items', () => {
       spyOn(GerberaApp, 'getType').and.returnValue('db');
       Items.setPage(2);
 
+      Items.viewFactory(viewFactoryData);
       await Items.previousPage(event);
       var data = {
         req_type: 'items',
+        action: 'browse',
         parent_id: 1235,
         start: 0,
-        count: 25,
+        count: 50,
+        select_it: 0,
         updates: 'check'
       };
       data[Auth.SID] = 'SESSION_ID';
