@@ -238,7 +238,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
       expect(result).toEqual({
         object_id: '39479',
         title: 'Test.mp4',
-        description: 'A%20description'
+        description: 'A%20description',
+        flags: 'Restricted',
+        resources: '',
       });
     });
 
@@ -250,7 +252,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
 
       expect(result).toEqual({
         object_id: '1471',
-        title: 'container%20title'
+        title: 'container%20title',
+        flags: 'Restricted',
+        resources: '',
       });
     });
 
@@ -266,7 +270,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
         location: 'http%3A%2F%2Flocalhost',
         description: 'description',
         'mime-type': 'video%2Fts',
-        protocol: 'http-get'
+        flags: 'Restricted',
+        protocol: 'http-get',
+        resources: '',
       });
     });
   });
@@ -293,7 +299,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
         class: 'object.item',
         location: '',
         title: '',
-        description: ''
+        flags: '',
+        description: '',
+        resources: '',
       });
     });
 
@@ -306,7 +314,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
         parent_id: '9999',
         obj_type: 'container',
         class: 'object.container',
-        title: ''
+        flags: '',
+        title: '',
+        resources: '',
       });
     });
 
@@ -323,7 +333,9 @@ describe('The jQuery Gerbera Editor Overlay', () => {
         location: '',
         description: '',
         'mime-type': '',
-        protocol: 'http-get'
+        flags: '',
+        protocol: 'http-get',
+        resources: '',
       });
     });
   });
