@@ -3,7 +3,7 @@
     Gerbera - https://gerbera.io/
 
     config_setup.cc - this file is part of Gerbera.
-    Copyright (C) 2020-2024 Gerbera Contributors
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -81,7 +81,7 @@ std::string ConfigSetup::getXmlContent(const pugi::xml_node& root, bool trim)
         return optValue;
     }
 
-    auto xAttr = ConfigDefinition::removeAttribute(option);
+    auto xAttr = definition->removeAttribute(option);
 
     if (root.attribute(xAttr.c_str())) {
         std::string optValue = trim ? trimString(root.attribute(xAttr.c_str()).as_string()) : root.attribute(xAttr.c_str()).as_string();
