@@ -3,7 +3,8 @@
     Gerbera - https://gerbera.io/
 
     config_setup_array.cc - this file is part of Gerbera.
-    Copyright (C) 2020-2024 Gerbera Contributors
+
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,18 +30,22 @@
 #include "config/config_definition.h"
 #include "config/config_options.h"
 #include "config/config_val.h"
+#include "config_setup_bool.h"
 #include "config_setup_int.h"
 #include "util/logger.h"
 
 #include <numeric>
 
 /// \brief Creates an array of strings from an XML nodeset.
-bool ConfigArraySetup::createOptionFromNode(const pugi::xml_node& element, std::vector<std::string>& result) const
+bool ConfigArraySetup::createOptionFromNode(
+    const pugi::xml_node& element,
+    std::vector<std::string>& result)
 {
     if (element) {
-        for (auto&& it : element.select_nodes(ConfigDefinition::mapConfigOption(nodeOption))) {
+        doExtend = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_LIST_EXTEND)->getXmlContent(element);
+        for (auto&& it : element.select_nodes(definition->mapConfigOption(nodeOption))) {
             const pugi::xml_node& child = it.node();
-            std::string attrValue = attrOption != ConfigVal::MAX ? child.attribute(ConfigDefinition::removeAttribute(attrOption).c_str()).as_string() : child.text().as_string();
+            std::string attrValue = attrOption != ConfigVal::MAX ? child.attribute(definition->removeAttribute(attrOption).c_str()).as_string() : child.text().as_string();
             if (itemCheck) {
                 if (!itemCheck(attrValue))
                     throw_std_runtime_error("Invalid array {} value {} empty '{}'", element.path(), xpath, attrValue);
@@ -60,7 +65,13 @@ void ConfigArraySetup::makeOption(const pugi::xml_node& root, const std::shared_
     setOption(config);
 }
 
-bool ConfigArraySetup::updateItem(const std::vector<std::size_t>& indexList, const std::string& optItem, const std::shared_ptr<Config>& config, const std::shared_ptr<ArrayOption>& value, const std::string& optValue, const std::string& status) const
+bool ConfigArraySetup::updateItem(
+    const std::vector<std::size_t>& indexList,
+    const std::string& optItem,
+    const std::shared_ptr<Config>& config,
+    const std::shared_ptr<ArrayOption>& value,
+    const std::string& optValue,
+    const std::string& status) const
 {
     auto index = getItemPath(indexList, {});
     if (optItem == index || !status.empty()) {
@@ -82,7 +93,27 @@ bool ConfigArraySetup::updateItem(const std::vector<std::size_t>& indexList, con
     return false;
 }
 
-bool ConfigArraySetup::updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments)
+bool ConfigArraySetup::createNodeFromDefaults(const std::shared_ptr<pugi::xml_node>& result) const
+{
+    if (defaultEntries.empty())
+        return false;
+    auto section = definition->mapConfigOption(nodeOption);
+    auto attrAttr = definition->removeAttribute(attrOption);
+    for (auto&& val : defaultEntries) {
+        auto entry = result->append_child(section);
+        if (attrOption != ConfigVal::MAX)
+            entry.append_attribute(attrAttr.c_str()) = val.c_str();
+        else
+            entry.append_child(pugi::node_pcdata).set_value(val.c_str());
+    }
+    return true;
+}
+
+bool ConfigArraySetup::updateDetail(
+    const std::string& optItem,
+    std::string& optValue,
+    const std::shared_ptr<Config>& config,
+    const std::map<std::string, std::string>* arguments)
 {
     if (startswith(optItem, xpath) && optionValue) {
         auto value = std::dynamic_pointer_cast<ArrayOption>(optionValue);
@@ -123,24 +154,24 @@ std::string ConfigArraySetup::getItemPath(const std::vector<std::size_t>& indexL
 {
     if (attrOption != ConfigVal::MAX) {
         if (indexList.size() == 0)
-            return fmt::format("{}/{}[_]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), ConfigDefinition::ensureAttribute(attrOption));
-        return fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), indexList[0], ConfigDefinition::ensureAttribute(attrOption));
+            return fmt::format("{}/{}[_]/{}", xpath, definition->mapConfigOption(nodeOption), definition->ensureAttribute(attrOption));
+        return fmt::format("{}/{}[{}]/{}", xpath, definition->mapConfigOption(nodeOption), indexList[0], definition->ensureAttribute(attrOption));
     }
     if (indexList.size() == 0)
-        return fmt::format("{}/{}[_]", xpath, ConfigDefinition::mapConfigOption(nodeOption));
-    return fmt::format("{}/{}[{}]", xpath, ConfigDefinition::mapConfigOption(nodeOption), indexList[0]);
+        return fmt::format("{}/{}[_]", xpath, definition->mapConfigOption(nodeOption));
+    return fmt::format("{}/{}[{}]", xpath, definition->mapConfigOption(nodeOption), indexList[0]);
 }
 
 std::string ConfigArraySetup::getItemPathRoot(bool prefix) const
 {
-    return fmt::format("{}/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption));
+    return fmt::format("{}/{}", xpath, definition->mapConfigOption(nodeOption));
 }
 
 std::vector<std::string> ConfigArraySetup::getXmlContent(const pugi::xml_node& optValue)
 {
     std::vector<std::string> result;
     if (initArray) {
-        if (!initArray(optValue, result, ConfigDefinition::mapConfigOption(nodeOption))) {
+        if (!initArray(optValue, result, definition->mapConfigOption(nodeOption))) {
             throw_std_runtime_error("Invalid {} array value '{}'", xpath, optValue.name());
         }
     } else {
@@ -152,6 +183,9 @@ std::vector<std::string> ConfigArraySetup::getXmlContent(const pugi::xml_node& o
         log_debug("{} assigning {} default values", xpath, defaultEntries.size());
         useDefault = true;
         result = defaultEntries;
+    } else if (doExtend) {
+        log_debug("{} extending by {} default values", xpath, defaultEntries.size());
+        result.insert(result.end(), defaultEntries.begin(), defaultEntries.end());
     }
     if (notEmpty && result.empty()) {
         throw_std_runtime_error("Invalid array {} empty '{}'", xpath, optValue.name());
