@@ -3,7 +3,8 @@
     Gerbera - https://gerbera.io/
 
     config_setup_dictionary.cc - this file is part of Gerbera.
-    Copyright (C) 2020-2024 Gerbera Contributors
+
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -28,17 +29,21 @@
 #include "config/config_definition.h"
 #include "config/config_options.h"
 #include "config/config_val.h"
+#include "config_setup_bool.h"
 #include "util/logger.h"
 
 #include <numeric>
 
 /// \brief Creates a dictionary from an XML nodeset.
-bool ConfigDictionarySetup::createOptionFromNode(const pugi::xml_node& element, std::map<std::string, std::string>& result) const
+bool ConfigDictionarySetup::createOptionFromNode(
+    const pugi::xml_node& element,
+    std::map<std::string, std::string>& result)
 {
     if (element) {
-        const auto dictNodes = element.select_nodes(ConfigDefinition::mapConfigOption(nodeOption));
-        auto keyAttr = ConfigDefinition::removeAttribute(keyOption);
-        auto valAttr = ConfigDefinition::removeAttribute(valOption);
+        doExtend = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_LIST_EXTEND)->getXmlContent(element);
+        const auto dictNodes = element.select_nodes(definition->mapConfigOption(nodeOption));
+        auto keyAttr = definition->removeAttribute(keyOption);
+        auto valAttr = definition->removeAttribute(valOption);
 
         for (auto&& it : dictNodes) {
             const pugi::xml_node child = it.node();
@@ -72,7 +77,30 @@ void ConfigDictionarySetup::makeOption(const pugi::xml_node& root, const std::sh
     setOption(config);
 }
 
-bool ConfigDictionarySetup::updateItem(const std::vector<std::size_t>& indexList, const std::string& optItem, const std::shared_ptr<Config>& config, const std::shared_ptr<DictionaryOption>& value, const std::string& optKey, const std::string& optValue, const std::string& status) const
+bool ConfigDictionarySetup::createNodeFromDefaults(const std::shared_ptr<pugi::xml_node>& result) const
+{
+    if (defaultEntries.empty())
+        return false;
+
+    auto section = definition->mapConfigOption(nodeOption);
+    auto keyAttr = definition->removeAttribute(keyOption);
+    auto valAttr = definition->removeAttribute(valOption);
+    for (auto&& [key, val] : defaultEntries) {
+        auto entry = result->append_child(section);
+        entry.append_attribute(keyAttr.c_str()) = key.c_str();
+        entry.append_attribute(valAttr.c_str()) = val.c_str();
+    }
+    return true;
+}
+
+bool ConfigDictionarySetup::updateItem(
+    const std::vector<std::size_t>& indexList,
+    const std::string& optItem,
+    const std::shared_ptr<Config>& config,
+    const std::shared_ptr<DictionaryOption>& value,
+    const std::string& optKey,
+    const std::string& optValue,
+    const std::string& status) const
 {
     auto i = indexList.at(0);
     auto keyIndex = getItemPath(indexList, { keyOption });
@@ -105,7 +133,10 @@ bool ConfigDictionarySetup::updateItem(const std::vector<std::size_t>& indexList
     return false;
 }
 
-bool ConfigDictionarySetup::updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments)
+bool ConfigDictionarySetup::updateDetail(const std::string& optItem,
+    std::string& optValue,
+    const std::shared_ptr<Config>& config,
+    const std::map<std::string, std::string>* arguments)
 {
     if (startswith(optItem, xpath) && optionValue) {
         auto value = std::dynamic_pointer_cast<DictionaryOption>(optionValue);
@@ -150,26 +181,26 @@ bool ConfigDictionarySetup::updateDetail(const std::string& optItem, std::string
 
 std::string ConfigDictionarySetup::getItemPath(const std::vector<std::size_t>& indexList, const std::vector<ConfigVal>& propOptions, const std::string& propText) const
 {
-    auto opt = propOptions.size() > 0 ? ConfigDefinition::ensureAttribute(propOptions[0]) : "";
+    auto opt = propOptions.size() > 0 ? definition->ensureAttribute(propOptions[0]) : "";
     if (indexList.size() == 0) {
-        return fmt::format("{}/{}[_]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), opt);
+        return fmt::format("{}/{}[_]/{}", xpath, definition->mapConfigOption(nodeOption), opt);
     }
 
-    return fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), indexList[0], opt);
+    return fmt::format("{}/{}[{}]/{}", xpath, definition->mapConfigOption(nodeOption), indexList[0], opt);
 }
 
 std::string ConfigDictionarySetup::getItemPathRoot(bool prefix) const
 {
     if (prefix)
         return xpath;
-    return fmt::format("{}/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption));
+    return fmt::format("{}/{}", xpath, definition->mapConfigOption(nodeOption));
 }
 
 std::string ConfigDictionarySetup::getUniquePath() const
 {
     if (!xpath)
-        return fmt::format("{}", ConfigDefinition::mapConfigOption(nodeOption));
-    return fmt::format("{}/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption));
+        return fmt::format("{}", definition->mapConfigOption(nodeOption));
+    return fmt::format("{}/{}", xpath, definition->mapConfigOption(nodeOption));
 }
 
 std::map<std::string, std::string> ConfigDictionarySetup::getXmlContent(const pugi::xml_node& optValue)
@@ -188,6 +219,9 @@ std::map<std::string, std::string> ConfigDictionarySetup::getXmlContent(const pu
         log_debug("{} assigning {} default values", xpath, defaultEntries.size());
         useDefault = true;
         result = defaultEntries;
+    } else if (doExtend) {
+        log_debug("{} extending by {} default values", xpath, defaultEntries.size());
+        result.merge(defaultEntries);
     }
     if (notEmpty && result.empty()) {
         throw_std_runtime_error("Invalid dictionary {} empty '{}'", xpath, optValue.name());
