@@ -3,7 +3,8 @@
     Gerbera - https://gerbera.io/
 
     config_setup_vector.cc - this file is part of Gerbera.
-    Copyright (C) 2022-2024 Gerbera Contributors
+
+    Copyright (C) 2022-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -28,19 +29,23 @@
 #include "config/config_definition.h"
 #include "config/config_options.h"
 #include "config/config_val.h"
+#include "config_setup_bool.h"
 #include "util/logger.h"
 
 #include <numeric>
 
 /// \brief Creates a vector from an XML nodeset.
-bool ConfigVectorSetup::createOptionFromNode(const pugi::xml_node& element, std::vector<std::vector<std::pair<std::string, std::string>>>& result) const
+bool ConfigVectorSetup::createOptionFromNode(
+    const pugi::xml_node& element,
+    std::vector<std::vector<std::pair<std::string, std::string>>>& result)
 {
     if (element) {
-        const auto dictNodes = element.select_nodes(ConfigDefinition::mapConfigOption(nodeOption));
+        doExtend = definition->findConfigSetup<ConfigBoolSetup>(ConfigVal::A_LIST_EXTEND)->getXmlContent(element);
+        const auto dictNodes = element.select_nodes(definition->mapConfigOption(nodeOption));
         std::vector<std::string> attrList;
         attrList.reserve(optionList.size());
         for (auto& opt : optionList) {
-            attrList.push_back(ConfigDefinition::removeAttribute(opt));
+            attrList.push_back(definition->removeAttribute(opt));
         }
 
         for (auto&& it : dictNodes) {
@@ -81,7 +86,26 @@ void ConfigVectorSetup::makeOption(const pugi::xml_node& root, const std::shared
     setOption(config);
 }
 
-bool ConfigVectorSetup::updateItem(const std::vector<std::size_t>& indexList, const std::string& optItem, const std::shared_ptr<Config>& config, const std::shared_ptr<VectorOption>& value, const std::string& optValue, const std::string& status) const
+bool ConfigVectorSetup::createNodeFromDefaults(const std::shared_ptr<pugi::xml_node>& result) const
+{
+    if (defaultEntries.empty())
+        return false;
+    auto section = definition->mapConfigOption(nodeOption);
+    for (auto&& defEntry : defaultEntries) {
+        auto entry = result->append_child(section);
+        for (auto&& [key, val] : defEntry) {
+            entry.append_attribute(key.c_str()) = val.c_str();
+        }
+    }
+    return true;
+}
+
+bool ConfigVectorSetup::updateItem(const std::vector<std::size_t>& indexList,
+    const std::string& optItem,
+    const std::shared_ptr<Config>& config,
+    const std::shared_ptr<VectorOption>& value,
+    const std::string& optValue,
+    const std::string& status) const
 {
     auto current = config->getVectorOption(option);
     auto i = indexList.at(0);
@@ -107,7 +131,10 @@ bool ConfigVectorSetup::updateItem(const std::vector<std::size_t>& indexList, co
     return false;
 }
 
-bool ConfigVectorSetup::updateDetail(const std::string& optItem, std::string& optValue, const std::shared_ptr<Config>& config, const std::map<std::string, std::string>* arguments)
+bool ConfigVectorSetup::updateDetail(const std::string& optItem,
+    std::string& optValue,
+    const std::shared_ptr<Config>& config,
+    const std::map<std::string, std::string>* arguments)
 {
     if (startswith(optItem, xpath) && optionValue) {
         auto value = std::dynamic_pointer_cast<VectorOption>(optionValue);
@@ -151,21 +178,21 @@ std::string ConfigVectorSetup::getItemPath(const std::vector<std::size_t>& index
 {
     if (!propText.empty()) {
         if (indexList.size() == 0)
-            return fmt::format("{}/{}[_]/{}{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), ConfigDefinition::ATTRIBUTE, propText);
-        return fmt::format("{}/{}[{}]/{}{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), indexList.at(0), ConfigDefinition::ATTRIBUTE, propText);
+            return fmt::format("{}/{}[_]/{}{}", xpath, definition->mapConfigOption(nodeOption), ConfigDefinition::ATTRIBUTE, propText);
+        return fmt::format("{}/{}[{}]/{}{}", xpath, definition->mapConfigOption(nodeOption), indexList.at(0), ConfigDefinition::ATTRIBUTE, propText);
     }
-    auto opt = ConfigDefinition::ensureAttribute(propOptions.size() > 0 ? propOptions.at(0) : ConfigVal::MAX);
+    auto opt = definition->ensureAttribute(propOptions.size() > 0 ? propOptions.at(0) : ConfigVal::MAX);
     if (indexList.size() == 0)
-        return fmt::format("{}/{}[_]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), opt);
+        return fmt::format("{}/{}[_]/{}", xpath, definition->mapConfigOption(nodeOption), opt);
 
-    return fmt::format("{}/{}[{}]/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption), indexList.at(0), opt);
+    return fmt::format("{}/{}[{}]/{}", xpath, definition->mapConfigOption(nodeOption), indexList.at(0), opt);
 }
 
 std::string ConfigVectorSetup::getItemPathRoot(bool prefix) const
 {
     if (prefix)
         return fmt::format("{}", xpath);
-    return fmt::format("{}/{}", xpath, ConfigDefinition::mapConfigOption(nodeOption));
+    return fmt::format("{}/{}", xpath, definition->mapConfigOption(nodeOption));
 }
 
 std::vector<std::vector<std::pair<std::string, std::string>>> ConfigVectorSetup::getXmlContent(const pugi::xml_node& optValue)
@@ -178,6 +205,9 @@ std::vector<std::vector<std::pair<std::string, std::string>>> ConfigVectorSetup:
         log_debug("{} assigning {} default values", xpath, defaultEntries.size());
         useDefault = true;
         result = defaultEntries;
+    } else if (doExtend) {
+        log_debug("{} extending by {} default values", xpath, defaultEntries.size());
+        result.insert(result.end(), defaultEntries.begin(), defaultEntries.end());
     }
     if (notEmpty && result.empty()) {
         throw_std_runtime_error("Invalid vector {} empty '{}'", xpath, optValue.name());
