@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -130,55 +130,61 @@ void ContentManager::run()
         throw_std_runtime_error("Could not start ContentTaskThread thread");
     }
 
+    log_debug("updateAutoscanList {} ui", AutoscanScanMode::Timed);
     autoscanList = database->getAutoscanList(AutoscanScanMode::Timed);
-    for (auto& dir : config->getAutoscanListOption(ConfigVal::IMPORT_AUTOSCAN_TIMED_LIST)) {
-        fs::path path = dir->getLocation();
-        if (fs::is_directory(path)) {
-            dir->setObjectID(ensurePathExistence(path));
-        }
-        try {
-            autoscanList->add(dir);
-        } catch (const std::runtime_error& e) {
-            // Work around existing config sourced autoscans that were stored to the DB for reasons
-            log_warning(e.what());
-            continue;
-        }
-    }
+    auto dbScans = std::vector<AutoscanScanMode> { AutoscanScanMode::Manual };
+    auto cfScans = std::vector<ConfigVal> { ConfigVal::IMPORT_AUTOSCAN_TIMED_LIST, ConfigVal::IMPORT_AUTOSCAN_MANUAL_LIST };
 
 #ifdef HAVE_INOTIFY
     inotify = std::make_unique<AutoscanInotify>(self);
-
     if (config->getBoolOption(ConfigVal::IMPORT_AUTOSCAN_USE_INOTIFY)) {
-        auto db = database->getAutoscanList(AutoscanScanMode::INotify);
-        for (std::size_t i = 0; i < db->size(); i++) {
-            auto dir = db->get(i);
-            auto path = dir->getLocation();
-            if (fs::is_directory(path)) {
-                dir->setObjectID(ensurePathExistence(path));
-            }
-            try {
-                autoscanList->add(dir);
-            } catch (const std::runtime_error& e) {
-                // Work around existing config sourced autoscans that were stored to the DB for reasons
-                log_warning(e.what());
-                continue;
+        dbScans.push_back(AutoscanScanMode::INotify);
+        cfScans.push_back(ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST);
+    }
+#endif
+    for (auto dbMode : dbScans) {
+        log_debug("updateAutoscanList {} ui", dbMode);
+        try {
+            auto db = database->getAutoscanList(dbMode);
+            for (std::size_t i = 0; i < db->size(); i++) {
+                auto dir = db->get(i);
+                auto path = dir->getLocation();
+                if (fs::is_directory(path)) {
+                    dir->setObjectID(ensurePathExistence(path));
+                }
+                try {
+                    dir->setInvalid();
+                    autoscanList->add(dir);
+                } catch (const std::logic_error& e) {
+                    log_warning(e.what());
+                } catch (const std::runtime_error& e) {
+                    // Work around existing config sourced autoscans that were stored to the DB for reasons
+                    log_warning(e.what());
+                }
             }
+        } catch (const std::logic_error& e) {
+            log_warning(e.what());
+        } catch (const std::runtime_error& e) {
+            log_warning(e.what());
         }
-        for (const auto& dir : config->getAutoscanListOption(ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST)) {
+    }
+    for (auto cfMode : cfScans) {
+        log_debug("updateAutoscanList {} config", cfMode);
+        for (auto& dir : config->getAutoscanListOption(cfMode)) {
             fs::path path = dir->getLocation();
             if (fs::is_directory(path)) {
                 dir->setObjectID(ensurePathExistence(path));
             }
             try {
                 autoscanList->add(dir);
+            } catch (const std::logic_error& e) {
+                log_warning(e.what());
             } catch (const std::runtime_error& e) {
-                // Work around existing config sourced autoscans that were stored to the DB for reasons
                 log_warning(e.what());
-                continue;
             }
         }
     }
-
+#ifdef HAVE_INOTIFY
     // Start INotify thread
     inotify->run();
 #endif
@@ -358,7 +364,7 @@ void ContentManager::addVirtualItem(const std::shared_ptr<CdsObject>& obj, bool
     if (ec || !isRegularFile(dirEnt, ec))
         throw_std_runtime_error("Not a file: {} - {}", path.c_str(), ec.message());
 
-    auto pcdir = database->findObjectByPath(path);
+    auto pcdir = database->findObjectByPath(path, UNUSED_CLIENT_GROUP);
     if (!pcdir) {
         auto adir = findAutoscanDirectory(path);
         pcdir = createObjectFromFile(adir, dirEnt, true, allowFifo);
@@ -385,7 +391,7 @@ std::shared_ptr<CdsObject> ContentManager::createSingleItem(
     const std::shared_ptr<AutoscanDirectory>& adir,
     const std::shared_ptr<CMAddFileTask>& task)
 {
-    auto obj = checkDatabase ? database->findObjectByPath(dirEnt.path()) : nullptr;
+    auto obj = checkDatabase ? database->findObjectByPath(dirEnt.path(), DEFAULT_CLIENT_GROUP) : nullptr;
     bool isNew = false;
 
     if (!obj) {
@@ -416,7 +422,11 @@ void ContentManager::parseMetafile(const std::shared_ptr<CdsObject>& obj, const
     importService->parseMetafile(obj, path);
 }
 
-std::shared_ptr<CdsObject> ContentManager::_addFile(const fs::directory_entry& dirEnt, fs::path rootPath, AutoScanSetting& asSetting, const std::shared_ptr<CMAddFileTask>& task)
+std::shared_ptr<CdsObject> ContentManager::_addFile(
+    const fs::directory_entry& dirEnt,
+    fs::path rootPath,
+    AutoScanSetting& asSetting,
+    const std::shared_ptr<CMAddFileTask>& task)
 {
     if (!asSetting.hidden && dirEnt.path().is_relative()) {
         return nullptr;
@@ -440,7 +450,7 @@ std::shared_ptr<CdsObject> ContentManager::_addFile(const fs::directory_entry& d
     }
     // checkDatabase, don't process existing
     std::shared_ptr<CdsContainer> parent;
-    auto parentObject = database->findObjectByPath(dirEnt.path().parent_path(), DbFileType::Directory);
+    auto parentObject = database->findObjectByPath(dirEnt.path().parent_path(), UNUSED_CLIENT_GROUP, DbFileType::Directory);
     if (parentObject && parentObject->isContainer())
         parent = std::dynamic_pointer_cast<CdsContainer>(parentObject);
     auto obj = createSingleItem(dirEnt, parent, rootPath, asSetting.followSymlinks, true, false, false, asSetting.adir, task);
@@ -464,7 +474,7 @@ bool ContentManager::updateAttachedResources(const std::shared_ptr<AutoscanDirec
         return false;
 
     bool parentRemoved = false;
-    auto parentObject = database->findObjectByPath(parentPath, DbFileType::Auto);
+    auto parentObject = database->findObjectByPath(parentPath, UNUSED_CLIENT_GROUP, DbFileType::Auto);
     if (parentObject) {
         // as there is no proper way to force refresh of unchanged files, delete whole dir and rescan it
         _removeObject(adir, parentObject, parentPath, false, all);
@@ -492,7 +502,12 @@ bool ContentManager::updateAttachedResources(const std::shared_ptr<AutoscanDirec
     return parentRemoved;
 }
 
-std::vector<int> ContentManager::_removeObject(const std::shared_ptr<AutoscanDirectory>& adir, const std::shared_ptr<CdsObject>& obj, const fs::path& path, bool rescanResource, bool all)
+std::vector<int> ContentManager::_removeObject(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& path,
+    bool rescanResource,
+    bool all)
 {
     if (!obj || obj->getID() == INVALID_OBJECT_ID)
         return {};
@@ -688,7 +703,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
             }
 
             if (!asSetting.followSymlinks && dirEnt.is_symlink()) {
-                auto object = database->findObjectByPath(newPath);
+                auto object = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (object) {
                     list.erase(object->getID());
                     removeObject(adir, object, newPath, false);
@@ -704,7 +719,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
             auto lwt = toSeconds(dirEnt.last_write_time(ec));
 
             if (isRegularFile(dirEnt, ec)) {
-                auto newObject = database->findObjectByPath(newPath);
+                auto newObject = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (newObject) {
                     list.erase(newObject->getID());
 
@@ -741,7 +756,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
                     }
                 }
             } else if (dirEnt.is_directory(ec) && asSetting.recursive) {
-                auto newObject = database->findObjectByPath(newPath);
+                auto newObject = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (lastModifiedNewMax < lwt && lwt <= currentTme)
                     lastModifiedNewMax = lwt;
                 if (newObject) {
@@ -777,7 +792,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
                 }
             }
             if (!parentContainer && !location.empty()) {
-                std::shared_ptr<CdsObject> obj = database->findObjectByPath(location, DbFileType::Directory);
+                std::shared_ptr<CdsObject> obj = database->findObjectByPath(location, UNUSED_CLIENT_GROUP, DbFileType::Directory);
                 if (!obj || !obj->isContainer()) {
                     log_error("Updated parent {} is not available or no container", location.string());
                 } else {
@@ -954,7 +969,7 @@ void ContentManager::addRecursive(
 }
 
 template <typename T>
-void ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
     std::string upnpClass = getValueOrDefault(parameters, "class");
@@ -965,10 +980,11 @@ void ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::
     std::string flags = getValueOrDefault(parameters, "flags");
 
     log_error("updateCdsObject: CdsObject {} not updated", title);
+    return nullptr;
 }
 
 template <>
-void ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
     std::string upnpClass = getValueOrDefault(parameters, "class");
@@ -999,10 +1015,11 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item,
         update_manager->containerChanged(item->getParentID());
         session_manager->containerChangedUI(item->getParentID());
     }
+    return clone;
 }
 
 template <>
-void ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
     std::string upnpClass = getValueOrDefault(parameters, "class");
@@ -1070,20 +1087,21 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const
         log_debug("updateObject: calling containerChanged on item {}", item->getTitle());
         update_manager->containerChanged(item->getParentID());
     }
+    return clone;
 }
 
-void ContentManager::updateObject(int objectID, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateObject(int objectID, const std::map<std::string, std::string>& parameters)
 {
     auto obj = database->loadObject(objectID);
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (item) {
-        updateCdsObject(item, parameters);
+        return updateCdsObject(item, parameters);
     } else {
         auto cont = std::dynamic_pointer_cast<CdsContainer>(obj);
         if (cont) {
-            updateCdsObject(cont, parameters);
+            return updateCdsObject(cont, parameters);
         } else {
-            updateCdsObject(obj, parameters);
+            return updateCdsObject(obj, parameters);
         }
     }
 }
@@ -1131,7 +1149,7 @@ std::shared_ptr<CdsContainer> ContentManager::addContainer(int parentID, const s
         cVec.push_back(std::make_shared<CdsContainer>(segment.string(), upnpClass));
     }
     addContainerTree(cVec, nullptr);
-    return importService->getContainer(cPath);
+    return importService->getContainer(fmt::format("/{}", cPath.string()));
 }
 
 std::pair<int, bool> ContentManager::addContainerTree(const std::vector<std::shared_ptr<CdsObject>>& chain, const std::shared_ptr<CdsObject>& refItem)
@@ -1293,7 +1311,11 @@ void ContentManager::addTask(std::shared_ptr<GenericTask> task, bool lowPriority
     threadRunner->notify();
 }
 
-std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& dirEnt, AutoScanSetting& asSetting, bool lowPriority, bool cancellable)
+std::shared_ptr<CdsObject> ContentManager::addFile(
+    const fs::directory_entry& dirEnt,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    bool cancellable)
 {
     fs::path rootpath;
     if (dirEnt.is_directory())
@@ -1301,13 +1323,23 @@ std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& di
     return addFileInternal(dirEnt, rootpath, asSetting, lowPriority, 0, cancellable);
 }
 
-std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& dirEnt, const fs::path& rootpath, AutoScanSetting& asSetting, bool lowPriority, bool cancellable)
+std::shared_ptr<CdsObject> ContentManager::addFile(
+    const fs::directory_entry& dirEnt,
+    const fs::path& rootpath,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    bool cancellable)
 {
     return addFileInternal(dirEnt, rootpath, asSetting, lowPriority, 0, cancellable);
 }
 
 std::shared_ptr<CdsObject> ContentManager::addFileInternal(
-    const fs::directory_entry& dirEnt, const fs::path& rootpath, AutoScanSetting& asSetting, bool lowPriority, unsigned int parentTaskID, bool cancellable)
+    const fs::directory_entry& dirEnt,
+    const fs::path& rootpath,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    unsigned int parentTaskID,
+    bool cancellable)
 {
     if (asSetting.async) {
         auto self = shared_from_this();
@@ -1407,7 +1439,13 @@ void ContentManager::invalidateTask(unsigned int taskID, TaskOwner taskOwner)
 #endif
 }
 
-std::vector<int> ContentManager::removeObject(const std::shared_ptr<AutoscanDirectory>& adir, const std::shared_ptr<CdsObject>& obj, const fs::path& path, bool rescanResource, bool async, bool all)
+std::vector<int> ContentManager::removeObject(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& path,
+    bool rescanResource,
+    bool async,
+    bool all)
 {
     if (async) {
         auto self = shared_from_this();
@@ -1459,7 +1497,11 @@ void ContentManager::cleanupTasks(const fs::path& path)
     }
 }
 
-void ContentManager::rescanDirectory(const std::shared_ptr<AutoscanDirectory>& adir, int objectId, fs::path descPath, bool cancellable)
+void ContentManager::rescanDirectory(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    int objectId,
+    fs::path descPath,
+    bool cancellable)
 {
     auto self = shared_from_this();
     auto task = std::make_shared<CMRescanDirectoryTask>(self, adir, objectId, cancellable);
@@ -1486,7 +1528,7 @@ std::shared_ptr<AutoscanDirectory> ContentManager::getAutoscanDirectory(int obje
 
 std::shared_ptr<AutoscanDirectory> ContentManager::getAutoscanDirectory(const fs::path& location) const
 {
-    return autoscanList->get(location);
+    return autoscanList->getKey(location);
 }
 
 std::vector<std::shared_ptr<AutoscanDirectory>> ContentManager::getAutoscanDirectories() const
@@ -1500,7 +1542,7 @@ void ContentManager::removeAutoscanDirectory(const std::shared_ptr<AutoscanDirec
         throw_std_runtime_error("can not remove autoscan directory - was not an autoscan");
 
     adir->setTaskCount(-1);
-    autoscanList->remove(adir->getScanID());
+    autoscanList->remove(adir);
     database->removeAutoscanDirectory(adir);
     session_manager->containerChangedUI(adir->getObjectID());
 
@@ -1585,17 +1627,22 @@ void ContentManager::setAutoscanDirectory(const std::shared_ptr<AutoscanDirector
     copy->setRecursive(dir->getRecursive());
     copy->setInterval(dir->getInterval());
     copy->setScanContent(dir->getScanContent());
+    copy->setDirTypes(dir->hasDirTypes());
 
-    autoscanList->remove(copy->getScanID());
+    if (dir->isValid())
+        autoscanList->remove(dir); // remove old version of dir
 
-    copy->setScanMode(dir->getScanMode());
     scanDir(copy, original->getScanMode() != copy->getScanMode());
+    copy->setScanMode(dir->getScanMode());
     database->updateAutoscanDirectory(copy);
 }
 
 void ContentManager::scanDir(const std::shared_ptr<AutoscanDirectory>& dir, bool updateUI)
 {
-    autoscanList->add(dir);
+    if (dir->isValid())
+        autoscanList->add(dir, dir->getScanID());
+    else
+        autoscanList->add(dir);
 
     if (dir->getScanMode() == AutoscanScanMode::Timed)
         timerNotify(dir->getTimerParameter());
@@ -1622,7 +1669,7 @@ void ContentManager::triggerPlayHook(const std::string& group, const std::shared
     playStatus->setLastPlayed();
     database->savePlayStatus(playStatus);
 
-    bool suppress = config->getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_SUPPRESS_CDS_UPDATES);
+    bool suppress = config->getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_ENABLED) && config->getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_SUPPRESS_CDS_UPDATES);
     log_debug("Marking object {} as played", obj->getTitle());
     if (!suppress)
         updateObject(obj, true);
