@@ -4,7 +4,7 @@
 
     import_service.cc - this file is part of Gerbera.
 
-    Copyright (C) 2023-2024 Gerbera Contributors
+    Copyright (C) 2023-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,6 +32,7 @@
 #include "config/config_val.h"
 #include "config/result/autoscan.h"
 #include "content_manager.h"
+#include "context.h"
 #include "database/database.h"
 #include "exceptions.h"
 #include "layout/builtin_layout.h"
@@ -55,24 +56,28 @@
 
 bool UpnpMap::checkValue(const std::string& op, const std::string& expect, const std::string& actual) const
 {
-    if (op == "=" && actual.find(expect) != std::string::npos)
-        return true;
-    if (op == "!=" && actual.find(expect) == std::string::npos)
-        return true;
-    if (op == "<" && actual < expect)
-        return true;
-    return (op == ">" && actual > expect);
+    if (op == "=" || op == "==")
+        return actual.find(expect) != std::string::npos;
+    if (op == "!=")
+        return actual.find(expect) == std::string::npos;
+    if (op == "<")
+        return actual < expect;
+    if (op == ">")
+        return actual > expect;
+    return false;
 }
 
 bool UpnpMap::checkValue(const std::string& op, int expect, int actual) const
 {
-    if (op == "=" && actual != expect)
-        return true;
-    if (op == "!=" && actual == expect)
-        return true;
-    if (op == "<" && actual < expect)
-        return true;
-    return (op == ">" && actual > expect);
+    if (op == "=" || op == "==")
+        return actual == expect;
+    if (op == "!=")
+        return actual != expect;
+    if (op == "<")
+        return actual < expect;
+    if (op == ">")
+        return actual > expect;
+    return false;
 }
 
 bool UpnpMap::isMatch(const std::shared_ptr<CdsItem>& item, const std::string& mt) const
@@ -96,7 +101,7 @@ bool UpnpMap::isMatch(const std::shared_ptr<CdsItem>& item, const std::string& m
 
 void UpnpMap::initMap(std::vector<UpnpMap>& target, const std::map<std::string, std::string>& source)
 {
-    auto re = std::regex("^([A-Za-z0-9_:]+)(<|>|=|!=)([A-Za-z0-9_]+)$");
+    auto re = std::regex("^([A-Za-z0-9_:]+)(<|>|=|==|!=)([A-Za-z0-9_]+)$");
     for (auto&& [key, cls] : source) {
         auto filterList = splitString(key, ';');
         auto filters = std::vector<std::tuple<std::string, std::string, std::string>>();
@@ -114,7 +119,7 @@ void UpnpMap::initMap(std::vector<UpnpMap>& target, const std::map<std::string,
                 filters.emplace_back(match[1], match[2], match[3]);
             }
         }
-        log_debug("UpnpMap: {} -> {}", mt, cls);
+        log_vdebug("UpnpMap: {} -> {}", mt, cls);
         target.emplace_back(mt, cls, filters);
     }
 }
@@ -168,7 +173,7 @@ ImportService::ImportService(std::shared_ptr<Context> context, std::shared_ptr<C
     configLayoutMapping = config->getDictionaryOption(ConfigVal::IMPORT_LAYOUT_MAPPING);
     containerImageParentCount = config->getIntOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_PARENTCOUNT);
     containerImageMinDepth = config->getIntOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_MINDEPTH);
-    virtualDirKeys = config->getArrayOption(ConfigVal::IMPORT_VIRTUAL_DIRECTORY_KEYS);
+    virtualDirKeys = config->getVectorOption(ConfigVal::IMPORT_VIRTUAL_DIRECTORY_KEYS);
     noMediaName = config->getOption(ConfigVal::IMPORT_NOMEDIA_FILE);
     UpnpMap::initMap(upnpMap, mimetypeUpnpclassMap);
 }
@@ -226,7 +231,7 @@ void ImportService::destroyLayout()
 #endif
 }
 
-std::string ImportService::mimeTypeToUpnpClass(const std::string& mimeType)
+std::string ImportService::mimeTypeToUpnpClass(const std::string& mimeType) const
 {
     auto it = std::find_if(upnpMap.begin(), upnpMap.end(), [=](auto&& um) { return startswith(mimeType, um.mimeType); });
     if (it != upnpMap.end())
@@ -242,7 +247,7 @@ std::string ImportService::mimeTypeToUpnpClass(const std::string& mimeType)
     if (parts.size() != 2)
         return {};
 
-    return getValueOrDefault(mimetypeUpnpclassMap, parts[0] + "/*");
+    return getValueOrDefault(mimetypeUpnpclassMap, parts.at(0) + "/*");
 }
 
 void ImportService::clearCache()
@@ -252,7 +257,11 @@ void ImportService::clearCache()
     containersWithFanArt.clear();
 }
 
-void ImportService::doImport(const fs::path& location, AutoScanSetting& settings, std::unordered_set<int>& currentContent, const std::shared_ptr<GenericTask>& task)
+void ImportService::doImport(
+    const fs::path& location,
+    AutoScanSetting& settings,
+    std::unordered_set<int>& currentContent,
+    const std::shared_ptr<GenericTask>& task)
 {
     log_debug("start {} root '{}' update {}", location.string(), rootPath.string(), !!settings.changedObject);
     if (activeScan.empty()) {
@@ -269,7 +278,7 @@ void ImportService::doImport(const fs::path& location, AutoScanSetting& settings
         auto rootDirEntry = fs::directory_entry(rootPath);
         cacheState(rootPath, rootDirEntry, ImportState::New, toSeconds(rootDirEntry.last_write_time(ec)));
     }
-    auto rootEntry = fs::directory_entry(location);
+    auto rootEntry = fs::directory_entry(location, ec);
     if (ec) {
         log_error("Failed to start {}, {}", location.c_str(), ec.message());
         return;
@@ -352,9 +361,15 @@ void ImportService::readDir(const fs::path& location, AutoScanSetting settings)
     log_debug("end {}", location.string());
 }
 
-void ImportService::cacheState(const fs::path& entryPath, const fs::directory_entry& dirEntry, ImportState state, std::chrono::seconds mtime, const std::shared_ptr<CdsObject>& cdsObject)
+void ImportService::cacheState(
+    const fs::path& entryPath,
+    const fs::directory_entry& dirEntry,
+    ImportState state,
+    std::chrono::seconds mtime,
+    const std::shared_ptr<CdsObject>& cdsObject)
 {
     auto cacheLock = CacheAutoLock(cacheMutex);
+    log_debug("cache '{}' , '{}' ({})", entryPath.string(), dirEntry.path().string(), state);
 
     if (entryPath.empty())
         return;
@@ -362,13 +377,13 @@ void ImportService::cacheState(const fs::path& entryPath, const fs::directory_en
     if (contentStateCache.find(entryPath) == contentStateCache.end()) {
         contentStateCache[entryPath] = std::make_shared<ContentState>(dirEntry, state, mtime, cdsObject);
     } else {
-        state = contentStateCache[entryPath]->getState() < state ? state : contentStateCache[entryPath]->getState();
+        state = contentStateCache.at(entryPath)->getState() < state ? state : contentStateCache[entryPath]->getState();
         if (!cdsObject)
-            contentStateCache[entryPath]->setState(state);
+            contentStateCache.at(entryPath)->setState(state);
         else
-            contentStateCache[entryPath]->setObject(state, cdsObject);
+            contentStateCache.at(entryPath)->setObject(state, cdsObject);
         if (mtime > std::chrono::seconds::zero())
-            contentStateCache[entryPath]->setMTime(mtime);
+            contentStateCache.at(entryPath)->setMTime(mtime);
     }
 }
 
@@ -419,10 +434,13 @@ void ImportService::removeHidden(const AutoScanSetting& settings)
 bool ImportService::isHiddenFile(const fs::path& entryPath, bool isDirectory, const fs::directory_entry& dirEntry, const AutoScanSetting& settings)
 {
     auto&& name = entryPath.filename().string();
-    if ((name[0] == '.' && !settings.hidden)
+    if (name.empty())
+        return true;
+    if ((name.at(0) == '.' && !settings.hidden)
         || (!settings.followSymlinks && dirEntry.is_symlink())
         || config->getConfigFilename() == entryPath) {
         cacheState(entryPath, dirEntry, ImportState::ToDelete);
+        log_vdebug("hidden {}", entryPath.string());
         return true;
     }
     if (!noMediaName.empty()) {
@@ -459,14 +477,14 @@ void ImportService::createContainers(int parentContainerId, AutoScanSetting& set
                 for (auto&& [childPath, childEntry] : contentStateCache) {
                     // find direct folder entry with old name and rely on iteration to rename hierarchy
                     if (childEntry && childPath.parent_path() == contPath) {
-                        auto childObj = database->findObjectByPath(oldLocation / childPath.filename(), DbFileType::Any);
+                        auto childObj = database->findObjectByPath(oldLocation / childPath.filename(), UNUSED_CLIENT_GROUP, DbFileType::Any);
                         if (childObj)
                             childEntry->setObject(ImportState::New, childObj);
                     }
                 }
             }
             if (!cdsObj)
-                cdsObj = database->findObjectByPath(contPath, DbFileType::Directory);
+                cdsObj = database->findObjectByPath(contPath, UNUSED_CLIENT_GROUP, DbFileType::Directory);
 
             if (cdsObj) {
                 try {
@@ -493,13 +511,40 @@ void ImportService::createContainers(int parentContainerId, AutoScanSetting& set
                 }
             } else {
                 // Create container
-                stateEntry->setObject(ImportState::Created, createSingleContainer(parentContainerId, dirEntry, UPNP_CLASS_CONTAINER));
+                stateEntry->setObject(ImportState::Created, createSingleContainer(parentContainerId, dirEntry, UPNP_CLASS_CONTAINER_FOLDER));
             }
         }
     }
     log_debug("end {}", rootPath.string());
 }
 
+std::tuple<bool, std::string, std::string> ImportService::getMimeForFile(const fs::path& objectPath) const
+{
+    /* retrieve information about item and decide if it should be included */
+    auto [skip, mimetype] = mime->getMimeType(objectPath, MIMETYPE_DEFAULT);
+    if (mimetype.empty()) {
+        if (skip)
+            log_debug("Mime set empty for file {}", objectPath.c_str());
+        else
+            log_error("Mime not found for file {}", objectPath.c_str());
+        return { skip, "", "" };
+    }
+    log_debug("Mime '{}' for file {}", mimetype, objectPath.c_str());
+
+    std::string upnpClass = mimeTypeToUpnpClass(mimetype);
+    if (upnpClass.empty()) {
+        std::string contentType = getValueOrDefault(mimetypeContenttypeMap, mimetype);
+        if (contentType == CONTENT_TYPE_OGG) {
+            upnpClass = isTheora(objectPath)
+                ? UPNP_CLASS_VIDEO_ITEM
+                : UPNP_CLASS_MUSIC_TRACK;
+        }
+    }
+    log_debug("UpnpClass '{}' for file {}", upnpClass, objectPath.c_str());
+
+    return { skip, mimetype, upnpClass };
+}
+
 ///\brief create items for all discovered files
 void ImportService::createItems(AutoScanSetting& settings)
 {
@@ -519,7 +564,7 @@ void ImportService::createItems(AutoScanSetting& settings)
         if (cdsObj && cdsObj->isContainer()) {
             std::shared_ptr<CdsContainer> container = std::dynamic_pointer_cast<CdsContainer>(cdsObj);
             if (!contPath.empty()) {
-                contentStateCache[contPath]->setMTime(lastModifiedNewMax);
+                contentStateCache.at(contPath)->setMTime(lastModifiedNewMax);
                 if (autoscanDir) {
                     autoscanDir->setCurrentLMT(contPath, lastModifiedNewMax);
                 }
@@ -540,7 +585,7 @@ void ImportService::createItems(AutoScanSetting& settings)
         auto dirEntry = stateEntry->getDirEntry();
         if (isRegularFile(dirEntry, ec)) {
             // Start with cached item
-            auto contState = contentStateCache[itemPath.parent_path()];
+            auto contState = contentStateCache.at(itemPath.parent_path());
             if (contState)
                 parentContainer = std::dynamic_pointer_cast<CdsContainer>(contState->getObject());
             else
@@ -549,14 +594,26 @@ void ImportService::createItems(AutoScanSetting& settings)
             if (!cdsObj) {
                 // Search item in database
                 log_debug("Searching Item {} in database", itemPath.string());
-                cdsObj = database->findObjectByPath(itemPath, DbFileType::File);
+                cdsObj = database->findObjectByPath(itemPath, UNUSED_CLIENT_GROUP, DbFileType::File);
             }
             if (cdsObj && cdsObj->isItem()) {
                 auto isChanged = stateEntry->getMTime() != cdsObj->getMTime() || cdsObj->getLocation().string() != dirEntry.path().string();
+                if (autoscanDir && autoscanDir->getForceRescan())
+                    isChanged = isChanged || cdsObj->getClass().empty() || cdsObj->getClass() == UPNP_CLASS_ITEM;
                 if (isChanged) {
                     // Update changed item in database
                     log_debug("Updating Item {} in database {}", itemPath.string(), cdsObj->getID());
                     auto item = std::dynamic_pointer_cast<CdsItem>(cdsObj);
+                    if (item->getMimeType().empty() || item->getClass().empty() || item->getClass() == UPNP_CLASS_ITEM) {
+                        auto [skip, mimetype, upnpClass] = getMimeForFile(itemPath);
+                        if (!mimetype.empty()) {
+                            item->setMimeType(mimetype);
+                        }
+                        if (!upnpClass.empty()) {
+                            item->setClass(upnpClass);
+                        }
+                        log_debug("Updating Item properties {} in database: skip {} mimeType {}, upnpClass {}", skip, itemPath.string(), mimetype, upnpClass);
+                    }
                     item->clearMetaData();
                     item->clearAuxData();
                     item->clearResources();
@@ -620,29 +677,10 @@ void ImportService::createItems(AutoScanSetting& settings)
 std::pair<bool, std::shared_ptr<CdsObject>> ImportService::createSingleItem(const fs::directory_entry& dirEntry) // ToDo: Use StateEntry here
 {
     const auto& objectPath = dirEntry.path();
-
-    /* retrieve information about item and decide if it should be included */
-    auto [skip, mimetype] = mime->getMimeType(objectPath, MIMETYPE_DEFAULT);
-    if (mimetype.empty()) {
-        if (skip)
-            log_debug("Mime set empty for file {}", objectPath.c_str());
-        else
-            log_error("Mime not found for file {}", objectPath.c_str());
+    auto [skip, mimetype, upnpClass] = getMimeForFile(objectPath);
+    if (mimetype.empty() && upnpClass.empty()) {
         return { skip, nullptr };
     }
-    log_debug("Mime '{}' for file {}", mimetype, objectPath.c_str());
-
-    std::string upnpClass = mimeTypeToUpnpClass(mimetype);
-    if (upnpClass.empty()) {
-        std::string contentType = getValueOrDefault(mimetypeContenttypeMap, mimetype);
-        if (contentType == CONTENT_TYPE_OGG) {
-            upnpClass = isTheora(objectPath)
-                ? UPNP_CLASS_VIDEO_ITEM
-                : UPNP_CLASS_MUSIC_TRACK;
-        }
-    }
-    log_debug("UpnpClass '{}' for file {}", upnpClass, objectPath.c_str());
-
     auto item = std::make_shared<CdsItem>();
     item->setLocation(objectPath);
 
@@ -659,7 +697,7 @@ std::pair<bool, std::shared_ptr<CdsObject>> ImportService::createSingleItem(cons
     return { skip, item };
 }
 
-std::string ImportService::makeTitle(const fs::path& objectPath, const std::string upnpClass)
+std::string ImportService::makeTitle(const fs::path& objectPath, const std::string upnpClass) const
 {
     auto f2i = converterManager->f2i();
     std::string title = objectPath.filename().string();
@@ -668,7 +706,11 @@ std::string ImportService::makeTitle(const fs::path& objectPath, const std::stri
         if (title.length() > 1)
             std::replace(title.begin() + 1, title.end() - 1, '_', ' ');
     }
-    return f2i->convert(title);
+    auto [mval, err] = f2i->convert(title);
+    if (!err.empty()) {
+        log_warning("{}: {}", title, err);
+    }
+    return mval;
 }
 
 void ImportService::updateSingleItem(const fs::directory_entry& dirEntry, const std::shared_ptr<CdsItem>& item, const std::string& mimetype)
@@ -678,8 +720,12 @@ void ImportService::updateSingleItem(const fs::directory_entry& dirEntry, const
     item->setUTime(mTime);
     item->setSizeOnDisk(getFileSize(dirEntry));
 
-    metadataService->extractMetaData(item, dirEntry);
-    updateItemData(item, mimetype);
+    try {
+        metadataService->extractMetaData(item, dirEntry);
+        updateItemData(item, mimetype);
+    } catch (const std::runtime_error& ex) {
+        log_error("updateSingleItem '{}' failed: {}", dirEntry.path().string(), ex.what());
+    }
 }
 
 void ImportService::fillLayout(const std::shared_ptr<GenericTask>& task)
@@ -687,13 +733,16 @@ void ImportService::fillLayout(const std::shared_ptr<GenericTask>& task)
     for (auto&& [contPath, stateEntry] : contentStateCache) {
         if (!stateEntry || stateEntry->getState() != ImportState::Created)
             continue;
-        contentStateCache[contPath]->setObject(ImportState::Loaded, stateEntry->getObject());
+        contentStateCache.at(contPath)->setObject(ImportState::Loaded, stateEntry->getObject());
         fillSingleLayout(stateEntry, nullptr, stateEntry->getParentObject(), task);
     }
 }
 
 /// \param object used to make code compatible with legacy scan
-void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state, std::shared_ptr<CdsObject> object, const std::shared_ptr<CdsContainer>& parent, const std::shared_ptr<GenericTask>& task)
+void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
+    std::shared_ptr<CdsObject> object,
+    const std::shared_ptr<CdsContainer>& parent,
+    const std::shared_ptr<GenericTask>& task)
 {
     std::shared_ptr<CdsObject> cdsObject = state ? state->getObject() : std::move(object);
     log_debug("cds {}, layout {}, autoscanDir {}", !!cdsObject, !!layout, !!autoscanDir);
@@ -702,8 +751,20 @@ void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
         try {
             std::string mimetype = std::static_pointer_cast<CdsItem>(cdsObject)->getMimeType();
             std::string contentType = getValueOrDefault(mimetypeContenttypeMap, mimetype);
+            log_vdebug("mimetype {}, contentype {}, autoscanDir {}", mimetype, contentType, (!autoscanDir || autoscanDir->hasContent(cdsObject->getClass())));
 
-            if (!autoscanDir || autoscanDir->hasContent(cdsObject->getClass())) {
+            if (contentType == CONTENT_TYPE_PLAYLIST) {
+#ifdef HAVE_JS
+                try {
+                    if (playlistParserScript)
+                        playlistParserScript->processPlaylistObject(cdsObject, task, rootPath);
+                } catch (const std::runtime_error& e) {
+                    log_error("{}", e.what());
+                }
+#else
+                log_warning("Playlist {} will not be parsed: Gerbera was compiled without JS support!", cdsObject->getLocation().c_str());
+#endif // HAVE_JS
+            } else if (!autoscanDir || autoscanDir->hasContent(cdsObject->getClass())) {
                 // only lock mutex while processing item layout
                 LayoutAutoLock lock(layoutMutex);
                 // get ref'd objects with last mod time
@@ -713,22 +774,11 @@ void ImportService::fillSingleLayout(const std::shared_ptr<ContentState>& state,
                     containerTypeMap,
                     refObjects);
             } else {
-                log_debug("file ignored: {} autoscanDir={}, class={}, hasContent={}, mediaType={}", cdsObject->getLocation().string(), !!autoscanDir, cdsObject->getClass(), autoscanDir ? autoscanDir->hasContent(cdsObject->getClass()) : false, autoscanDir ? autoscanDir->getMediaType() : -2);
+                log_debug("File ignored: {} autoscanDir={}, class={}, hasContent={}, mediaType={}", cdsObject->getLocation().string(), !!autoscanDir, cdsObject->getClass(), autoscanDir ? autoscanDir->hasContent(cdsObject->getClass()) : false, autoscanDir ? autoscanDir->getMediaType() : -2);
             }
 
-#ifdef HAVE_JS
-            try {
-                if (playlistParserScript && contentType == CONTENT_TYPE_PLAYLIST)
-                    playlistParserScript->processPlaylistObject(cdsObject, task, rootPath);
-            } catch (const std::runtime_error& e) {
-                log_error("{}", e.what());
-            }
-#else
-            if (contentType == CONTENT_TYPE_PLAYLIST)
-                log_warning("Playlist {} will not be parsed: Gerbera was compiled without JS support!", cdsObject->getLocation().c_str());
-#endif // HAVE_JS
-        } catch (const std::runtime_error& e) {
-            log_error("{}", e.what());
+        } catch (const std::runtime_error& ex) {
+            log_error("{}", ex.what());
         }
     }
 }
@@ -759,7 +809,11 @@ void ImportService::updateItemData(const std::shared_ptr<CdsItem>& item, const s
     }
 }
 
-void ImportService::finishScan(const fs::path& location, const std::shared_ptr<CdsContainer>& parent, std::chrono::seconds lmt, const std::shared_ptr<CdsObject>& firstObject)
+void ImportService::finishScan(
+    const fs::path& location,
+    const std::shared_ptr<CdsContainer>& parent,
+    std::chrono::seconds lmt,
+    const std::shared_ptr<CdsObject>& firstObject)
 {
     if (autoscanDir) {
         autoscanDir->setCurrentLMT(location, lmt > std::chrono::seconds::zero() ? lmt : std::chrono::seconds(1));
@@ -771,12 +825,23 @@ void ImportService::finishScan(const fs::path& location, const std::shared_ptr<C
                 count = std::distance(objectLocation.begin(), objectLocation.end()) - std::distance(location.begin(), location.end());
             }
             database->updateObject(parent, nullptr);
-            assignFanArt(parent, firstObject, AutoscanMediaMode::Mixed, false, count);
+            assignFanArt(parent,
+                firstObject,
+                AutoscanMediaMode::Mixed,
+                false /* isDir */,
+                count,
+                false /* isNew */);
         }
     }
 }
 
-void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container, const std::shared_ptr<CdsObject>& refObj, AutoscanMediaMode mediaMode, bool isDir, int count)
+void ImportService::assignFanArt(
+    const std::shared_ptr<CdsContainer>& container,
+    const std::shared_ptr<CdsObject>& refObj,
+    AutoscanMediaMode mediaMode,
+    bool isDir,
+    int count,
+    bool isNew)
 {
     if (!container)
         return;
@@ -794,7 +859,7 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
     }
 
     if (containersWithFanArt.find(container->getID()) != containersWithFanArt.end()) {
-        log_debug("Already assigned fanart {}", container->getID());
+        log_debug("Already {} assigned fanart {}", container->getLocation().string(), container->getID());
         if (doUpdate)
             database->updateObject(container, nullptr);
         return;
@@ -806,15 +871,17 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
         auto fanartObjId = stoiString(fanart->getAttribute(ResourceAttribute::FANART_OBJ_ID));
         try {
             if (fanartObjId > CDS_ID_ROOT) {
+                log_debug("Object {} assigned fanart {}", container->getLocation().string(), container->getID());
                 database->loadObject(fanartObjId);
             }
         } catch (const ObjectNotFoundException&) {
+            log_warning("Object fanart {} broken {}", container->getLocation().string(), fanartObjId);
             container->removeResource(fanart->getHandlerType());
             doUpdate = true;
             fanart = nullptr;
         }
     }
-    if (!fanart || fanart->getHandlerType() != ContentHandler::CONTAINERART) {
+    if (!fanart || isNew || fanart->getHandlerType() != ContentHandler::CONTAINERART) {
         if (fanart) {
             container->clearResources();
             doUpdate = true;
@@ -823,8 +890,10 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
         auto containerart = container->getResource(ResourcePurpose::Thumbnail);
         if (containerart) {
             fanart = containerart;
+        } else if (fanart) {
+            container->addResource(fanart); // restore if no image matches
         }
-        log_debug("fanart from dir {}", containerart != nullptr);
+        log_debug("fanart {} from dir {}", container->getLocation().string(), containerart != nullptr);
     }
 
     if (refObj && !fanart) {
@@ -856,7 +925,7 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
                 } else {
                     fanart = refFanArt;
                 }
-                log_debug("fanart from ref {}", fanart != nullptr);
+                log_debug("fanart {} from ref {}", location.string(), fanart != nullptr);
             }
         }
     }
@@ -870,12 +939,19 @@ void ImportService::assignFanArt(const std::shared_ptr<CdsContainer>& container,
         database->updateObject(container, nullptr);
 }
 
-std::shared_ptr<CdsContainer> ImportService::getContainer(const fs::path& location) const
+std::shared_ptr<CdsContainer> ImportService::getContainer(const std::string& location) const
 {
+    if (!hasCaseSensitiveNames) {
+        auto llocation = toLower(location);
+        return containerMap.at(llocation);
+    }
     return containerMap.at(location);
 }
 
-std::shared_ptr<CdsContainer> ImportService::createSingleContainer(int parentContainerId, const fs::directory_entry& dirEntry, const std::string& upnpClass)
+std::shared_ptr<CdsContainer> ImportService::createSingleContainer(
+    int parentContainerId,
+    const fs::directory_entry& dirEntry,
+    const std::string& upnpClass)
 {
     std::vector<std::shared_ptr<CdsObject>> cVec;
     auto location = dirEntry.path();
@@ -920,18 +996,34 @@ std::pair<int, bool> ImportService::addContainerTree(
                 tree = std::regex_replace(tree, std::regex(key), val);
             }
             auto dirKeyValues = std::vector<std::string>();
-            for (auto&& field : virtualDirKeys) {
+            for (auto&& vdirSetting : virtualDirKeys) {
+                std::string field;
+                std::string upnpClass;
+                for (auto&& [key, vdirField] : vdirSetting) {
+                    if (key == "metadata")
+                        field = vdirField;
+                    if (key == "class")
+                        upnpClass = vdirField;
+                }
+                if (!item->isSubClass(upnpClass))
+                    continue;
                 if (field == "LOCATION") {
                     std::string location = item->getLocation().c_str();
                     if (!location.empty()) {
                         dirKeyValues.push_back(location);
                         item->setLocation("");
                     }
-                } else {
-                    auto metaField = MetaEnumMapper::remapMetaDataField(field);
+                } else if (endswith(field, "_1")) {
+                    auto metaField = MetaEnumMapper::remapMetaDataField(field.replace(field.end() - 2, field.end(), ""));
                     auto keyValue = item->getMetaData(metaField);
                     if (!keyValue.empty())
                         dirKeyValues.push_back(keyValue);
+                } else if (!field.empty()) {
+                    auto metaField = MetaEnumMapper::remapMetaDataField(field);
+                    auto keyValueGroup = item->getMetaGroup(metaField);
+                    if (!keyValueGroup.empty())
+                        for (auto&& keyValue : keyValueGroup)
+                            dirKeyValues.push_back(keyValue);
                 }
             }
             if (!dirKeyValues.empty()) {
@@ -942,8 +1034,8 @@ std::pair<int, bool> ImportService::addContainerTree(
             if (!hasCaseSensitiveNames) {
                 subTree = toLower(subTree);
             }
-            if (containerMap.find(subTree) == containerMap.end() || !containerMap[subTree]) {
-                auto cont = database->findObjectByPath(subTree, DbFileType::Virtual);
+            if (containerMap.find(subTree) == containerMap.end() || !containerMap.at(subTree)) {
+                auto cont = database->findObjectByPath(subTree, UNUSED_CLIENT_GROUP, DbFileType::Virtual);
                 if (cont && cont->isContainer())
                     containerMap[subTree] = std::dynamic_pointer_cast<CdsContainer>(cont);
                 else
@@ -953,7 +1045,7 @@ std::pair<int, bool> ImportService::addContainerTree(
         } else if (subTree.empty()) {
             subTree = tree;
         }
-        if (containerMap.find(subTree) == containerMap.end() || !containerMap[subTree]) {
+        if (containerMap.find(subTree) == containerMap.end() || !containerMap.at(subTree)) {
             item->removeMetaData(MetadataFields::M_TITLE);
             item->addMetaData(MetadataFields::M_TITLE, item->getTitle());
             item->setParentID(result);
@@ -969,15 +1061,24 @@ std::pair<int, bool> ImportService::addContainerTree(
                 createdIds.push_back(result); // ensure update
             }
             isNew = true;
+            if (item->isContainer()) {
+                std::shared_ptr<CdsContainer> itemContainer = std::dynamic_pointer_cast<CdsContainer>(item);
+                container->setUpnpShortcut(itemContainer->getUpnpShortcut());
+            }
         } else {
-            result = containerMap[subTree]->getID();
-            if (item->getMTime() > containerMap[subTree]->getMTime()) {
+            result = containerMap.at(subTree)->getID();
+            if (item->getMTime() > containerMap.at(subTree)->getMTime()) {
                 createdIds.push_back(result);
             }
         }
         count++;
         if (isVirtual) // && chain.size() - count < containerImageParentCount
-            assignFanArt(containerMap[subTree], refItem && count > containerImageMinDepth ? refItem : item, AutoscanMediaMode::Mixed, false, chain.size() - count);
+            assignFanArt(containerMap.at(subTree),
+                refItem && count > containerImageMinDepth ? refItem : item,
+                AutoscanMediaMode::Mixed,
+                false /* isDir */,
+                chain.size() - count,
+                isNew);
     }
     log_debug("end '{}' {} {}", tree, result, isNew);
     return { result, isNew };
@@ -989,6 +1090,11 @@ void ImportService::updateFanArt(bool isDir)
         if (!stateEntry || !stateEntry->getObject() || !stateEntry->getObject()->isContainer())
             continue;
         std::shared_ptr<CdsContainer> container = std::dynamic_pointer_cast<CdsContainer>(stateEntry->getObject());
-        assignFanArt(container, stateEntry->getFirstObject(), stateEntry->getMediaMode(), isDir, 1);
+        assignFanArt(container,
+            stateEntry->getFirstObject(),
+            stateEntry->getMediaMode(),
+            isDir,
+            1,
+            false /* isNew */);
     }
 }
