@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -62,6 +62,8 @@ BuiltinLayout::BuiltinLayout(std::shared_ptr<Content> content)
     for (auto&& bl : blOption->getArrayCopy()) {
         // We could copy only the enabled containers, but to avoid checking dependencies here, we copy all of them!
         container[bl->getKey()] = std::make_shared<CdsContainer>(bl->getTitle());
+        if (!bl->getUpnpShortcut().empty())
+            container[bl->getKey()]->setUpnpShortcut(bl->getUpnpShortcut());
     }
 
     containerAt(BoxKeys::videoRoot)->addMetaData(MetadataFields::M_CONTENT_CLASS, UPNP_CLASS_VIDEO_ITEM);
@@ -76,7 +78,7 @@ BuiltinLayout::BuiltinLayout(std::shared_ptr<Content> content)
 
     containerAt(BoxKeys::audioRoot)->addMetaData(MetadataFields::M_CONTENT_CLASS, UPNP_CLASS_AUDIO_ITEM);
     chain["/Audio/All Audio"] = this->content->addContainerTree({ containerAt(BoxKeys::audioRoot), containerAt(BoxKeys::audioAll) }, nullptr);
-    if (blOption->get(BoxKeys::audioAllTracks)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllTracks)->getEnabled()) {
         chain["/Audio/All - full name"] = this->content->addContainerTree({ containerAt(BoxKeys::audioRoot), containerAt(BoxKeys::audioAllTracks) }, nullptr);
     }
 
@@ -84,7 +86,10 @@ BuiltinLayout::BuiltinLayout(std::shared_ptr<Content> content)
 #endif
 }
 
-int BuiltinLayout::add(const std::shared_ptr<CdsObject>& obj, const std::pair<int, bool>& parentID, bool useRef)
+int BuiltinLayout::add(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::pair<int, bool>& parentID,
+    bool useRef)
 {
     obj->setParentID(parentID.first);
     if (useRef)
@@ -95,7 +100,12 @@ int BuiltinLayout::add(const std::shared_ptr<CdsObject>& obj, const std::pair<in
     return obj->getID();
 }
 
-int BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path& rootPath, const std::string_view& c1, const std::string_view& c2, const std::string& upnpClass)
+int BuiltinLayout::getDir(
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& rootPath,
+    const std::string_view& c1,
+    const std::string_view& c2,
+    const std::string& upnpClass)
 {
     fs::path dir;
     auto&& objPath = obj->getLocation();
@@ -108,7 +118,11 @@ int BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path&
         dir = getLastPath(objPath);
 
     auto f2i = converterManager->f2i();
-    dir = f2i->convert(dir);
+    auto [mval, err] = f2i->convert(dir);
+    if (!err.empty()) {
+        log_warning("{}: {}", dir.string(), err);
+    }
+    dir = mval;
 
     if (!dir.empty()) {
         std::vector<std::shared_ptr<CdsObject>> dirVect;
@@ -125,7 +139,11 @@ int BuiltinLayout::getDir(const std::shared_ptr<CdsObject>& obj, const fs::path&
     return INVALID_OBJECT_ID;
 }
 
-std::vector<int> BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addVideo(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = converterManager->f2i();
     auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_BOX);
@@ -157,7 +175,7 @@ std::vector<int> BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj,
                 month.resize(m);
         }
 
-        if (blOption->get(BoxKeys::videoAllYears)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::videoAllYears)->getEnabled()) {
             if ((y > 0) && (m > 0)) {
                 std::vector<std::shared_ptr<CdsObject>> ct;
                 ct.reserve(4);
@@ -170,7 +188,7 @@ std::vector<int> BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj,
             }
         }
 
-        if (blOption->get(BoxKeys::videoAllDates)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::videoAllDates)->getEnabled()) {
             auto t = date.find('T');
             if (t != std::string::npos) {
                 date.resize(t);
@@ -184,13 +202,17 @@ std::vector<int> BuiltinLayout::addVideo(const std::shared_ptr<CdsObject>& obj,
         }
     }
 
-    if (blOption->get(BoxKeys::videoAllDirectories)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::videoAllDirectories)->getEnabled()) {
         result.push_back(getDir(obj, rootpath, BoxKeys::videoRoot, BoxKeys::videoAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Video, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Video))));
     }
     return result;
 }
 
-std::vector<int> BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addImage(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = converterManager->f2i();
     auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_BOX);
@@ -222,7 +244,7 @@ std::vector<int> BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj,
                 month.resize(m);
         }
 
-        if (blOption->get(BoxKeys::imageAllYears)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::imageAllYears)->getEnabled()) {
             if ((y > 0) && (m > 0)) {
                 std::vector<std::shared_ptr<CdsObject>> ct;
                 ct.push_back(containerAt(BoxKeys::imageRoot));
@@ -234,27 +256,31 @@ std::vector<int> BuiltinLayout::addImage(const std::shared_ptr<CdsObject>& obj,
             }
         }
 
-        if (blOption->get(BoxKeys::imageAllDates)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::imageAllDates)->getEnabled()) {
             auto t = date.find('T');
             if (t != std::string::npos) {
                 date.resize(t);
             }
             std::vector<std::shared_ptr<CdsObject>> ct;
             ct.push_back(containerAt(BoxKeys::imageRoot));
-            ct.push_back(containerAt(BoxKeys::imageAllYears));
+            ct.push_back(containerAt(BoxKeys::imageAllDates));
             ct.push_back(std::make_shared<CdsContainer>(date));
             id = content->addContainerTree(ct, obj);
             result.push_back(add(obj, id));
         }
     }
 
-    if (blOption->get(BoxKeys::imageAllDirectories)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::imageAllDirectories)->getEnabled()) {
         result.push_back(getDir(obj, rootpath, BoxKeys::imageRoot, BoxKeys::imageAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Image, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Image))));
     }
     return result;
 }
 
-std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& parent, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addAudio(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& parent,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     auto f2i = converterManager->f2i();
     auto blOption = config->getBoxLayoutListOption(ConfigVal::BOXLAYOUT_BOX);
@@ -340,7 +366,10 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
     }
 
     auto artistContainer = std::make_shared<CdsContainer>(artist, UPNP_CLASS_MUSIC_ARTIST);
-    if (blOption->get(BoxKeys::audioAllSongs)->getEnabled() && blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    artistContainer->addMetaData(MetadataFields::M_ARTIST, artist);
+    artistContainer->addMetaData(MetadataFields::M_ALBUMARTIST, artist);
+    artistContainer->addMetaData(MetadataFields::M_GENRE, genre);
+    if (blOption->getKey(BoxKeys::audioAllSongs)->getEnabled() && blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> arc;
         arc.push_back(containerAt(BoxKeys::audioRoot));
         arc.push_back(containerAt(BoxKeys::audioAllArtists));
@@ -361,11 +390,12 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
 
     auto albumContainer = std::make_shared<CdsContainer>(album, UPNP_CLASS_MUSIC_ALBUM);
     albumContainer->setMetaData(obj->getMetaData());
-    albumContainer->setResources(obj->getResources());
+    if (parent && parent->getResourceCount() > 0)
+        albumContainer->setResources(parent->getResources());
     albumContainer->setRefID(obj->getID());
     artistContainer->setSearchable(true);
 
-    if (blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> alc;
         alc.push_back(containerAt(BoxKeys::audioRoot));
         alc.push_back(containerAt(BoxKeys::audioAllArtists));
@@ -376,7 +406,7 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
     }
 
     albumContainer->setSearchable(true);
-    if (blOption->get(BoxKeys::audioAllAlbums)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllAlbums)->getEnabled()) {
         std::vector<std::shared_ptr<CdsObject>> allc;
         allc.push_back(containerAt(BoxKeys::audioRoot));
         allc.push_back(containerAt(BoxKeys::audioAllAlbums));
@@ -385,17 +415,20 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
         result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllGenres)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllGenres)->getEnabled()) {
+        auto genreContainer = std::make_shared<CdsContainer>(genre, UPNP_CLASS_MUSIC_GENRE);
+        genreContainer->addMetaData(MetadataFields::M_GENRE, genre);
         std::vector<std::shared_ptr<CdsObject>> ct;
         ct.push_back(containerAt(BoxKeys::audioRoot));
         ct.push_back(containerAt(BoxKeys::audioAllGenres));
-        ct.push_back(std::make_shared<CdsContainer>(genre, UPNP_CLASS_MUSIC_GENRE));
+        ct.push_back(std::move(genreContainer));
         id = content->addContainerTree(ct, obj);
         result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllComposers)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllComposers)->getEnabled()) {
         auto composerContainer = std::make_shared<CdsContainer>(composer, UPNP_CLASS_MUSIC_COMPOSER);
+        composerContainer->addMetaData(MetadataFields::M_COMPOSER, composer);
         composerContainer->setSearchable(true);
         std::vector<std::shared_ptr<CdsObject>> cc;
         cc.push_back(containerAt(BoxKeys::audioRoot));
@@ -405,8 +438,10 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
         result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllYears)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllYears)->getEnabled()) {
         auto yearContainer = std::make_shared<CdsContainer>(date);
+        yearContainer->addMetaData(MetadataFields::M_DATE, date);
+        yearContainer->addMetaData(MetadataFields::M_UPNP_DATE, date);
         yearContainer->setSearchable(true);
         std::vector<std::shared_ptr<CdsObject>> yt;
         yt.push_back(containerAt(BoxKeys::audioRoot));
@@ -416,11 +451,11 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
         result.push_back(add(obj, id));
     }
 
-    if (blOption->get(BoxKeys::audioAllDirectories)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllDirectories)->getEnabled()) {
         result.push_back(getDir(obj, rootpath, BoxKeys::audioRoot, BoxKeys::audioAllDirectories, getValueOrDefault(containerMap, AutoscanMediaMode::Audio, AutoscanDirectory::ContainerTypesDefaults.at(AutoscanMediaMode::Audio))));
     }
 
-    if (blOption->get(BoxKeys::audioArtistChronology)->getEnabled() && blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioArtistChronology)->getEnabled() && blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
         artistContainer->setSearchable(false);
         std::vector<std::shared_ptr<CdsObject>> chronology;
         chronology.push_back(containerAt(BoxKeys::audioRoot));
@@ -433,12 +468,12 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
     }
 
     // Keep this last, since it's modifying the object title
-    if (blOption->get(BoxKeys::audioAllTracks)->getEnabled()) {
+    if (blOption->getKey(BoxKeys::audioAllTracks)->getEnabled()) {
         artistContainer->setSearchable(true);
         obj->setTitle(fmt::format("{}{}", prefixTitle, title));
         result.push_back(add(obj, chain["/Audio/All - full name"]));
 
-        if (blOption->get(BoxKeys::audioAllArtists)->getEnabled()) {
+        if (blOption->getKey(BoxKeys::audioAllArtists)->getEnabled()) {
             std::vector<std::shared_ptr<CdsObject>> all;
             all.push_back(containerAt(BoxKeys::audioRoot));
             all.push_back(containerAt(BoxKeys::audioAllArtists));
@@ -452,7 +487,11 @@ std::vector<int> BuiltinLayout::addAudio(const std::shared_ptr<CdsObject>& obj,
 }
 
 #ifdef ONLINE_SERVICES
-std::vector<int> BuiltinLayout::addOnlineItem(const std::shared_ptr<CdsObject>& obj, OnlineServiceType serviceType, const fs::path& rootpath, const std::map<AutoscanMediaMode, std::string>& containerMap)
+std::vector<int> BuiltinLayout::addOnlineItem(
+    const std::shared_ptr<CdsObject>& obj,
+    OnlineServiceType serviceType,
+    const fs::path& rootpath,
+    const std::map<AutoscanMediaMode, std::string>& containerMap)
 {
     switch (serviceType) {
     case OnlineServiceType::Max:
