@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -75,7 +75,7 @@ std::pair<std::shared_ptr<CdsObject>, int> PlaylistParserScript::createObject2cd
     std::error_code ec;
     auto dirEnt = fs::directory_entry(loc, ec);
     if (!ec && isRegularFile(dirEnt, ec)) {
-        auto mainObj = database->findObjectByPath(dirEnt.path());
+        auto mainObj = database->findObjectByPath(dirEnt.path(), UNUSED_CLIENT_GROUP);
 
         if (!mainObj) {
             AutoScanSetting asSetting;
@@ -131,12 +131,15 @@ void PlaylistParserScript::handleObject2cdsItem(duk_context* ctx, const std::sha
         for (auto&& sym : keys) {
             auto val = ScriptNamedProperty(ctx, sym).getStringValue();
             if (!val.empty()) {
-                val = sc->convert(val);
-                item->addMetaData(sym, val);
+                auto [mval, err] = sc->convert(val);
+                if (!err.empty()) {
+                    log_warning("{}: {}", sym, err);
+                }
+                item->addMetaData(sym, mval);
 
                 if (writeThrough > 0 && pcd) {
                     pcd->removeMetaData(sym);
-                    setMetaData(pcd, std::static_pointer_cast<CdsItem>(pcd), sym, val);
+                    setMetaData(pcd, std::static_pointer_cast<CdsItem>(pcd), sym, mval);
                 }
             }
         }
