@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -48,6 +48,7 @@
 #include "content/content.h"
 #include "context.h"
 #include "database/database.h"
+#include "duk_compat.h"
 #include "js_functions.h"
 #include "script_names.h"
 #include "script_property.h"
@@ -75,16 +76,32 @@ static constexpr std::array jsGlobalFunctions {
     duk_function_list_entry { nullptr, nullptr, 0 },
 };
 
-void Script::setProperty(const std::string& name, const std::string& value)
+void Script::setProperty(const std::string& name, const std::string& value, bool doEmpty)
 {
-    duk_push_string(ctx, value.c_str());
-    duk_put_prop_string(ctx, -2, name.c_str());
+    if (doEmpty || !value.empty()) {
+        duk_push_string(ctx, value.c_str());
+        duk_put_prop_string(ctx, -2, camelCaseString(name).c_str());
+    }
 }
 
 void Script::setIntProperty(const std::string& name, int value)
 {
     duk_push_int(ctx, value);
-    duk_put_prop_string(ctx, -2, name.c_str());
+    duk_put_prop_string(ctx, -2, camelCaseString(name).c_str());
+}
+
+void Script::setIntProperty(const std::string& name, int value, int checkValue)
+{
+    if (value != checkValue) {
+        duk_push_int(ctx, value);
+        duk_put_prop_string(ctx, -2, camelCaseString(name).c_str());
+    }
+}
+
+void Script::setBoolProperty(const std::string& name, bool value)
+{
+    duk_push_int(ctx, value ? 1 : 0);
+    duk_put_prop_string(ctx, -2, camelCaseString(name).c_str());
 }
 
 /* **************** */
@@ -94,6 +111,7 @@ Script::Script(const std::shared_ptr<Content>& content, const std::string& paren
     : config(content->getContext()->getConfig())
     , database(content->getContext()->getDatabase())
     , converterManager(content->getContext()->getConverterManager())
+    , definition(content->getContext()->getDefinition())
     , content(content)
     , runtime(content->getScriptingRuntime())
     , sc(std::move(sc))
@@ -173,16 +191,13 @@ Script::Script(const std::shared_ptr<Content>& content, const std::string& paren
 
     duk_push_object(ctx); // config
     for (auto&& i : ConfigOptionIterator()) {
-        auto scs = ConfigDefinition::findConfigSetup(i, true);
+        auto scs = definition->findConfigSetup(i, true);
         if (!scs)
             continue;
-        auto value = scs->getCurrentValue();
-        if (!value.empty()) {
-            setProperty(scs->getItemPathRoot(), value);
-        }
+        setProperty(scs->getItemPathRoot(), scs->getCurrentValue(), false);
     }
 
-    for (auto&& dcs : ConfigDefinition::getConfigSetupList<ConfigDictionarySetup>()) {
+    for (auto&& dcs : definition->getConfigSetupList<ConfigDictionarySetup>()) {
         duk_push_object(ctx);
         auto dictionary = dcs->getValue()->getDictionaryOption(true);
         for (auto&& [key, val] : dictionary) {
@@ -191,7 +206,7 @@ Script::Script(const std::shared_ptr<Content>& content, const std::string& paren
         duk_put_prop_string(ctx, -2, dcs->getItemPathRoot().c_str());
     }
 
-    for (auto&& acs : ConfigDefinition::getConfigSetupList<ConfigArraySetup>()) {
+    for (auto&& acs : definition->getConfigSetupList<ConfigArraySetup>()) {
         auto array = acs->getValue()->getArrayOption(true);
         auto dukArray = duk_push_array(ctx);
         for (duk_uarridx_t i = 0; i < array.size(); i++) {
@@ -202,44 +217,54 @@ Script::Script(const std::shared_ptr<Content>& content, const std::string& paren
         duk_put_prop_string(ctx, -2, acs->getItemPathRoot().c_str());
     }
 
-    duk_push_object(ctx); // autoscan
-    std::string autoscanItemPath;
-    for (auto&& ascs : ConfigDefinition::getConfigSetupList<ConfigAutoscanSetup>()) {
-        auto autoscan = ascs->getValue()->getAutoscanListOption();
+    for (auto&& ascs : definition->getConfigSetupList<ConfigAutoscanSetup>()) {
+        duk_push_object(ctx); // autoscan
+
+        std::size_t idx = 0;
+        std::string autoscanItemPath = ascs->getItemPathRoot(true); // prefix
         for (const auto& adir : content->getAutoscanDirectories()) {
-            duk_push_object(ctx);
-            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LOCATION), adir->getLocation());
-            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MODE), AutoscanDirectory::mapScanmode(adir->getScanMode()));
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_INTERVAL), adir->getInterval().count());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_RECURSIVE), adir->getRecursive());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MEDIATYPE), adir->getMediaType());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_HIDDENFILES), adir->getHidden());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_SCANCOUNT), adir->getActiveScanCount());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_TASKCOUNT), adir->getTaskCount());
-            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LMT), fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT().count())));
+            if (adir->getScanMode() == ascs->getScanMode()) {
+                duk_push_object(ctx);
 
-            duk_put_prop_string(ctx, -2, fmt::to_string(adir->getScanID()).c_str());
+                setProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LOCATION), adir->getLocation());
+                setProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MODE), AutoscanDirectory::mapScanmode(adir->getScanMode()));
+                setIntProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_INTERVAL), adir->getInterval().count());
+                setBoolProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_RECURSIVE), adir->getRecursive());
+                setIntProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_MEDIATYPE), adir->getMediaType());
+                setBoolProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_HIDDENFILES), adir->getHidden());
+                setBoolProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_DIRTYPES), adir->hasDirTypes());
+                setBoolProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_FORCE_REREAD_UNKNOWN), adir->getForceRescan());
+                setIntProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_SCANCOUNT), adir->getActiveScanCount());
+                setIntProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_TASKCOUNT), adir->getTaskCount());
+                setIntProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_RETRYCOUNT), adir->getRetryCount());
+                setProperty(definition->removeAttribute(ConfigVal::A_AUTOSCAN_DIRECTORY_LMT), fmt::format("{:%Y-%m-%d %H:%M:%S}", fmt::localtime(adir->getPreviousLMT().count())));
+
+                duk_put_prop_string(ctx, -2, fmt::to_string(adir->getScanID()).c_str());
+                log_debug("Adding config[{}][{}] {}", autoscanItemPath, adir->getScanID(), adir->getLocation().string());
+                idx++;
+            }
         }
-        autoscanItemPath = ascs->getItemPathRoot(true); // prefix
+        duk_put_prop_string(ctx, -2, autoscanItemPath.c_str()); // autoscan
+        log_debug("Adding config[{}] {}", autoscanItemPath, idx);
     }
-    duk_put_prop_string(ctx, -2, autoscanItemPath.c_str()); // autoscan
 
-    for (auto&& bcs : ConfigDefinition::getConfigSetupList<ConfigBoxLayoutSetup>()) {
+    for (auto&& bcs : definition->getConfigSetupList<ConfigBoxLayoutSetup>()) {
         duk_push_object(ctx); // box-layout
-        std::string boxLayoutItemPath;
         auto boxLayoutList = bcs->getValue()->getBoxLayoutListOption();
         for (std::size_t i = 0; i < boxLayoutList->size(); i++) {
             duk_push_object(ctx);
             auto boxLayout = boxLayoutList->get(i);
             setIntProperty("id", boxLayout->getId());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_SIZE), boxLayout->getSize());
-            setIntProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_ENABLED), boxLayout->getEnabled());
-            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_TITLE), boxLayout->getTitle());
-            setProperty(ConfigDefinition::removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_CLASS), boxLayout->getClass());
+            setIntProperty(definition->removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_SIZE), boxLayout->getSize());
+            setBoolProperty(definition->removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_ENABLED), boxLayout->getEnabled());
+            setProperty(definition->removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_TITLE), boxLayout->getTitle());
+            setProperty(definition->removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_CLASS), boxLayout->getClass());
+            setProperty(definition->removeAttribute(ConfigVal::A_BOXLAYOUT_BOX_UPNP_SHORTCUT), boxLayout->getUpnpShortcut());
             duk_put_prop_string(ctx, -2, boxLayout->getKey().c_str());
         }
-        boxLayoutItemPath = bcs->getItemPathRoot(true); // prefix
+        std::string boxLayoutItemPath = bcs->getItemPathRoot(true); // prefix
         duk_put_prop_string(ctx, -2, boxLayoutItemPath.c_str()); // box-layout
+        log_debug("Adding config[{}] {}", boxLayoutItemPath, boxLayoutList->size());
     }
 
     duk_put_global_string(ctx, "config");
@@ -315,18 +340,18 @@ void Script::_load(const fs::path& scriptPath)
 
     auto j2i = converterManager->j2i();
     try {
-        scriptText = j2i->convert(scriptText, true);
+        auto [mval, err] = j2i->convert(scriptText, true);
+        if (!err.empty()) {
+            log_warning("{}: {}", scriptPath.string(), err);
+        }
+        scriptText = mval;
     } catch (const std::runtime_error& e) {
         throw_std_runtime_error("Failed to convert import script: {}", e.what());
     }
 
     duk_push_string(ctx, scriptPath.c_str());
     if (duk_pcompile_lstring_filename(ctx, 0, scriptText.c_str(), scriptText.length()) != 0) {
-#if DUK_VERSION > 20399
         log_error("Failed to load script {}: {}", scriptPath.c_str(), duk_safe_to_stacktrace(ctx, -1));
-#else
-        log_error("Failed to load script {}: {}", scriptPath.c_str(), duk_safe_to_string(ctx, -1));
-#endif
         throw_std_runtime_error("Scripting: failed to compile {}", scriptPath.c_str());
     }
 }
@@ -344,11 +369,7 @@ void Script::load(const fs::path& scriptPath)
 void Script::_execute()
 {
     if (duk_pcall(ctx, 0) != DUK_EXEC_SUCCESS) {
-#if DUK_VERSION > 20399
         log_error("Failed to execute script {}: {}", scriptPath.c_str(), duk_safe_to_stacktrace(ctx, -1));
-#else
-        log_error("Failed to execute script {}: {}", scriptPath.c_str(), duk_safe_to_string(ctx, -1));
-#endif
         throw_std_runtime_error("Script: failed to execute script");
     }
     duk_pop(ctx);
@@ -452,10 +473,15 @@ std::vector<int> Script::execute(const std::shared_ptr<CdsObject>& obj, const st
     return result;
 }
 
-std::vector<int> Script::call(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsContainer>& cont, const std::string& functionName, const fs::path& rootPath, const std::string& containerType)
+std::vector<int> Script::call(const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsContainer>& cont,
+    const std::string& functionName,
+    const fs::path& rootPath,
+    const std::string& containerType)
 {
     // write global object used in callback functionss
     cdsObject2dukObject(obj);
+    log_debug("wrote global object {} as {}", obj != nullptr, objectName);
     duk_put_global_string(ctx, objectName.c_str());
 
     // functionName(object, rootPath, autoScanId, containerType)
@@ -500,11 +526,7 @@ std::vector<int> Script::call(const std::shared_ptr<CdsObject>& obj, const std::
     if (duk_pcall(ctx, static_cast<duk_idx_t>(narg)) != DUK_EXEC_SUCCESS) {
         // Note: The invoked function will be blamed for execution errors, not the actual offending line of code
         // https://github.com/svaarala/duktape/blob/master/doc/error-objects.rst
-#if DUK_VERSION > 20399
         log_error("javascript {} runtime error: {}() - {}\n", contextName, functionName, duk_safe_to_stacktrace(ctx, -1));
-#else
-        log_error("javascript {} runtime error: {}() - {}\n", contextName, functionName, duk_safe_to_string(ctx, -1));
-#endif
         duk_pop(ctx);
         throw_std_runtime_error("javascript runtime error");
     }
@@ -536,7 +558,11 @@ void Script::setMetaData(const std::shared_ptr<CdsObject>& obj, const std::share
         } else
             item->setPartNumber(0);
     } else {
-        obj->addMetaData(sym, sc->convert(val));
+        auto [mval, err] = sc->convert(val);
+        if (!err.empty()) {
+            log_warning("{}: {}", obj->getLocation().string(), err);
+        }
+        obj->addMetaData(sym, mval);
     }
 }
 
@@ -584,10 +610,12 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
         if (flags >= 0)
             obj->setFlags(flags);
 
+        // get resources
         ScriptNamedProperty(ctx, "res").getObject([&]() {
             auto keys = ScriptProperty(ctx).getPropertyNames();
 
             int resCount = 0;
+            // read resources
             for (auto&& sym : keys) {
                 if (sym.find("handlerType") != std::string::npos) {
                     int ht = ScriptNamedProperty(ctx, sym).getIntValue(-1);
@@ -601,14 +629,19 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
                     resCount++;
                 }
             }
+            // update resource attributes
             for (auto&& res : obj->getResources()) {
                 resCount = res->getResId();
                 // only attributes enumerated in res_names are allowed
                 for (auto&& [key, upnp] : res_names) {
                     auto val = ScriptNamedProperty(ctx, resCount == 0 ? EnumMapper::getAttributeName(key) : fmt::format("{}-{}", resCount, EnumMapper::getAttributeName(key))).getStringValue();
                     if (!val.empty()) {
-                        val = sc->convert(val);
-                        res->addAttribute(key, val);
+                        auto [mval, err] = sc->convert(val);
+                        if (!err.empty()) {
+                            log_warning("{}: {}", obj->getLocation().string(), err);
+                        }
+                        res->addAttribute(key, mval);
+                        log_debug("add res attributes {}={}", key, mval);
                     }
                 }
                 auto head = fmt::format("{}#", resCount);
@@ -630,13 +663,17 @@ std::shared_ptr<CdsObject> Script::createObject(const std::shared_ptr<CdsObject>
             }
         });
 
+        // update aux data
         ScriptNamedProperty(ctx, "aux").getObject([&]() {
             auto keys = ScriptProperty(ctx).getPropertyNames();
             for (auto&& sym : keys) {
                 auto val = ScriptNamedProperty(ctx, sym).getStringValue();
                 if (!val.empty()) {
-                    val = sc->convert(val);
-                    obj->setAuxData(sym, val);
+                    auto [mval, err] = sc->convert(val);
+                    if (!err.empty()) {
+                        log_warning("{}: {}", obj->getLocation().string(), err);
+                    }
+                    obj->setAuxData(sym, mval);
                 }
             }
         });
@@ -656,30 +693,40 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         obj->setMTime(std::chrono::seconds(mtime));
     }
 
+    // update title
     {
         auto val = ScriptNamedProperty(ctx, "title").getStringValue();
         if (!val.empty()) {
-            val = sc->convert(val);
-            obj->setTitle(val);
+            auto [mval, err] = sc->convert(val);
+            if (!err.empty()) {
+                log_warning("{}: {}", obj->getLocation().string(), err);
+            }
+            obj->setTitle(mval);
         } else if (pcd) {
             obj->setTitle(pcd->getTitle());
         }
     }
 
+    // update upnpclass
     {
         auto val = ScriptNamedProperty(ctx, "upnpclass").getStringValue();
         if (!val.empty()) {
-            val = sc->convert(val);
-            obj->setClass(val);
+            auto [mval, err] = sc->convert(val);
+            if (!err.empty()) {
+                log_warning("{}: {}", obj->getLocation().string(), err);
+            }
+            obj->setClass(mval);
         } else if (pcd) {
             obj->setClass(pcd->getClass());
         }
     }
 
+    // update restricted
     int restricted = ScriptNamedProperty(ctx, "restricted").getBoolValue();
     if (restricted >= 0)
         obj->setRestricted(restricted);
 
+    // update metaData
     ScriptNamedProperty(ctx, "metaData").getObject([&]() {
         auto item = std::static_pointer_cast<CdsItem>(obj);
         auto keys = ScriptProperty(ctx).getPropertyNames();
@@ -699,11 +746,15 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         obj->setLocation(location);
     }
 
+    // update description
     auto description = ScriptNamedProperty(ctx, "description").getStringValue();
     if (!description.empty()) {
-        description = sc->convert(description);
+        auto [mval, err] = sc->convert(description);
+        if (!err.empty()) {
+            log_warning("{}: {}", obj->getLocation().string(), err);
+        }
         obj->removeMetaData(MetadataFields::M_DESCRIPTION);
-        obj->addMetaData(MetadataFields::M_DESCRIPTION, description);
+        obj->addMetaData(MetadataFields::M_DESCRIPTION, mval);
     }
 
     // CdsItem
@@ -714,48 +765,67 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         if (pcd)
             pcdItem = std::static_pointer_cast<CdsItem>(pcd);
 
+        // update mimetype
         auto mimetype = ScriptNamedProperty(ctx, "mimetype").getStringValue();
         if (!mimetype.empty()) {
-            mimetype = sc->convert(mimetype);
-            item->setMimeType(mimetype);
+            auto [mval, err] = sc->convert(mimetype);
+            if (!err.empty()) {
+                log_warning("{}: {}", obj->getLocation().string(), err);
+            }
+            item->setMimeType(mval);
         } else if (pcdItem) {
             item->setMimeType(pcdItem->getMimeType());
         }
 
+        // update serviceID for onlineservice
         auto serviceID = ScriptNamedProperty(ctx, "serviceID").getStringValue();
         if (!serviceID.empty()) {
-            serviceID = sc->convert(serviceID);
-            item->setServiceID(serviceID);
+            auto [mval, err] = sc->convert(serviceID);
+            if (!err.empty()) {
+                log_warning("{}: {}", obj->getLocation().string(), err);
+            }
+            item->setServiceID(mval);
         }
 
+        // update description if not set in script
         {
             auto val = ScriptNamedProperty(ctx, "description").getStringValue();
             if (!val.empty()) {
-                val = sc->convert(val);
+                auto [mval, err] = sc->convert(val);
+                if (!err.empty()) {
+                    log_warning("{}: {}", obj->getLocation().string(), err);
+                }
                 item->removeMetaData(MetadataFields::M_DESCRIPTION);
-                item->addMetaData(MetadataFields::M_DESCRIPTION, val);
+                item->addMetaData(MetadataFields::M_DESCRIPTION, mval);
             } else if (pcdItem && item->getMetaData(MetadataFields::M_DESCRIPTION).empty() && !pcdItem->getMetaData(MetadataFields::M_DESCRIPTION).empty()) {
                 item->addMetaData(MetadataFields::M_DESCRIPTION, pcdItem->getMetaData(MetadataFields::M_DESCRIPTION));
             }
         }
 
+        // update location if not set in script
         if (location.empty() && pcd) {
             obj->setLocation(pcd->getLocation());
         }
 
+        // CdsExternalItem (like links)
         if (obj->isExternalItem()) {
             std::string protocolInfo;
 
             obj->setRestricted(true);
 
+            // update protocol
             auto protocol = ScriptNamedProperty(ctx, "protocol").getStringValue();
             if (!protocol.empty()) {
-                protocol = sc->convert(protocol);
-                protocolInfo = renderProtocolInfo(item->getMimeType(), protocol);
+                auto [mval, err] = sc->convert(protocol);
+                if (!err.empty()) {
+                    log_warning("{}: {}", obj->getLocation().string(), err);
+                }
+                protocolInfo = renderProtocolInfo(item->getMimeType(), mval);
             } else {
                 protocolInfo = renderProtocolInfo(item->getMimeType(), PROTOCOL);
             }
 
+            // add resources
             std::shared_ptr<CdsResource> resource;
             if (item->getResourceCount() == 0) {
                 resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
@@ -780,11 +850,22 @@ std::shared_ptr<CdsObject> Script::dukObject2cdsObject(const std::shared_ptr<Cds
         if (id >= CDS_ID_ROOT)
             cont->setUpdateID(id);
 
+        // update searchable
         int searchable = ScriptNamedProperty(ctx, "searchable").getBoolValue();
         log_debug("{} searchable {}", cont->getTitle(), searchable);
         if (searchable >= 0)
             cont->setSearchable(searchable);
 
+        // update upnpShortcut
+        auto upnpShortcut = ScriptNamedProperty(ctx, "upnpShortcut").getStringValue();
+        if (!upnpShortcut.empty()) {
+            auto [mval, err] = sc->convert(upnpShortcut);
+            if (!err.empty()) {
+                log_warning("{}: {}", obj->getLocation().string(), err);
+            }
+            cont->setUpnpShortcut(mval);
+        }
+
         handleObject2cdsContainer(ctx, pcd, cont);
     }
 
@@ -803,47 +884,24 @@ bool Script::isHiddenFile(const std::shared_ptr<CdsObject>& cdsObj, const std::s
 
 void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
 {
-    std::string val;
-
     duk_push_object(ctx);
 
     // CdsObject
     setIntProperty("objectType", obj->getObjectType());
+    setIntProperty("id", obj->getID(), INVALID_OBJECT_ID);
+    setIntProperty("parentID", obj->getParentID(), INVALID_OBJECT_ID);
 
-    int id = obj->getID();
-
-    if (id != INVALID_OBJECT_ID)
-        setIntProperty("id", id);
-
-    id = obj->getParentID();
-    if (id != INVALID_OBJECT_ID)
-        setIntProperty("parentID", id);
-
-    val = obj->getTitle();
-    if (!val.empty())
-        setProperty("title", val);
-
-    val = obj->getClass();
-    if (!val.empty())
-        setProperty("upnpclass", val);
-
-    val = obj->getLocation();
-    if (!val.empty())
-        setProperty("location", val);
+    setProperty("title", obj->getTitle(), false);
+    setProperty("upnpclass", obj->getClass(), false);
+    setProperty("location", obj->getLocation(), false);
 
     setIntProperty("mtime", static_cast<int>(obj->getMTime().count()));
     setIntProperty("utime", static_cast<int>(obj->getUTime().count()));
     setIntProperty("sizeOnDisk", static_cast<int>(obj->getSizeOnDisk()));
     setIntProperty("flags", obj->getFlags());
 
-    // TODO: boolean type
-    int restricted = obj->isRestricted();
-    setIntProperty("restricted", restricted);
-
-    if (obj->getFlag(OBJECT_FLAG_OGG_THEORA))
-        setIntProperty("theora", 1);
-    else
-        setIntProperty("theora", 0);
+    setBoolProperty("restricted", obj->isRestricted());
+    setBoolProperty("theora", obj->getFlag(OBJECT_FLAG_OGG_THEORA));
 
 #ifdef ONLINE_SERVICES
     if (obj->getFlag(OBJECT_FLAG_ONLINE_SERVICE)) {
@@ -945,41 +1003,41 @@ void Script::cdsObject2dukObject(const std::shared_ptr<CdsObject>& obj)
     if (obj->isItem() && item) {
         setIntProperty("trackNumber", item->getTrackNumber());
         setIntProperty("partNumber", item->getPartNumber());
-        val = item->getMimeType();
-        if (!val.empty())
-            setProperty("mimetype", val);
-
-        val = item->getServiceID();
-        if (!val.empty())
-            setProperty("serviceID", val);
+        setProperty("mimetype", item->getMimeType(), false);
+        setProperty("serviceID", item->getServiceID(), false);
     }
 
     // CdsDirectory
     if (obj->isContainer()) {
         auto cont = std::static_pointer_cast<CdsContainer>(obj);
-        // TODO: boolean type, hide updateID
         setIntProperty("updateID", cont->getUpdateID());
-
-        int searchable = cont->isSearchable();
-        setIntProperty("searchable", searchable);
+        setBoolProperty("searchable", cont->isSearchable());
     }
 }
 
-std::string Script::convertToCharset(const std::string& str, charset_convert_t chr)
+std::string Script::convertToCharset(const std::string& str, CharsetConversion chr)
 {
-    switch (chr) {
-    case P2I:
-        return _p2i->convert(str);
-    case M2I:
-        return _m2i->convert(str);
-    case F2I:
-        return _f2i->convert(str);
-    case J2I:
-        return _j2i->convert(str);
-    case I2I:
-        return _i2i->convert(str);
+    auto convCode = [=]() {
+        switch (chr) {
+        case CharsetConversion::P2I:
+            return _p2i->convert(str);
+        case CharsetConversion::M2I:
+            return _m2i->convert(str);
+        case CharsetConversion::F2I:
+            return _f2i->convert(str);
+        case CharsetConversion::J2I:
+            return _j2i->convert(str);
+        case CharsetConversion::I2I:
+            return _i2i->convert(str);
+        default:
+            throw_std_runtime_error("Illegal charset given to convertToCharset(): {}", chr);
+        }
+    };
+    auto [mval, err] = convCode();
+    if (!err.empty()) {
+        log_warning("{}: {}", str, err);
     }
-    throw_std_runtime_error("Illegal charset given to convertToCharset(): {}", chr);
+    return mval;
 }
 
 #endif // HAVE_JS
