@@ -4,7 +4,7 @@
 
     context.h - this file is part of Gerbera.
 
-    Copyright (C) 2021-2024 Gerbera Contributors
+    Copyright (C) 2021-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -28,8 +28,9 @@
 
 #include <memory>
 
-// forward declaration
+// forward declarations
 class Config;
+class ConfigDefinition;
 class ClientManager;
 class Database;
 class Mime;
@@ -42,13 +43,20 @@ class SessionManager;
 
 class Context {
 public:
-    Context(std::shared_ptr<Config> config,
+    Context(
+        std::shared_ptr<ConfigDefinition> definition,
+        std::shared_ptr<Config> config,
         std::shared_ptr<ClientManager> clients,
         std::shared_ptr<Mime> mime,
         std::shared_ptr<Database> database,
         std::shared_ptr<Web::SessionManager> sessionManager,
         std::shared_ptr<ConverterManager> converterManager);
 
+    std::shared_ptr<ConfigDefinition> getDefinition() const
+    {
+        return definition;
+    }
+
     std::shared_ptr<Config> getConfig() const
     {
         return config;
@@ -80,6 +88,7 @@ public:
     }
 
 private:
+    std::shared_ptr<ConfigDefinition> definition;
     std::shared_ptr<Config> config;
     std::shared_ptr<ClientManager> clients;
     std::shared_ptr<Mime> mime;
