@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -42,10 +42,6 @@
 
 #include <netinet/in.h>
 
-//#define MYSQL_SET_NAMES "/*!40101 SET NAMES utf8 */"
-//#define MYSQL_SELECT_DEBUG
-//#define MYSQL_EXEC_DEBUG
-
 #define MYSQL_SET_VERSION "INSERT INTO `mt_internal_setting` VALUES ('db_version','{}')"
 static constexpr auto mysqlUpdateVersion = std::string_view("UPDATE `mt_internal_setting` SET `value`='{}' WHERE `key`='db_version' AND `value`='{}'");
 static constexpr auto mysqlAddResourceAttr = std::string_view("ALTER TABLE `grb_cds_resource` ADD COLUMN `{}` varchar(255) default NULL");
@@ -57,10 +53,10 @@ MySQLDatabase::MySQLDatabase(const std::shared_ptr<Config>& config, const std::s
     table_quote_end = '`';
 
     // if mysql.sql or mysql-upgrade.xml is changed hashies have to be updated
-    hashies = { 3813459699, // index 0 is used for create script mysql.sql = Version 1
+    hashies = { 684441550, // index 0 is used for create script mysql.sql = Version 1
         928913698, 1984244483, 2241152998, 1748460509, 2860006966, 974692115, 70310290, 1863649106, 4238128129, 2979337694, // upgrade 2-11
         1512596496, 507706380, 3545156190, 31528140, 372163748, 4097073836, 751952276, 3893982139, 798767550, 3731206823, // upgrade 12-21
-        3643149536 };
+        3643149536, 4280737637 };
 }
 
 MySQLDatabase::~MySQLDatabase()
@@ -72,8 +68,8 @@ MySQLDatabase::~MySQLDatabase()
         mysql_close(&db);
         mysql_connection = false;
     }
-    log_debug("calling mysql_server_end...");
-    mysql_server_end();
+    log_debug("calling mysql_library_end ...");
+    mysql_library_end();
     log_debug("...ok");
 }
 
@@ -96,57 +92,58 @@ void MySQLDatabase::threadCleanup()
     }
 }
 
-void MySQLDatabase::init()
+void MySQLDatabase::connect()
 {
-    log_debug("start");
-    SQLDatabase::init();
-
-    std::unique_lock<decltype(sqlMutex)> lock(sqlMutex);
-
     if (!mysql_thread_safe()) {
         throw_std_runtime_error("mysql library is not thread safe");
     }
 
-    /// \todo write destructor function
-    int ret = pthread_key_create(&mysql_init_key, nullptr);
-    if (ret) {
+    // Desctructor function not necessary because no memory is allocated
+    if (pthread_key_create(&mysql_init_key, nullptr)) {
         throw_std_runtime_error("could not create pthread_key");
     }
-    mysql_server_init(0, nullptr, nullptr);
+    mysql_library_init(0, nullptr, nullptr);
     pthread_setspecific(mysql_init_key, &mysql_init_val);
 
-    MYSQL* resMysql = mysql_init(&db);
-    if (!resMysql) {
-        throw_std_runtime_error("mysql_init() failed");
+    {
+        MYSQL* resMysql = mysql_init(&db);
+        if (!resMysql) {
+            throw_std_runtime_error("mysql_init() failed");
+        }
+
+        mysql_options(&db, MYSQL_SET_CHARSET_NAME, "utf8mb4");
+
+        bool myBoolVar = true;
+        mysql_options(&db, MYSQL_OPT_RECONNECT, &myBoolVar);
     }
 
-    mysql_options(&db, MYSQL_SET_CHARSET_NAME, "utf8mb4");
+    {
+        std::string dbHost = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_HOST);
+        std::string dbName = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_DATABASE);
+        std::string dbUser = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_USERNAME);
+        auto dbPort = in_port_t(config->getIntOption(ConfigVal::SERVER_STORAGE_MYSQL_PORT));
+        std::string dbPass = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_PASSWORD);
+        std::string dbSock = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_SOCKET);
 
-    bool myBoolVar = true;
-    mysql_options(&db, MYSQL_OPT_RECONNECT, &myBoolVar);
-
-    std::string dbHost = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_HOST);
-    std::string dbName = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_DATABASE);
-    std::string dbUser = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_USERNAME);
-    auto dbPort = in_port_t(config->getIntOption(ConfigVal::SERVER_STORAGE_MYSQL_PORT));
-    std::string dbPass = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_PASSWORD);
-    std::string dbSock = config->getOption(ConfigVal::SERVER_STORAGE_MYSQL_SOCKET);
-
-    resMysql = mysql_real_connect(&db,
-        dbHost.c_str(),
-        dbUser.c_str(),
-        (dbPass.empty() ? nullptr : dbPass.c_str()),
-        dbName.c_str(),
-        dbPort, // port
-        (dbSock.empty() ? nullptr : dbSock.c_str()), // socket
-        0 // flags
-    );
-    if (!resMysql) {
-        throw_std_runtime_error("Connecting to database {}:{}/{} failed: {}", dbHost, dbPort, dbName, getError(&db));
+        MYSQL* resMysql = mysql_real_connect(&db,
+            dbHost.c_str(),
+            dbUser.c_str(),
+            (dbPass.empty() ? nullptr : dbPass.c_str()),
+            dbName.c_str(),
+            dbPort, // port
+            (dbSock.empty() ? nullptr : dbSock.c_str()), // socket
+            0 // flags
+        );
+        if (!resMysql) {
+            throw_std_runtime_error("Connecting to database {}:{}/{} failed: {}", dbHost, dbPort, dbName, getError(&db));
+        }
     }
 
     mysql_connection = true;
+}
 
+std::string MySQLDatabase::prepareDatabase()
+{
     std::string dbVersion;
     try {
         dbVersion = getInternalSetting("db_version");
@@ -168,7 +165,7 @@ void MySQLDatabase::init()
                     continue;
                 }
                 log_debug("executing statement: '{}'", statement);
-                ret = mysql_real_query(&db, statement.c_str(), statement.size());
+                int ret = mysql_real_query(&db, statement.c_str(), statement.size());
                 if (ret) {
                     std::string myError = getError(&db);
                     throw DatabaseException(myError, fmt::format("Mysql: error while creating db: {}", myError));
@@ -186,8 +183,20 @@ void MySQLDatabase::init()
         }
         log_info("Database created successfully!");
     }
+    return dbVersion;
+}
 
+void MySQLDatabase::init()
+{
+    log_debug("start");
+    SQLDatabase::init();
+
+    std::unique_lock<decltype(sqlMutex)> lock(sqlMutex);
+
+    connect();
+    auto dbVersion = prepareDatabase();
     upgradeDatabase(std::stoul(dbVersion), hashies, ConfigVal::SERVER_STORAGE_MYSQL_UPGRADE_FILE, mysqlUpdateVersion, mysqlAddResourceAttr);
+    initDynContainers();
 
     lock.unlock();
 
@@ -217,11 +226,6 @@ std::string MySQLDatabase::quote(const std::string& value) const
 
 std::string MySQLDatabase::getError(MYSQL* db)
 {
-#ifdef MYSQL_SELECT_DEBUG
-    if (hasDebugging)
-        print_backtrace();
-#endif
-
     auto res = fmt::format("mysql_error ({}): \"{}\"", mysql_errno(db), mysql_error(db));
     log_debug("{}", res);
     return res;
@@ -269,9 +273,6 @@ void MySQLDatabaseWithTransactions::commit(std::string_view tName)
 std::shared_ptr<SQLResult> MySQLDatabaseWithTransactions::select(const std::string& query)
 {
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
@@ -303,9 +304,6 @@ std::shared_ptr<SQLResult> MySQLDatabaseWithTransactions::select(const std::stri
 std::shared_ptr<SQLResult> MySQLDatabase::select(const std::string& query)
 {
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
@@ -330,9 +328,6 @@ void MySQLDatabase::del(std::string_view tableName, const std::string& clause, c
         ? fmt::format("DELETE FROM {}", identifier(std::string(tableName))) //
         : fmt::format("DELETE FROM {} WHERE {}", identifier(std::string(tableName)), clause);
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
@@ -347,9 +342,6 @@ void MySQLDatabase::del(std::string_view tableName, const std::string& clause, c
 void MySQLDatabase::exec(std::string_view tableName, const std::string& query, int objId)
 {
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
@@ -364,9 +356,6 @@ void MySQLDatabase::exec(std::string_view tableName, const std::string& query, i
 int MySQLDatabase::exec(const std::string& query, bool getLastInsertId)
 {
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
@@ -385,9 +374,6 @@ int MySQLDatabase::exec(const std::string& query, bool getLastInsertId)
 void MySQLDatabase::execOnly(const std::string& query)
 {
     log_debug("{}", query);
-#ifdef MYSQL_EXEC_DEBUG
-    print_backtrace();
-#endif
 
     checkMysqlThreadInit();
     SqlAutoLock lock(sqlMutex);
