@@ -3,7 +3,7 @@
 
   search_handler.h - this file is part of Gerbera.
 
-  Copyright (C) 2018-2024 Gerbera Contributors
+  Copyright (C) 2018-2025 Gerbera Contributors
 
   Gerbera is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
@@ -28,6 +28,7 @@
 #ifndef __SEARCH_HANDLER_H__
 #define __SEARCH_HANDLER_H__
 
+#include "upnp/upnp_common.h"
 #include "util/tools.h"
 
 #include <algorithm>
@@ -38,14 +39,11 @@
 #include <unordered_map>
 #include <vector>
 
-#define UPNP_SEARCH_CLASS "upnp:class"
 #define UPNP_SEARCH_ID "@id"
 #define UPNP_SEARCH_REFID "@refID"
 #define UPNP_SEARCH_PARENTID "@parentID"
 #define UPNP_SEARCH_LAST_UPDATED "last_updated"
 #define UPNP_SEARCH_LAST_MODIFIED "last_modified"
-#define UPNP_SEARCH_LAST_PLAYED "upnp:lastPlaybackTime"
-#define UPNP_SEARCH_PLAY_COUNT "upnp:playbackCount"
 #define UPNP_SEARCH_PLAY_GROUP "play_group"
 #define UPNP_SEARCH_PATH "path"
 #define META_NAME "name"
@@ -293,23 +291,31 @@ public:
     virtual ~SQLEmitter() = default;
     virtual std::string emitSQL(const ASTNode* node) const = 0;
     virtual std::string emit(const ASTAsterisk* node) const = 0;
-    virtual std::string emit(const ASTParenthesis* node, const std::string& bracketedNode) const = 0;
+    virtual std::string emit(const ASTParenthesis* node,
+        const std::string& bracketedNode) const
+        = 0;
     virtual std::string emit(const ASTDQuote* node) const = 0;
     virtual std::string emit(const ASTCompareOperator* node,
-        const std::string& property, const std::string& value) const = 0;
+        const std::string& property, const std::string& value) const
+        = 0;
     virtual std::string emit(const ASTStringOperator* node,
-        const std::string& property, const std::string& value) const = 0;
+        const std::string& property, const std::string& value) const
+        = 0;
     virtual std::string emit(const ASTExistsOperator* node,
-        const std::string& property, const std::string& value) const = 0;
+        const std::string& property, const std::string& value) const
+        = 0;
     virtual std::string emit(const ASTAndOperator* node,
-        const std::string& lhs, const std::string& rhs) const = 0;
+        const std::string& lhs, const std::string& rhs) const
+        = 0;
     virtual std::string emit(const ASTOrOperator* node,
-        const std::string& lhs, const std::string& rhs) const = 0;
+        const std::string& lhs, const std::string& rhs) const
+        = 0;
 };
 
 enum class FieldType {
     String,
     Integer,
+    Bool,
     Date,
 };
 
