@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -50,7 +50,7 @@ class CdsResource;
 class SQLResult;
 class SQLEmitter;
 
-#define DBVERSION 22
+#define DBVERSION 23
 
 #define CDS_OBJECT_TABLE "mt_cds_object"
 #define INTERNAL_SETTINGS_TABLE "mt_internal_setting"
@@ -145,7 +145,7 @@ public:
     std::unique_ptr<ChangedContainers> removeObject(int objectID, const fs::path& path, bool all) override;
     std::unique_ptr<ChangedContainers> removeObjects(const std::unordered_set<int>& list, bool all = false) override;
 
-    std::shared_ptr<CdsObject> loadObjectByServiceID(const std::string& serviceID) override;
+    std::shared_ptr<CdsObject> loadObjectByServiceID(const std::string& serviceID, const std::string& group) override;
     std::vector<int> getServiceObjectIDs(char servicePrefix) override;
 
     /* accounting methods */
@@ -153,13 +153,13 @@ public:
     std::map<std::string, long long> getGroupStats(const StatsParam& stats) override;
 
     std::vector<std::shared_ptr<CdsObject>> browse(BrowseParam& param) override;
-    std::vector<std::shared_ptr<CdsObject>> search(const SearchParam& param, int* numMatches) override;
+    std::vector<std::shared_ptr<CdsObject>> search(SearchParam& param) override;
 
     std::vector<std::string> getMimeTypes() override;
+    std::map<std::string, std::shared_ptr<CdsContainer>> getShortcuts() override;
 
-    std::vector<std::shared_ptr<CdsObject>> findObjectByContentClass(const std::string& contentClass) override;
-    // virtual std::shared_ptr<CdsObject> findObjectByTitle(std::string title, int parentID);
-    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) override;
+    std::vector<std::shared_ptr<CdsObject>> findObjectByContentClass(const std::string& contentClass, const std::string& group) override;
+    std::shared_ptr<CdsObject> findObjectByPath(const fs::path& fullpath, const std::string& group, DbFileType fileType = DbFileType::Auto) override;
     int findObjectIDByPath(const fs::path& fullpath, DbFileType fileType = DbFileType::Auto) override;
     std::string incrementUpdateIDs(const std::unordered_set<int>& ids) override;
 
@@ -214,6 +214,7 @@ public:
 protected:
     explicit SQLDatabase(const std::shared_ptr<Config>& config, std::shared_ptr<Mime> mime, std::shared_ptr<ConverterManager> converterManager);
     void init() override;
+    void initDynContainers(const std::shared_ptr<CdsObject>& sParent = {});
 
     /// \brief migrate metadata from mt_cds_objects to mt_metadata before removing the column (DBVERSION 12)
     bool doMetadataMigration();
@@ -257,6 +258,10 @@ private:
     // only columns listed here are added to the insert and update statements
     std::map<std::string, std::vector<std::string>> tableColumnOrder;
 
+    /// \brief Configuration content for dynamic folders
+    std::shared_ptr<DynamicContentList> dynamicContentList;
+    bool dynamicContentEnabled;
+
     std::shared_ptr<CdsObject> createObjectFromRow(const std::string& group, const std::unique_ptr<SQLRow>& row);
     std::shared_ptr<CdsObject> createObjectFromSearchRow(const std::string& group, const std::unique_ptr<SQLRow>& row);
     std::vector<std::pair<std::string, std::string>> retrieveMetaDataForObject(int objectId);
@@ -313,7 +318,7 @@ private:
     std::vector<int> _checkOverlappingAutoscans(const std::shared_ptr<AutoscanDirectory>& adir);
 
     /* location helper: filesystem path or virtual path to db location*/
-    static std::string addLocationPrefix(char prefix, const fs::path& path);
+    static std::string addLocationPrefix(char prefix, const fs::path& path, const std::string_view& suffix = "");
     /* location helpers: db location to filesystem path */
     static std::pair<fs::path, char> stripLocationPrefix(std::string_view dbLocation);
 
