@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -77,7 +77,7 @@ grb_read_t MemIOHandler::read(std::byte* buf, std::size_t length)
 
     std::memcpy(buf, buffer + pos, length);
     pos = pos + length;
-    ret = static_cast<int>(length);
+    ret = length;
 
     if (pos >= this->length) {
         pos = -1;
