@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -39,6 +39,7 @@
 
 #define GRB_LOG_FAC GrbLogFacility::server
 
+#include "config/config_definition.h"
 #include "config/config_manager.h"
 #include "config/config_val.h"
 #include "config/grb_runtime.h"
@@ -102,7 +103,7 @@ static void signalHandler(int signum)
         runtime.exit(EXIT_FAILURE);
     }
 
-    if ((signum == SIGINT) || (signum == SIGTERM)) {
+    if (signum == SIGINT || signum == SIGTERM) {
         _ctx.shutdownFlag++;
         if (_ctx.shutdownFlag == 1) {
             log_info("Gerbera shutting down. Please wait...");
@@ -159,8 +160,11 @@ static void installSignalHandler()
 
 int main(int argc, char** argv, char** envp)
 {
+    std::shared_ptr<ConfigDefinition> definition = std::make_shared<ConfigDefinition>();
+    definition->init(definition);
+
     cxxopts::Options options(GerberaRuntime::ProgramName, "Gerbera UPnP Media Server - https://gerbera.io");
-    runtime.init(&options);
+    runtime.init(definition, &options);
     std::optional<std::string> magic = runtime.getMagic();
     const auto additionalArgs = std::vector<ConfigOptionArgs> {
         { ConfigVal::SERVER_PORT, "p", "port", "Port to bind with, must be >=49152", std::optional<std::string>(), "PORT" },
@@ -181,8 +185,8 @@ int main(int argc, char** argv, char** envp)
         ("D," GRB_OPTION_DEBUG, "Enable debugging", cxxopts::value<bool>()->default_value("false")) //
         ("v," GRB_OPTION_VERSION, "Print version info and exit") //
         (GRB_OPTION_COMPILEINFO, "Print compile info and exit") //
-        (GRB_OPTION_CREATECONFIG, "Print a default config.xml file and exit") //
-        (GRB_OPTION_CREATEEXAMPLECONFIG, "Print a example config.xml file and exit") //
+        (GRB_OPTION_CREATECONFIG, "Print a default config.xml file and exit", cxxopts::value<std::string>()->implicit_value("All"), "Section") //
+        (GRB_OPTION_CREATEEXAMPLECONFIG, "Print a example config.xml file and exit", cxxopts::value<std::string>()->implicit_value("All"), "Section") //
         (GRB_OPTION_CHECKCONFIG, "Check config.xml file and exit") //
         (GRB_OPTION_PRINTOPTIONS, "Print simple config options and exit") //
         (GRB_OPTION_OFFLINE, "Do not answer UPnP content requests") // good for initial scans
@@ -214,6 +218,7 @@ int main(int argc, char** argv, char** envp)
         int portnum = -1;
         try {
             configManager = std::make_shared<ConfigManager>(
+                definition,
                 runtime.getConfigFile(), runtime.getHome(), runtime.getConfDir(),
                 runtime.getDataDir(), runtime.getDebug());
             configManager->load(runtime.getHome());
@@ -226,11 +231,14 @@ int main(int argc, char** argv, char** envp)
                 runtime.exit(EXIT_SUCCESS);
             }
             portnum = configManager->getUIntOption(ConfigVal::SERVER_PORT);
-        } catch (const ConfigParseException& ce) {
-            log_error("Error parsing config file '{}': {}", runtime.getConfigFile().c_str(), ce.what());
+        } catch (const ConfigParseException& ex) {
+            log_error("Error parsing config file '{}': {}", runtime.getConfigFile().c_str(), ex.what());
             runtime.exit(EXIT_FAILURE);
-        } catch (const std::runtime_error& e) {
-            log_error("{}", e.what());
+        } catch (const std::runtime_error& ex) {
+            log_error("Runtime error parsing config file '{}': {}", runtime.getConfigFile().c_str(), ex.what());
+            runtime.exit(EXIT_FAILURE);
+        } catch (const std::logic_error& ex) {
+            log_error("Logic error parsing config file '{}': {}", runtime.getConfigFile().c_str(), ex.what());
             runtime.exit(EXIT_FAILURE);
         }
 
@@ -241,7 +249,7 @@ int main(int argc, char** argv, char** envp)
         auto server = std::make_shared<Server>(configManager);
 
         try {
-            server->init(runtime.getOffline());
+            server->init(definition, runtime.getOffline());
             server->run();
         } catch (const UpnpException& ue) {
             log_error("{}", ue.what());
@@ -272,13 +280,16 @@ int main(int argc, char** argv, char** envp)
                 server.reset();
                 configManager.reset();
                 runtime.shutdown();
-            } catch (const std::runtime_error& e) {
-                log_error("{}", e.what());
+            } catch (const std::runtime_error& ex) {
+                log_error("Shutdown: {}", ex.what());
             }
 
             runtime.exit(EXIT_FAILURE);
-        } catch (const std::runtime_error& e) {
-            log_error("{}", e.what());
+        } catch (const std::runtime_error& ex) {
+            log_error("Run: runtime error {}", ex.what());
+            runtime.exit(EXIT_FAILURE);
+        } catch (const std::logic_error& ex) {
+            log_error("Run: logic error {}", ex.what());
             runtime.exit(EXIT_FAILURE);
         }
 
@@ -297,12 +308,16 @@ int main(int argc, char** argv, char** envp)
                     server->shutdown();
                     server.reset();
                     configManager.reset();
+                    definition.reset();
                     runtime.shutdown();
 
-                    GerberaRuntime runtimeChild(&options);
+                    definition = std::make_shared<ConfigDefinition>();
+                    definition->init(definition);
+                    GerberaRuntime runtimeChild(definition, &options);
                     try {
                         runtimeChild.handleOptions(&results, false);
                         configManager = std::make_shared<ConfigManager>(
+                            definition,
                             runtimeChild.getConfigFile(), runtimeChild.getHome(), runtimeChild.getConfDir(),
                             runtimeChild.getDataDir(), runtimeChild.getDebug());
                         configManager->load(runtimeChild.getHome());
@@ -313,31 +328,41 @@ int main(int argc, char** argv, char** envp)
                         GrbLogger::Logger.init(configManager->getIntOption(ConfigVal::SERVER_LOG_DEBUG_MODE));
 #endif
                         portnum = configManager->getUIntOption(ConfigVal::SERVER_PORT);
-                    } catch (const ConfigParseException& ce) {
-                        log_error("Error parsing config file '{}': {}", runtimeChild.getConfigFile().c_str(), ce.what());
+                    } catch (const ConfigParseException& ex) {
+                        log_error("Error parsing config file '{}': {}", runtimeChild.getConfigFile().c_str(), ex.what());
                         log_error("Could not restart Gerbera");
                         // at this point upnp shutdown has already been called
                         // therefore it is safe to exit
                         runtime.exit(EXIT_FAILURE);
-                    } catch (const std::runtime_error& e) {
-                        log_error("Error reloading configuration: {}", e.what());
+                    } catch (const std::runtime_error& ex) {
+                        log_error("Runtime error reloading configuration: {}", ex.what());
+                        runtime.exit(EXIT_FAILURE);
+                    } catch (const std::logic_error& ex) {
+                        log_error("Logic error reloading configuration: {}", ex.what());
                         runtime.exit(EXIT_FAILURE);
                     }
 
                     installSignalHandler();
 
                     server = std::make_shared<Server>(configManager);
-                    server->init(runtimeChild.getOffline());
+                    server->init(definition, runtimeChild.getOffline());
                     server->run();
 
                     _ctx.restartFlag = 0;
-                } catch (const std::runtime_error& e) {
+                } catch (const std::runtime_error& ex) {
                     _ctx.restartFlag = 0;
                     _ctx.shutdownFlag = 1;
 
                     if (pthread_sigmask(SIG_SETMASK, &emptyMaskSet, nullptr))
                         log_error("Error clearing masked signals {}", std::strerror(errno));
-                    log_error("Could not restart Gerbera {}", e.what());
+                    log_error("Could not restart Gerbera {}", ex.what());
+                } catch (const std::logic_error& ex) {
+                    _ctx.restartFlag = 0;
+                    _ctx.shutdownFlag = 1;
+
+                    if (pthread_sigmask(SIG_SETMASK, &emptyMaskSet, nullptr))
+                        log_error("Error clearing masked signals {}", std::strerror(errno));
+                    log_error("Restart: logic error {}", ex.what());
                 }
             }
         }
@@ -349,10 +374,13 @@ int main(int argc, char** argv, char** envp)
             server.reset();
             configManager.reset();
         } catch (const UpnpException& upnpE) {
-            log_error("main: upnp error {}", upnpE.getErrorCode());
+            log_error("Shutdown: upnp error {}", upnpE.getErrorCode());
             ret = EXIT_FAILURE;
-        } catch (const std::runtime_error& e) {
-            log_error("main: error {}", e.what());
+        } catch (const std::runtime_error& ex) {
+            log_error("Shutdown: runtime error {}", ex.what());
+            ret = EXIT_FAILURE;
+        } catch (const std::logic_error& ex) {
+            log_error("Shutdown: logic error {}", ex.what());
             ret = EXIT_FAILURE;
         }
 
