@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -43,6 +43,7 @@
 
 #ifdef HAVE_FFMPEG
 #define GRB_LOG_FAC GrbLogFacility::ffmpeg
+
 #include "ffmpeg_handler.h"
 
 #include "cds/cds_item.h"
@@ -54,6 +55,7 @@
 #include "util/tools.h"
 
 #include <algorithm>
+#include <cinttypes>
 #include <fmt/chrono.h>
 
 extern "C" {
@@ -61,6 +63,7 @@ extern "C" {
 #include <libavformat/avformat.h>
 #include <libavutil/avutil.h>
 #include <libavutil/display.h>
+#include <libavutil/pixdesc.h>
 
 } // extern "C"
 
@@ -85,6 +88,7 @@ public:
 private:
     FfmpegLogger(const FfmpegLogger&) = delete;
     FfmpegLogger& operator=(const FfmpegLogger&) = delete;
+
     static int printPrefix;
     static int logLevel;
 
@@ -133,34 +137,126 @@ int FfmpegLogger::printPrefix = 1;
 int FfmpegLogger::logLevel = AV_LOG_INFO;
 
 FfmpegHandler::FfmpegHandler(const std::shared_ptr<Context>& context)
-    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_FFMPEG_ENABLED, ConfigVal::IMPORT_LIBOPTS_FFMPEG_METADATA_TAGS_LIST, ConfigVal::IMPORT_LIBOPTS_FFMPEG_AUXDATA_TAGS_LIST)
+    : MediaMetadataHandler(context,
+          ConfigVal::IMPORT_LIBOPTS_FFMPEG_ENABLED,
+          ConfigVal::IMPORT_LIBOPTS_FFMPEG_METADATA_TAGS_LIST,
+          ConfigVal::IMPORT_LIBOPTS_FFMPEG_AUXDATA_TAGS_LIST,
+          ConfigVal::IMPORT_LIBOPTS_FFMPEG_COMMENT_ENABLED,
+          ConfigVal::IMPORT_LIBOPTS_FFMPEG_COMMENT_LIST)
 {
-    mappings = this->config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
 }
 
-void FfmpegHandler::addFfmpegAuxdataFields(const std::shared_ptr<CdsItem>& item, const std::shared_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const
+/// \brief Wrapper class to interface with FFMpeg
+class FfmpegObject {
+public:
+    std::string location;
+    std::shared_ptr<StringConverter> sc;
+    AVFormatContext* pFormatCtx = nullptr;
+
+    FfmpegObject(const std::shared_ptr<ConverterManager>& converterManager, const std::shared_ptr<CdsItem>& item)
+        : location(item->getLocation())
+        , sc(converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_FFMPEG_CHARSET, location))
+    {
+        // Open media file
+        if (avformat_open_input(&pFormatCtx, item->getLocation().c_str(), nullptr, nullptr) != 0) {
+            pFormatCtx = nullptr;
+            return; // Couldn't open file
+        }
+
+        // Retrieve stream information
+        if (avformat_find_stream_info(pFormatCtx, nullptr) < 0) {
+            avformat_close_input(&pFormatCtx);
+            pFormatCtx = nullptr;
+            return; // Couldn't find stream information
+        }
+    }
+    ~FfmpegObject()
+    {
+        // Close the media file
+        if (pFormatCtx)
+            avformat_close_input(&pFormatCtx);
+    }
+
+    /// \brief check if FFMpeg information is available
+    operator bool() const
+    {
+        return pFormatCtx && pFormatCtx->metadata;
+    }
+
+    /// \brief get key value from image
+    std::string getKey(const std::string& desiredTag) const
+    {
+        log_debug("key: {} ", desiredTag);
+        auto tag = av_dict_get(pFormatCtx->metadata, desiredTag.c_str(), nullptr, AV_DICT_IGNORE_SUFFIX);
+        if (tag && tag->value && tag->value[0]) {
+            auto [val, err] = sc->convert(trimString(tag->value));
+            if (!err.empty()) {
+                log_warning("{} {}: {}", location, desiredTag, err);
+            }
+            return val;
+        }
+        return "";
+    }
+};
+
+void FfmpegHandler::addFfmpegAuxdataFields(
+    const std::shared_ptr<CdsItem>& item,
+    const FfmpegObject& ffmpegObject) const
 {
-    if (!pFormatCtx->metadata) {
+    if (!ffmpegObject) {
         log_debug("no metadata");
         return;
     }
 
     for (auto&& desiredTag : auxTags) {
         if (!desiredTag.empty()) {
-            auto tag = av_dict_get(pFormatCtx->metadata, desiredTag.c_str(), nullptr, AV_DICT_IGNORE_SUFFIX);
-            if (tag && tag->value && tag->value[0]) {
-                log_debug("Added {}: {}", desiredTag.c_str(), tag->value);
-                item->setAuxData(desiredTag, sc->convert(tag->value));
+            auto val = ffmpegObject.getKey(desiredTag);
+            if (!val.empty()) {
+                log_debug("Added {}: {}", desiredTag.c_str(), val);
+                item->setAuxData(desiredTag, val);
             }
         }
     }
 } // addFfmpegAuxdataFields
 
-void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item, const std::shared_ptr<StringConverter>& sc, const AVFormatContext* pFormatCtx) const
+void FfmpegHandler::addFfmpegComment(
+    const std::shared_ptr<CdsItem>& item,
+    const FfmpegObject& ffmpegObject) const
 {
-    AVDictionaryEntry* e = nullptr;
+    if (!ffmpegObject) {
+        log_debug("no metadata");
+        return;
+    }
+    if (!isCommentEnabled)
+        return;
 
-    // only use ffmpeg meta data if not found by other handler
+    std::vector<std::string> snippets;
+    for (auto&& [label, desiredTag] : commentMap) {
+        if (!desiredTag.empty()) {
+            auto val = ffmpegObject.getKey(desiredTag);
+            if (!val.empty()) {
+                log_debug("Added {}: {}", label, val);
+                snippets.push_back(fmt::format("{}: {}", label, val));
+            }
+        }
+    }
+    if (!snippets.empty()) {
+        auto comment = fmt::format("{}", fmt::join(snippets, ", "));
+        log_debug("Fabricated Comment: {}", comment);
+        item->addMetaData(MetadataFields::M_DESCRIPTION, comment);
+    }
+}
+
+void FfmpegHandler::addFfmpegMetadataFields(
+    const std::shared_ptr<CdsItem>& item,
+    const FfmpegObject& ffmpegObject) const
+{
+    if (!ffmpegObject) {
+        log_debug("no metadata");
+        return;
+    }
+
+    // only use ffmpeg meta data if not found by other handlers
     auto emptyProperties = std::map<MetadataFields, bool>();
     for (auto&& prop : propertyMap) {
         emptyProperties[prop.first] = item->getMetaData(prop.first).empty();
@@ -170,14 +266,24 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
         emptySpecProperties[prop.second] = item->getMetaData(prop.second).empty();
     }
 
-    while ((e = av_dict_get(pFormatCtx->metadata, "", e, AV_DICT_IGNORE_SUFFIX))) {
-        std::string key = toLower(e->key);
-        std::string value = e->value;
+    AVDictionaryEntry* avEntry = nullptr;
+    while ((avEntry = av_dict_get(ffmpegObject.pFormatCtx->metadata, "", avEntry, AV_DICT_IGNORE_SUFFIX))) {
+        std::string key = toLower(avEntry->key);
+        std::string value = avEntry->value;
         log_debug("FFMpeg tag: {}: {}", key, value);
-        auto it = metaTags.find(e->key);
+        {
+            auto [val, err] = ffmpegObject.sc->convert(trimString(value));
+            if (!err.empty()) {
+                log_warning("{} {}: {}", ffmpegObject.location, key, err);
+            }
+            value = val;
+        }
+        auto it = metaTags.find(avEntry->key);
+        if (it == metaTags.end())
+            it = metaTags.find(key); // search lowercase string as well
         if (it != metaTags.end() && emptySpecProperties[it->second]) {
             log_debug("Identified special metadata '{}' as '{}': '{}'", it->first, it->second, value);
-            item->addMetaData(it->second, sc->convert(trimString(value)));
+            item->addMetaData(it->second, value);
             continue; // iterate while loop
         }
         auto pIt = std::find_if(propertyMap.begin(), propertyMap.end(),
@@ -190,16 +296,16 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
             if (emptyProperties[field]) {
                 if (field == MetadataFields::M_DATE || field == MetadataFields::M_CREATION_DATE) {
                     std::tm tmWork {};
-                    if (strptime(e->value, "%Y-%m-%dT%T.000000%Z", &tmWork)) {
+                    if (strptime(avEntry->value, "%Y-%m-%dT%T.000000%Z", &tmWork)) {
                         // convert creation_time to local time
                         auto utcTime = timegm(&tmWork);
                         if (utcTime == -1) {
                             continue;
                         }
                         tmWork = fmt::localtime(utcTime);
-                    } else if (strptime(e->value, "%Y-%m-%d", &tmWork)) {
+                    } else if (strptime(avEntry->value, "%Y-%m-%d", &tmWork)) {
                         ; // use the value as is
-                    } else if (strptime(e->value, "%Y", &tmWork)) {
+                    } else if (strptime(avEntry->value, "%Y", &tmWork)) {
                         // convert the value to "XXXX-01-01"
                         tmWork.tm_mon = 0; // Month (0-11)
                         tmWork.tm_mday = 1; // Day of the month (1-31)
@@ -207,8 +313,9 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
                         continue;
                     auto mDate = fmt::format("{:%Y-%m-%d}", tmWork);
                     item->addMetaData(field, mDate);
-                } else
-                    item->addMetaData(field, sc->convert(trimString(value)));
+                } else {
+                    item->addMetaData(field, value);
+                }
             }
             if (field == MetadataFields::M_TRACKNUMBER) {
                 item->setTrackNumber(stoiString(value));
@@ -221,6 +328,7 @@ void FfmpegHandler::addFfmpegMetadataFields(const std::shared_ptr<CdsItem>& item
     }
 }
 
+/// \brief Convert Rotaiton Informaion to 360 degree value
 static double get_rotation(const std::int32_t* displaymatrix)
 {
     double rot = 0;
@@ -236,24 +344,31 @@ static double get_rotation(const std::int32_t* displaymatrix)
 }
 
 // ffmpeg library calls
-void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item, const AVFormatContext* pFormatCtx)
+void FfmpegHandler::addFfmpegResourceFields(
+    const std::shared_ptr<CdsItem>& item,
+    const FfmpegObject& ffmpegObject)
 {
+    if (!ffmpegObject) {
+        log_debug("no metadata");
+        return;
+    }
+
     auto resource = item->getResource(ContentHandler::DEFAULT);
     bool isAudioFile = item->isSubClass(UPNP_CLASS_AUDIO_ITEM) && item->getResource(ResourcePurpose::Thumbnail);
     auto resource2 = isAudioFile ? item->getResource(ResourcePurpose::Thumbnail) : item->getResource(ContentHandler::DEFAULT);
 
     // duration
-    if (pFormatCtx->duration > 0) {
-        auto duration = millisecondsToHMSF(pFormatCtx->duration / (AV_TIME_BASE / 1000));
+    if (ffmpegObject.pFormatCtx->duration > 0) {
+        auto duration = millisecondsToHMSF(ffmpegObject.pFormatCtx->duration / (AV_TIME_BASE / 1000));
         log_debug("Added duration: {}", duration);
         resource->addAttribute(ResourceAttribute::DURATION, duration);
     }
 
     // bitrate
-    if (pFormatCtx->bit_rate > 0) {
+    if (ffmpegObject.pFormatCtx->bit_rate > 0) {
         // ffmpeg's bit_rate is in bits/sec, upnp wants it in bytes/sec
         // See http://www.upnp.org/schemas/av/didl-lite-v3.xsd
-        auto bitrate = pFormatCtx->bit_rate / 8;
+        auto bitrate = ffmpegObject.pFormatCtx->bit_rate / 8;
         log_debug("Added bitrate: {} kb/s", bitrate);
         resource->addAttribute(ResourceAttribute::BITRATE, fmt::to_string(bitrate));
     }
@@ -261,8 +376,8 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
     // video resolution, audio sampling rate, nr of audio channels
     bool audioSet = false;
     bool videoSet = false;
-    for (std::size_t i = 0; i < pFormatCtx->nb_streams; i++) {
-        auto st = pFormatCtx->streams[i];
+    for (std::size_t i = 0; i < ffmpegObject.pFormatCtx->nb_streams; i++) {
+        auto st = ffmpegObject.pFormatCtx->streams[i];
 
         if (st && !videoSet && as_codecpar(st)->codec_type == AVMEDIA_TYPE_VIDEO) {
             // orientation
@@ -274,11 +389,23 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
                     rot = stoiString(entry->value);
                     log_debug("{} = {}", "rotate", rot);
                 } else {
+#if (LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(60, 0, 0))
                     auto displaymatrix = av_stream_get_side_data(st, AV_PKT_DATA_DISPLAYMATRIX, nullptr);
                     if (displaymatrix) {
                         rot = get_rotation(reinterpret_cast<std::int32_t*>(displaymatrix));
                         log_debug("{} = {}", "displaymatrix", rot);
                     }
+#else
+                    int32_t* displayMatrix = nullptr;
+                    auto psd = av_packet_side_data_get(as_codecpar(st)->coded_side_data,
+                        as_codecpar(st)->nb_coded_side_data, AV_PKT_DATA_DISPLAYMATRIX);
+                    if (psd)
+                        displayMatrix = reinterpret_cast<std::int32_t*>(psd->data);
+                    if (displayMatrix) {
+                        rot = get_rotation(displayMatrix);
+                        log_debug("{} = {}", "displayMatrix", rot);
+                    }
+#endif
                 }
                 int orientation = 0;
                 if (rot == 0) {
@@ -320,6 +447,19 @@ void FfmpegHandler::addFfmpegResourceFields(const std::shared_ptr<CdsItem>& item
                 resource2->addAttribute(ResourceAttribute::RESOLUTION, resolution);
                 videoSet = true;
             }
+            // pixelformat
+            {
+                AVPixelFormat pix_fmt = static_cast<AVPixelFormat>(as_codecpar(st)->format);
+
+                // Get pixel format name
+                auto pix_fmt_name = av_get_pix_fmt_name(pix_fmt);
+                if (pix_fmt_name) {
+                    resource2->addAttribute(ResourceAttribute::PIXELFORMAT, pix_fmt_name);
+                    log_debug("Pixel Format: {}", pix_fmt_name);
+                } else {
+                    log_debug("Unknown Pixel Format");
+                }
+            }
         }
         if (st && !audioSet && as_codecpar(st)->codec_type == AVMEDIA_TYPE_AUDIO) {
             auto codecId = as_codecpar(st)->codec_id;
@@ -369,39 +509,34 @@ void FfmpegHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 
     log_debug("Running ffmpeg handler on {}", item->getLocation().c_str());
 
-    AVFormatContext* pFormatCtx = nullptr;
-
 #if (LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58, 9, 100))
     // Register all formats and codecs
     av_register_all();
 #endif
-    // Open video file
-    if (avformat_open_input(&pFormatCtx, item->getLocation().c_str(), nullptr, nullptr) != 0)
-        return; // Couldn't open file
+    FfmpegObject ffmpegObject(converterManager, item);
 
-    // Retrieve stream information
-    if (avformat_find_stream_info(pFormatCtx, nullptr) < 0) {
-        avformat_close_input(&pFormatCtx);
-        return; // Couldn't find stream information
-    }
-    auto sc = converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_FFMPEG_CHARSET, item->getLocation());
-    // Add metadata using ffmpeg library calls
-    addFfmpegMetadataFields(item, sc, pFormatCtx);
+    // Add metadata for unset values
+    addFfmpegMetadataFields(item, ffmpegObject);
     // Add auxdata
-    addFfmpegAuxdataFields(item, sc, pFormatCtx);
-    // Add resources using ffmpeg library calls
-    addFfmpegResourceFields(item, pFormatCtx);
-    // Close the video file
-    avformat_close_input(&pFormatCtx);
+    addFfmpegAuxdataFields(item, ffmpegObject);
+    // Add special resource properties
+    addFfmpegResourceFields(item, ffmpegObject);
+    // Fabricate comment
+    if (item->getMetaData(MetadataFields::M_DESCRIPTION).empty())
+        addFfmpegComment(item, ffmpegObject);
 }
 
-std::unique_ptr<IOHandler> FfmpegHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
+std::unique_ptr<IOHandler> FfmpegHandler::serveContent(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsResource>& resource)
 {
+    // nothing to serve here, yet
     return nullptr;
 }
 
 std::string FfmpegHandler::getMimeType() const
 {
-    return getValueOrDefault(mappings, CONTENT_TYPE_JPG, "image/jpeg");
+    return getValueOrDefault(mimeContentTypeMappings, CONTENT_TYPE_JPG, "image/jpeg");
 }
+
 #endif // HAVE_FFMPEG
