@@ -4,7 +4,7 @@
 
     matroska_handler.cc - this file is part of Gerbera.
 
-    Copyright (C) 2019-2024 Gerbera Contributors
+    Copyright (C) 2019-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -49,6 +49,8 @@
 #include <matroska/KaxContexts.h>
 #include <matroska/KaxSeekHead.h>
 #include <matroska/KaxSegment.h>
+#include <matroska/KaxTag.h>
+#include <matroska/KaxTags.h>
 
 using namespace libebml;
 using namespace libmatroska;
@@ -107,7 +109,12 @@ public:
 };
 
 MatroskaHandler::MatroskaHandler(const std::shared_ptr<Context>& context)
-    : MediaMetadataHandler(context, ConfigVal::IMPORT_LIBOPTS_MKV_ENABLED)
+    : MediaMetadataHandler(context,
+          ConfigVal::IMPORT_LIBOPTS_MKV_ENABLED,
+          ConfigVal::IMPORT_LIBOPTS_MKV_METADATA_TAGS_LIST,
+          ConfigVal::IMPORT_LIBOPTS_MKV_AUXDATA_TAGS_LIST,
+          ConfigVal::IMPORT_LIBOPTS_MKV_COMMENT_ENABLED,
+          ConfigVal::IMPORT_LIBOPTS_MKV_COMMENT_LIST)
 {
 }
 
@@ -120,7 +127,9 @@ void MatroskaHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
     parseMKV(item, nullptr);
 }
 
-std::unique_ptr<IOHandler> MatroskaHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
+std::unique_ptr<IOHandler> MatroskaHandler::serveContent(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::shared_ptr<CdsResource>& resource)
 {
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (!item)
@@ -139,7 +148,9 @@ std::unique_ptr<IOHandler> MatroskaHandler::serveContent(const std::shared_ptr<C
 
 #define GRB_MATROSKA_INFO (GRB_MATROSKA_TITLE | GRB_MATROSKA_DATE | GRB_MATROSKA_DURATION)
 
-void MatroskaHandler::parseMKV(const std::shared_ptr<CdsItem>& item, std::unique_ptr<MemIOHandler>* pIoHandler)
+void MatroskaHandler::parseMKV(
+    const std::shared_ptr<CdsItem>& item,
+    std::unique_ptr<MemIOHandler>* pIoHandler)
 {
     auto ebmlFile = FileIOCallback(item->getLocation().c_str());
     auto ebmlStream = EbmlStream(ebmlFile);
@@ -152,7 +163,7 @@ void MatroskaHandler::parseMKV(const std::shared_ptr<CdsItem>& item, std::unique
         iUpperLevel = 0;
         EbmlElement* elL1;
         while ((elL1 = ebmlStream.FindNextElement(EBML_CONTEXT(elL0), iUpperLevel, ~0, true))) {
-            parseLevel1Element(item, ebmlFile, ebmlStream, elL1, pIoHandler);
+            parseSegmentContent(item, ebmlFile, ebmlStream, elL1, pIoHandler);
 
             delete (elL1->SkipData(ebmlStream, EBML_CONTEXT(elL1)));
             delete elL1;
@@ -167,9 +178,52 @@ void MatroskaHandler::parseMKV(const std::shared_ptr<CdsItem>& item, std::unique
     } // while elementLevel0
 
     ebmlFile.close();
+
+    std::vector<std::string> snippets;
+    for (auto&& mkv : mkvTags) {
+        auto mkvTag = mkv.first;
+        auto mkvValue = mkv.second;
+        // if there are any metadata tags that the user wants - add them
+        auto meta = metaTags.find(mkvTag);
+        if (meta != metaTags.end() && !mkvValue.empty()) {
+            item->addMetaData(metaTags.at(mkvTag), mkvValue);
+            log_debug("Adding tag '{}' as '{}' with value '{}'", mkvTag, metaTags.at(mkvTag), mkvValue);
+        }
+        // if there are any auxilary tags that the user wants - add them
+        auto aux = std::find(auxTags.begin(), auxTags.end(), mkvTag);
+        if (aux != auxTags.end() && !mkvValue.empty()) {
+            item->setAuxData(mkvTag, mkvValue);
+            log_debug("Adding aux: {} with value {}", mkvTag, mkvValue);
+        }
+        // if user want a fabricated comment
+        auto comment = std::find_if(commentMap.begin(), commentMap.end(), [&](auto&& c) { return c.second == mkvTag; });
+        if (comment != commentMap.end()) {
+            log_debug("Added comment {}: {}", comment->second, mkvValue);
+            snippets.push_back(fmt::format("{}: {}", comment->second, mkvValue));
+        }
+    }
+    if (item->getMetaData(MetadataFields::M_DESCRIPTION).empty()) {
+        auto meta = metaTags.find("COMMENT");
+        std::string description;
+        if (meta != metaTags.end()) {
+            description = meta->second;
+        }
+        if (description.empty() && isCommentEnabled) {
+            description = fmt::format("{}", fmt::join(snippets, ", "));
+            log_debug("Fabricated Comment: {}", description);
+        }
+        if (!description.empty()) {
+            item->addMetaData(MetadataFields::M_DESCRIPTION, description);
+        }
+    }
 }
 
-void MatroskaHandler::parseLevel1Element(const std::shared_ptr<CdsItem>& item, IOCallback& ebmlFile, EbmlStream& ebmlStream, EbmlElement* elL1, std::unique_ptr<MemIOHandler>* pIoHandler)
+void MatroskaHandler::parseSegmentContent(
+    const std::shared_ptr<CdsItem>& item,
+    IOCallback& ebmlFile,
+    EbmlStream& ebmlStream,
+    EbmlElement* elL1,
+    std::unique_ptr<MemIOHandler>* pIoHandler)
 {
     // Looking at just at EbmlId is not reliable since it can be a dummy element.
     if (!elL1->IsMaster())
@@ -180,12 +234,15 @@ void MatroskaHandler::parseLevel1Element(const std::shared_ptr<CdsItem>& item, I
         return;
     }
     log_debug("MatroskaHandler found {}", EbmlId(*master).GetValue());
-    log_debug("MatroskaHandler KaxSeekHead {}", EBML_ID(KaxSeekHead).GetValue());
-    log_debug("MatroskaHandler KaxInfo {}", EBML_ID(KaxInfo).GetValue());
-    log_debug("MatroskaHandler KaxAttachments {}", EBML_ID(KaxAttachments).GetValue());
+    log_debug("MatroskaHandler SeekHead {}", EBML_ID(KaxSeekHead).GetValue());
+    log_debug("MatroskaHandler Info {}", EBML_ID(KaxInfo).GetValue());
+    log_debug("MatroskaHandler Attachments {}", EBML_ID(KaxAttachments).GetValue());
+    log_debug("MatroskaHandler Tags {}", EBML_ID(KaxTags).GetValue());
     if (EbmlId(*master) == EBML_ID(KaxSeekHead)) {
         parseHead(item, ebmlFile, ebmlStream, master, pIoHandler);
         activeFlag = 0; // If there is a head we assume that info and attachments are handled
+    } else if (EbmlId(*master) == EBML_ID(KaxTags)) {
+        parseTags(item, ebmlStream, master);
     } else if (EbmlId(*master) == EBML_ID(KaxInfo)) {
         parseInfo(item, ebmlStream, master);
     } else if (EbmlId(*master) == EBML_ID(KaxAttachments)) {
@@ -194,7 +251,12 @@ void MatroskaHandler::parseLevel1Element(const std::shared_ptr<CdsItem>& item, I
 }
 
 // This code is inspired by https://github.com/TypesettingTools/DirectASS/blob/master/DSlibass/MatroskaParser.cpp#L58+
-void MatroskaHandler::parseHead(const std::shared_ptr<CdsItem>& item, IOCallback& ebmlFile, EbmlStream& ebmlStream, EbmlMaster* info, std::unique_ptr<MemIOHandler>* pIoHandler)
+void MatroskaHandler::parseHead(
+    const std::shared_ptr<CdsItem>& item,
+    IOCallback& ebmlFile,
+    EbmlStream& ebmlStream,
+    EbmlMaster* info,
+    std::unique_ptr<MemIOHandler>* pIoHandler)
 {
     auto metaSeek = dynamic_cast<KaxSeekHead*>(info);
     if (!metaSeek)
@@ -240,14 +302,16 @@ void MatroskaHandler::parseHead(const std::shared_ptr<CdsItem>& item, IOCallback
 #endif
             auto segmentPosition = static_cast<EbmlMaster*>(ebmlHead)->GetDataStart() + static_cast<std::uint16_t>(*seekPos);
             log_debug("parseHead: Seek ID {} at {}", seekIdValue.GetValue(), segmentPosition);
-            if (seekPoint->IsEbmlId(EBML_ID(KaxAttachments)) || seekPoint->IsEbmlId(EBML_ID(KaxInfo))) {
+            if (seekPoint->IsEbmlId(EBML_ID(KaxAttachments))
+                || seekPoint->IsEbmlId(EBML_ID(KaxTags))
+                || seekPoint->IsEbmlId(EBML_ID(KaxInfo))) {
                 ebmlFile.setFilePointer(segmentPosition);
                 auto attStream = EbmlStream(ebmlFile);
                 int upperLvlElement = 0;
                 auto level1 = attStream.FindNextElement(EBML_CLASS_CONTEXT(KaxSegment), upperLvlElement, ~0, true, 1);
 
                 if (EbmlId(*level1) == seekIdValue) {
-                    parseLevel1Element(item, ebmlFile, attStream, level1, pIoHandler);
+                    parseSegmentContent(item, ebmlFile, attStream, level1, pIoHandler);
                 }
                 delete level1;
             }
@@ -259,15 +323,129 @@ void MatroskaHandler::parseHead(const std::shared_ptr<CdsItem>& item, IOCallback
     delete ebmlHead;
 }
 
-void MatroskaHandler::parseInfo(const std::shared_ptr<CdsItem>& item, EbmlStream& ebmlStream, EbmlMaster* info)
+class MkvObject {
+public:
+    std::string location;
+    std::shared_ptr<StringConverter> sc;
+
+    MkvObject(
+        const std::shared_ptr<ConverterManager>& converterManager,
+        const std::shared_ptr<CdsItem>& item,
+        EbmlStream& ebmlStream,
+        EbmlMaster* info)
+        : location(item->getLocation())
+        , sc(converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_MKV_CHARSET, location))
+    {
+        EbmlElement* dummyEl;
+        int iUpperLevel = 0;
+
+        // master elements
+        info->Read(ebmlStream, EBML_CONTEXT(info), iUpperLevel, dummyEl, true);
+    }
+    template <class MkvKey>
+    std::string getDate(EbmlElement* el)
+    {
+        MkvKey* dateEl = dynamic_cast<MkvKey*>(el);
+        if (!dateEl)
+            return "";
+
+        auto fDate = fmt::format("{:%FT%T%z}", fmt::gmtime(dateEl->GetEpochDate()));
+        if (!fDate.empty()) {
+            auto [val, err] = sc->convert(fDate);
+            if (!err.empty()) {
+                log_warning("{}: {}", location, err);
+                return "";
+            }
+            return val;
+        }
+        return "";
+    }
+    template <class MkvKey>
+    std::string getString(EbmlElement* el)
+    {
+        MkvKey* elValue = dynamic_cast<MkvKey*>(el);
+        if (!elValue)
+            return "";
+
+        auto [val, err] = sc->convert(UTFstring(*elValue).GetUTF8());
+        if (!err.empty()) {
+            log_warning("{}: {}", location, err);
+            return "";
+        }
+        return val;
+    }
+    template <class MkvKey>
+    std::string getTime(EbmlElement* el)
+    {
+        auto durationEl = dynamic_cast<MkvKey*>(el);
+        if (!durationEl)
+            return "";
+
+        auto fDuration = millisecondsToHMSF(static_cast<double>(*static_cast<EbmlFloat*>(durationEl)));
+        auto [val, err] = sc->convert(fDuration);
+        if (!err.empty()) {
+            log_warning("{}: {}", location, err);
+            return "";
+        }
+        return val;
+    }
+};
+
+void MatroskaHandler::parseTags(
+    const std::shared_ptr<CdsItem>& item,
+    EbmlStream& ebmlStream,
+    EbmlMaster* info)
 {
-    EbmlElement* dummyEl;
-    int iUpperLevel = 0;
+    MkvObject mkvObject(converterManager, item, ebmlStream, info);
+    for (auto&& el : *info) {
+        if (EbmlId(*el) == EBML_ID(KaxTag)) {
+            auto tagEl = dynamic_cast<KaxTag*>(el);
+            if (!tagEl)
+                continue;
+            for (auto&& tagEln : *tagEl) {
+                if (EbmlId(*tagEln) == EBML_ID(KaxTagTargets)) {
+                    auto tagTargets = dynamic_cast<KaxTagTargets*>(tagEln);
+                    for (auto&& tagTarget : *tagTargets) {
+                        if (EbmlId(*tagTarget) == EBML_ID(KaxTagTargetType)) {
+                            log_debug("{} target type", mkvObject.location);
+                        } else if (EbmlId(*tagTarget) == EBML_ID(KaxTagTargetTypeValue)) {
+                            log_debug("{} targeti type value", mkvObject.location);
+                        } else if (EbmlId(*tagTarget) == EBML_ID(KaxTagTrackUID)) {
+                            log_debug("{} target track", mkvObject.location);
+                        } else if (EbmlId(*tagTarget) == EBML_ID(KaxTagChapterUID)) {
+                            log_debug("{} target chapter", mkvObject.location);
+                        } else if (EbmlId(*tagTarget) == EBML_ID(KaxTagEditionUID)) {
+                            log_debug("{} target edition", mkvObject.location);
+                        } else if (EbmlId(*tagTarget) == EBML_ID(KaxTagAttachmentUID)) {
+                            log_debug("{} target attachment", mkvObject.location);
+                        }
+                    }
+                } else if (EbmlId(*tagEln) == EBML_ID(KaxTagSimple)) {
+                    auto tagElS = dynamic_cast<KaxTagSimple*>(tagEln);
+                    std::string name;
+                    std::string value;
+                    for (auto&& tagElv : *tagElS) {
+                        if (EbmlId(*tagElv) == EBML_ID(KaxTagName)) {
+                            name = mkvObject.getString<KaxTagName>(tagElv);
+                        } else if (EbmlId(*tagElv) == EBML_ID(KaxTagString)) {
+                            value = mkvObject.getString<KaxTagString>(tagElv);
+                        }
+                    }
+                    log_debug("{} tag {} = {}", item->getLocation().string(), name, value);
+                    if (!name.empty())
+                        mkvTags[name] = value;
+                }
+            }
+        }
+    }
+}
 
-    // master elements
-    info->Read(ebmlStream, EBML_CONTEXT(info), iUpperLevel, dummyEl, true);
-
-    auto sc = converterManager->m2i(ConfigVal::IMPORT_LIBOPTS_MKV_CHARSET, item->getLocation());
+void MatroskaHandler::parseInfo(
+    const std::shared_ptr<CdsItem>& item,
+    EbmlStream& ebmlStream,
+    EbmlMaster* info)
+{
+    MkvObject mkvObject(converterManager, item, ebmlStream, info);
     for (auto&& el : *info) {
         if (EbmlId(*el) == EBML_ID(KaxTitle)) {
             auto titleEl = dynamic_cast<KaxTitle*>(el);
@@ -275,9 +453,9 @@ void MatroskaHandler::parseInfo(const std::shared_ptr<CdsItem>& item, EbmlStream
                 log_error("Malformed MKV file; KaxTitle cast failed!");
                 continue;
             }
-            auto title = std::string(UTFstring(*titleEl).GetUTF8());
+            auto title = mkvObject.getString<KaxTitle>(el);
             log_debug("KaxTitle = {}", title);
-            item->addMetaData(MetadataFields::M_TITLE, sc->convert(title));
+            item->addMetaData(MetadataFields::M_TITLE, title);
             activeFlag &= ~GRB_MATROSKA_TITLE;
         } else if (EbmlId(*el) == EBML_ID(KaxDateUTC)) {
             auto dateEl = dynamic_cast<KaxDateUTC*>(el);
@@ -285,11 +463,11 @@ void MatroskaHandler::parseInfo(const std::shared_ptr<CdsItem>& item, EbmlStream
                 log_error("Malformed MKV file; KaxDateUTC cast failed!");
                 continue;
             }
-            auto fDate = fmt::format("{:%FT%T%z}", fmt::gmtime(dateEl->GetEpochDate()));
+            auto fDate = mkvObject.getDate<KaxDateUTC>(el);
             if (!fDate.empty()) {
                 log_debug("KaxDateUTC = {}", fDate);
-                item->addMetaData(MetadataFields::M_DATE, sc->convert(fDate));
-                item->addMetaData(MetadataFields::M_CREATION_DATE, sc->convert(fDate));
+                item->addMetaData(MetadataFields::M_DATE, fDate);
+                item->addMetaData(MetadataFields::M_CREATION_DATE, fDate);
                 activeFlag &= ~GRB_MATROSKA_DATE;
             }
         } else if (EbmlId(*el) == EBML_ID(KaxDuration)) {
@@ -298,10 +476,10 @@ void MatroskaHandler::parseInfo(const std::shared_ptr<CdsItem>& item, EbmlStream
                 log_error("Malformed MKV file; KaxDuration cast failed!");
                 continue;
             }
-            auto fDuration = millisecondsToHMSF(static_cast<double>(*static_cast<EbmlFloat*>(durationEl)));
+            auto fDuration = mkvObject.getTime<KaxDuration>(el);
             log_debug("KaxDuration = {}", fDuration);
-            if (item->getResourceCount() > 0) {
-                item->getResource(ContentHandler::DEFAULT)->addAttribute(ResourceAttribute::DURATION, sc->convert(fDuration));
+            if (!fDuration.empty() && item->getResourceCount() > 0) {
+                item->getResource(ContentHandler::DEFAULT)->addAttribute(ResourceAttribute::DURATION, fDuration);
             }
             activeFlag &= ~GRB_MATROSKA_DURATION;
         }
@@ -310,7 +488,11 @@ void MatroskaHandler::parseInfo(const std::shared_ptr<CdsItem>& item, EbmlStream
     }
 }
 
-void MatroskaHandler::parseAttachments(const std::shared_ptr<CdsItem>& item, EbmlStream& ebmlStream, EbmlMaster* attachments, std::unique_ptr<MemIOHandler>* pIoHandler)
+void MatroskaHandler::parseAttachments(
+    const std::shared_ptr<CdsItem>& item,
+    EbmlStream& ebmlStream,
+    EbmlMaster* attachments,
+    std::unique_ptr<MemIOHandler>* pIoHandler)
 {
     EbmlElement* dummyEl;
     int iUpperLevel = 0;
@@ -355,7 +537,9 @@ std::string MatroskaHandler::getContentTypeFromByteVector(const KaxFileData& dat
 #endif
 }
 
-void MatroskaHandler::addArtworkResource(const std::shared_ptr<CdsItem>& item, const std::string& artMimetype)
+void MatroskaHandler::addArtworkResource(
+    const std::shared_ptr<CdsItem>& item,
+    const std::string& artMimetype)
 {
     // if we could not determine the mimetype, then there is no
     // point to add the resource - it's probably garbage
