@@ -4,7 +4,7 @@
 
     metacontent_handler.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2022 Gerbera Contributors
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -22,36 +22,47 @@
 */
 
 /// \file metacontent_handler.cc
+#define GRB_LOG_FAC GrbLogFacility::metadata
 
 #include "metacontent_handler.h" // API
 
+#include "cds/cds_objects.h"
+#include "config/config.h"
+#include "config/config_definition.h"
+#include "config/config_val.h"
+#include "config/result/directory_tweak.h"
+#include "content/content.h"
+#include "context.h"
+#include "iohandler/file_io_handler.h"
+#include "util/mime.h"
+#include "util/string_converter.h"
+#include "util/tools.h"
+
 #include <array>
 #include <regex>
 #include <sys/stat.h>
 
-#include "cds_objects.h"
-#include "config/config.h"
-#include "config/directory_tweak.h"
-#include "iohandler/file_io_handler.h"
-#include "util/mime.h"
-#include "util/tools.h"
-
-ContentPathSetup::ContentPathSetup(std::shared_ptr<Config> config, config_option_t fileListOption, config_option_t dirListOption)
+ContentPathSetup::ContentPathSetup(std::shared_ptr<Config> config, const std::shared_ptr<ConfigDefinition>& definition, ConfigVal fileListOption, ConfigVal dirListOption)
     : config(std::move(config))
     , names(this->config->getArrayOption(fileListOption))
-    , patterns(this->config->getDictionaryOption(dirListOption))
-    , allTweaks(this->config->getDirectoryTweakOption(CFG_IMPORT_DIRECTORIES_LIST))
-    , caseSensitive(this->config->getBoolOption(CFG_IMPORT_RESOURCES_CASE_SENSITIVE))
+    , patterns(this->config->getVectorOption(dirListOption))
+    , allTweaks(this->config->getDirectoryTweakOption(ConfigVal::IMPORT_DIRECTORIES_LIST))
+    , caseSensitive(this->config->getBoolOption(ConfigVal::IMPORT_RESOURCES_CASE_SENSITIVE))
+    , definition(definition)
 {
 }
 
 std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<CdsObject>& obj, const std::string& setting, fs::path folder) const
 {
     auto objLocation = obj->getLocation();
-    auto tweak = allTweaks->get(objLocation);
+    auto tweak = allTweaks->getKey(objLocation);
     auto files = !tweak || !tweak->hasSetting(setting) ? this->names : std::vector<std::string> { tweak->getSetting(setting) };
     auto isCaseSensitive = tweak && tweak->hasCaseSensitive() ? tweak->getCaseSensitive() : this->caseSensitive;
 
+    static auto nameOption = definition->removeAttribute(ConfigVal::A_IMPORT_RESOURCES_NAME);
+    static auto extOption = definition->removeAttribute(ConfigVal::A_IMPORT_RESOURCES_EXT);
+    static auto pttOption = definition->removeAttribute(ConfigVal::A_IMPORT_RESOURCES_PTT);
+
     std::vector<fs::path> result;
 
     if (!files.empty()) {
@@ -61,6 +72,7 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
         log_debug("Folder name: {}", folder.c_str());
 
         if (isCaseSensitive) {
+            // directly search files using name
             for (auto&& name : files) {
                 auto contentFile = folder / expandName(name, obj);
                 std::error_code ec;
@@ -73,10 +85,12 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
         } else {
             std::map<std::string, fs::path> fileNames;
             std::error_code ec;
+            // get all files in lowercase
             for (auto&& p : fs::directory_iterator(folder, ec))
                 if (isRegularFile(p, ec) && p.path() != objLocation)
                     fileNames[toLower(p.path().filename().string())] = p;
 
+            // filter files matching filenames
             for (auto&& name : files) {
                 auto fileName = toLower(expandName(name, obj));
                 for (auto&& [f, s] : fileNames) {
@@ -88,9 +102,22 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
             }
         }
         if (!patterns.empty()) {
-            for (auto&& [dir, ext] : patterns) {
+            // filter files matching patterns
+            for (auto&& pattern : patterns) {
+                std::string dir;
+                std::string ext;
+                std::string ptt;
+                for (auto&& [key, val] : pattern) {
+                    if (key == nameOption)
+                        dir = val;
+                    else if (key == extOption)
+                        ext = val;
+                    else if (key == pttOption)
+                        ptt = val;
+                }
                 auto contentPath = fs::path(expandName(dir, obj));
                 auto extn = fs::path(expandName(ext, obj));
+                ptt = expandName(ptt, obj);
                 auto stem = isCaseSensitive ? extn.stem().string() : toLower(extn.stem().string());
                 if (extn.has_extension()) {
                     extn = isCaseSensitive ? extn.extension().string() : toLower(extn.extension().string());
@@ -98,6 +125,9 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
                     extn = fmt::format(".{}", isCaseSensitive ? ext : toLower(ext));
                     stem.clear();
                 }
+                if (!ptt.empty()) {
+                    stem = fmt::format("{}{}", ptt, stem);
+                }
                 std::error_code ec;
                 if (contentPath.is_relative()) {
                     contentPath = fs::weakly_canonical(folder / contentPath);
@@ -106,14 +136,16 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
                     log_debug("{}: not a directory", contentPath.string());
                     continue;
                 }
+                // Check files using patterns
                 for (auto&& contentFile : fs::directory_iterator(contentPath, ec)) {
-                    if (isRegularFile(contentFile, ec) && ((isCaseSensitive && contentFile.path().extension() == extn) || (!isCaseSensitive && toLower(contentFile.path().extension().string()) == extn)) && contentFile.path() != objLocation) {
+                    if (isRegularFile(contentFile, ec)
+                        && (ext.empty() || (isCaseSensitive && contentFile.path().extension() == extn) || (!isCaseSensitive && toLower(contentFile.path().extension().string()) == extn))
+                        && contentFile.path() != objLocation) {
                         if (!stem.empty()) {
+                            replaceAllString(stem, "?", ".");
                             replaceAllString(stem, "*", ".*");
-                            replaceAllString(stem, ".", "?");
                             auto re = std::regex(fmt::format("^{}$", stem));
-                            std::cmatch m;
-                            if (std::regex_match(contentFile.path().stem().string().c_str(), m, re)) {
+                            if (std::regex_match(contentFile.path().stem().string(), re)) {
                                 log_debug("{}: found", contentFile.path().string());
                                 result.push_back(contentFile.path());
                             }
@@ -131,18 +163,24 @@ std::vector<fs::path> ContentPathSetup::getContentPath(const std::shared_ptr<Cds
     return result;
 }
 
-static constexpr auto metaTags = std::array {
-    std::pair("%album%", M_ALBUM),
-    std::pair("%albumArtist%", M_ALBUMARTIST),
-    std::pair("%artist%", M_ARTIST),
-    std::pair("%genre%", M_GENRE),
-    std::pair("%title%", M_TITLE),
-    std::pair("%composer%", M_COMPOSER),
+static constexpr std::array metaTags {
+    std::pair("%album%", MetadataFields::M_ALBUM),
+    std::pair("%albumArtist%", MetadataFields::M_ALBUMARTIST),
+    std::pair("%artist%", MetadataFields::M_ARTIST),
+    std::pair("%genre%", MetadataFields::M_GENRE),
+    std::pair("%title%", MetadataFields::M_TITLE),
+    std::pair("%composer%", MetadataFields::M_COMPOSER),
 };
 
-std::string ContentPathSetup::expandName(std::string_view name, const std::shared_ptr<CdsObject>& obj)
+std::string ContentPathSetup::expandName(const std::string& name, const std::shared_ptr<CdsObject>& obj)
 {
     std::string copy(name);
+    auto path = (obj->isItem()) ? obj->getLocation().parent_path().string() : obj->getLocation().string();
+    if (name.empty())
+        return path;
+    else if (name.at(0) == '.') {
+        return copy.replace(0, 1, path);
+    }
 
     for (auto&& [key, val] : metaTags)
         replaceString(copy, key, obj->getMetaData(val));
@@ -163,11 +201,18 @@ std::string ContentPathSetup::expandName(std::string_view name, const std::share
 
 std::unique_ptr<ContentPathSetup> FanArtHandler::setup {};
 
+MetacontentHandler::MetacontentHandler(const std::shared_ptr<Context>& context)
+    : MetadataHandler(context)
+    , f2i(context->getConverterManager()->f2i())
+    , definition(context->getDefinition())
+{
+}
+
 FanArtHandler::FanArtHandler(const std::shared_ptr<Context>& context)
     : MetacontentHandler(context)
 {
     if (!setup) {
-        setup = std::make_unique<ContentPathSetup>(config, CFG_IMPORT_RESOURCES_FANART_FILE_LIST, CFG_IMPORT_RESOURCES_FANART_DIR_LIST);
+        setup = std::make_unique<ContentPathSetup>(config, definition, ConfigVal::IMPORT_RESOURCES_FANART_FILE_LIST, ConfigVal::IMPORT_RESOURCES_FANART_DIR_LIST);
     }
 }
 
@@ -181,20 +226,22 @@ void FanArtHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 
     for (auto&& path : pathList) {
         if (!path.empty()) {
-            auto resource = std::make_shared<CdsResource>(ContentHandler::FANART, CdsResource::Purpose::Thumbnail);
+            auto resource = std::make_shared<CdsResource>(ContentHandler::FANART, ResourcePurpose::Thumbnail);
             std::string type = path.extension().string().substr(1);
-            std::string mimeType = mime->getMimeType(path, fmt::format("image/{}", type));
+            auto mimeType = std::get<1>(mime->getMimeType(path, fmt::format("image/{}", type)));
 
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
-            resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, path.string());
+            if (!mimeType.empty()) {
+                resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
+                resource->addAttribute(ResourceAttribute::RESOURCE_FILE, path.string());
+            }
             obj->addResource(resource);
         }
     }
 }
 
-std::unique_ptr<IOHandler> FanArtHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> FanArtHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
-    fs::path path = obj->getResource(resNum)->getAttribute(CdsResource::Attribute::RESOURCE_FILE);
+    fs::path path = resource->getAttribute(ResourceAttribute::RESOURCE_FILE);
     if (path.empty()) {
         path = setup->getContentPath(obj, SETTING_FANART)[0];
     }
@@ -214,13 +261,13 @@ ContainerArtHandler::ContainerArtHandler(const std::shared_ptr<Context>& context
     : MetacontentHandler(context)
 {
     if (!setup) {
-        setup = std::make_unique<ContentPathSetup>(config, CFG_IMPORT_RESOURCES_CONTAINERART_FILE_LIST, CFG_IMPORT_RESOURCES_CONTAINERART_DIR_LIST);
+        setup = std::make_unique<ContentPathSetup>(config, definition, ConfigVal::IMPORT_RESOURCES_CONTAINERART_FILE_LIST, ConfigVal::IMPORT_RESOURCES_CONTAINERART_DIR_LIST);
     }
 }
 
 void ContainerArtHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
-    auto pathList = setup->getContentPath(obj, SETTING_CONTAINERART, config->getOption(CFG_IMPORT_RESOURCES_CONTAINERART_LOCATION));
+    auto pathList = setup->getContentPath(obj, SETTING_CONTAINERART, config->getOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_LOCATION));
     if (pathList.empty() || pathList[0].empty()) {
         pathList = setup->getContentPath(obj, SETTING_CONTAINERART);
     }
@@ -229,24 +276,29 @@ void ContainerArtHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
         obj->removeResource(ContentHandler::CONTAINERART);
 
     for (auto&& path : pathList) {
-        log_debug("Running ContainerArt handler on {}", !path.empty() ? path.c_str() : obj->getLocation().c_str());
         if (!path.empty()) {
-            auto resource = std::make_shared<CdsResource>(ContentHandler::CONTAINERART, CdsResource::Purpose::Thumbnail);
+            log_debug("Running ContainerArt handler on {}", !path.empty() ? path.c_str() : obj->getLocation().c_str());
+            auto resource = std::make_shared<CdsResource>(ContentHandler::CONTAINERART, ResourcePurpose::Thumbnail);
             std::string type = path.extension().string().substr(1);
-            std::string mimeType = mime->getMimeType(path, fmt::format("image/{}", type));
-
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
-            resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, path.string());
+            auto mimeType = std::get<1>(mime->getMimeType(path, fmt::format("image/{}", type)));
+            if (!mimeType.empty()) {
+                resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
+                auto [val, err] = f2i->convert(path.string());
+                if (!err.empty()) {
+                    log_warning("{}: {}", path.string(), err);
+                }
+                resource->addAttribute(ResourceAttribute::RESOURCE_FILE, val);
+            }
             obj->addResource(resource);
         }
     }
 }
 
-std::unique_ptr<IOHandler> ContainerArtHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> ContainerArtHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
-    fs::path path = obj->getResource(resNum)->getAttribute(CdsResource::Attribute::RESOURCE_FILE);
+    fs::path path = resource->getAttribute(ResourceAttribute::RESOURCE_FILE);
     if (path.empty()) {
-        path = setup->getContentPath(obj, SETTING_CONTAINERART, config->getOption(CFG_IMPORT_RESOURCES_CONTAINERART_LOCATION))[0];
+        path = setup->getContentPath(obj, SETTING_CONTAINERART, config->getOption(ConfigVal::IMPORT_RESOURCES_CONTAINERART_LOCATION))[0];
         if (path.empty()) {
             path = setup->getContentPath(obj, SETTING_CONTAINERART)[0];
         }
@@ -267,48 +319,55 @@ SubtitleHandler::SubtitleHandler(const std::shared_ptr<Context>& context)
     : MetacontentHandler(context)
 {
     if (!setup) {
-        setup = std::make_unique<ContentPathSetup>(config, CFG_IMPORT_RESOURCES_SUBTITLE_FILE_LIST, CFG_IMPORT_RESOURCES_SUBTITLE_DIR_LIST);
+        setup = std::make_unique<ContentPathSetup>(config, definition, ConfigVal::IMPORT_RESOURCES_SUBTITLE_FILE_LIST, ConfigVal::IMPORT_RESOURCES_SUBTITLE_DIR_LIST);
     }
 }
 
 void SubtitleHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
 {
     auto pathList = setup->getContentPath(obj, SETTING_SUBTITLE);
-    auto objFilename = obj->getLocation().filename().string() + ".";
+    auto objFilename = obj->getLocation().filename().stem().string();
 
     if (pathList.empty() || pathList[0].empty())
         obj->removeResource(ContentHandler::SUBTITLE);
 
     for (auto&& path : pathList) {
-        log_debug("Running subtitle handler on {} -> {}", obj->getLocation().c_str(), path.c_str());
         if (!path.empty()) {
-            auto resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, CdsResource::Purpose::Subtitle);
+            log_debug("Running subtitle handler on {} -> {}", obj->getLocation().c_str(), path.c_str());
+            auto resource = std::make_shared<CdsResource>(ContentHandler::SUBTITLE, ResourcePurpose::Subtitle);
             std::string type = path.extension().string().substr(1);
 
-            std::string mimeType = mime->getMimeType(path, fmt::format("text/{}", type));
+            auto mimeType = std::get<1>(mime->getMimeType(path, fmt::format("text/{}", type)));
             auto pos = mimeType.find("plain");
             if (pos != std::string::npos) {
                 mimeType = fmt::format("{}{}", mimeType.substr(0, pos), type);
             }
 
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
-            resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, path.string());
-            resource->addAttribute(CdsResource::Attribute::TYPE, type);
+            resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo(mimeType));
+            auto [mval, err] = f2i->convert(path.string());
+            if (!err.empty()) {
+                log_warning("{}: {}", path.string(), err);
+            }
+            resource->addAttribute(ResourceAttribute::RESOURCE_FILE, mval);
+            resource->addAttribute(ResourceAttribute::TYPE, type);
             auto lang = path.stem().string();
             if (startswith(lang, objFilename)) {
                 replaceAllString(lang, objFilename, ""); // remove starting part with filename
             }
+            while (!lang.empty() && lang[0] == '.') {
+                lang = lang.substr(1); // remove starting dot
+            }
             if (!lang.empty())
-                resource->addAttribute(CdsResource::Attribute::LANGUAGE, lang); // assume file name is related to some language
+                resource->addAttribute(ResourceAttribute::LANGUAGE, lang); // assume file name is related to some language
             resource->addParameter("type", type);
             obj->addResource(resource);
         }
     }
 }
 
-std::unique_ptr<IOHandler> SubtitleHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> SubtitleHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
-    fs::path path = obj->getResource(resNum)->getAttribute(CdsResource::Attribute::RESOURCE_FILE);
+    fs::path path = resource->getAttribute(ResourceAttribute::RESOURCE_FILE);
     if (path.empty()) {
         path = setup->getContentPath(obj, SETTING_SUBTITLE)[0];
     }
@@ -322,13 +381,46 @@ std::unique_ptr<IOHandler> SubtitleHandler::serveContent(const std::shared_ptr<C
     return std::make_unique<FileIOHandler>(path);
 }
 
+std::unique_ptr<ContentPathSetup> MetafileHandler::setup {};
+
+MetafileHandler::MetafileHandler(const std::shared_ptr<Context>& context, std::shared_ptr<Content> content)
+    : MetacontentHandler(context)
+    , content(std::move(content))
+{
+    if (!setup) {
+        setup = std::make_unique<ContentPathSetup>(config, definition, ConfigVal::IMPORT_RESOURCES_METAFILE_FILE_LIST, ConfigVal::IMPORT_RESOURCES_METAFILE_DIR_LIST);
+    }
+}
+
+void MetafileHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
+{
+#ifdef HAVE_JS
+    auto pathList = setup->getContentPath(obj, SETTING_METAFILE);
+
+    if (pathList.empty() || pathList[0].empty())
+        obj->removeResource(ContentHandler::METAFILE);
+
+    for (auto&& path : pathList) {
+        if (!path.empty()) {
+            log_debug("Running metafile handler on {} -> '{}'", obj->getLocation().c_str(), path.c_str());
+            content->parseMetafile(obj, path);
+        }
+    }
+#endif
+}
+
+std::unique_ptr<IOHandler> MetafileHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
+{
+    return {};
+}
+
 std::unique_ptr<ContentPathSetup> ResourceHandler::setup {};
 
 ResourceHandler::ResourceHandler(const std::shared_ptr<Context>& context)
     : MetacontentHandler(context)
 {
     if (!setup) {
-        setup = std::make_unique<ContentPathSetup>(config, CFG_IMPORT_RESOURCES_RESOURCE_FILE_LIST, CFG_IMPORT_RESOURCES_RESOURCE_DIR_LIST);
+        setup = std::make_unique<ContentPathSetup>(config, definition, ConfigVal::IMPORT_RESOURCES_RESOURCE_FILE_LIST, ConfigVal::IMPORT_RESOURCES_RESOURCE_DIR_LIST);
     }
 }
 
@@ -340,20 +432,23 @@ void ResourceHandler::fillMetadata(const std::shared_ptr<CdsObject>& obj)
         obj->removeResource(ContentHandler::RESOURCE);
 
     for (auto&& path : pathList) {
-        log_debug("Running resource handler check on {} -> {}", obj->getLocation().string(), path.string());
-
         if (!path.empty() && toLower(path.string()) == toLower(obj->getLocation().string())) {
-            auto resource = std::make_shared<CdsResource>(ContentHandler::RESOURCE, CdsResource::Purpose::Thumbnail);
-            resource->addAttribute(CdsResource::Attribute::PROTOCOLINFO, renderProtocolInfo("res"));
-            resource->addAttribute(CdsResource::Attribute::RESOURCE_FILE, path.string());
+            log_debug("Running resource handler check on {} -> {}", obj->getLocation().string(), path.string());
+            auto resource = std::make_shared<CdsResource>(ContentHandler::RESOURCE, ResourcePurpose::Thumbnail);
+            resource->addAttribute(ResourceAttribute::PROTOCOLINFO, renderProtocolInfo("res"));
+            auto [mval, err] = f2i->convert(path.string());
+            if (!err.empty()) {
+                log_warning("{}: {}", path.string(), err);
+            }
+            resource->addAttribute(ResourceAttribute::RESOURCE_FILE, mval);
             obj->addResource(resource);
         }
     }
 }
 
-std::unique_ptr<IOHandler> ResourceHandler::serveContent(const std::shared_ptr<CdsObject>& obj, int resNum)
+std::unique_ptr<IOHandler> ResourceHandler::serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource)
 {
-    fs::path path = obj->getResource(resNum)->getAttribute(CdsResource::Attribute::RESOURCE_FILE);
+    fs::path path = resource->getAttribute(ResourceAttribute::RESOURCE_FILE);
     if (path.empty()) {
         path = setup->getContentPath(obj, SETTING_RESOURCE)[0];
     }
