@@ -4,7 +4,7 @@
 
     metacontent_handler.h - this file is part of Gerbera.
 
-    Copyright (C) 2020-2024 Gerbera Contributors
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -32,25 +32,33 @@
 #include "config/config.h"
 #include "metadata_handler.h"
 
+class ConfigDefinition;
 class Content;
+class StringConverter;
 
 class ContentPathSetup {
 public:
-    explicit ContentPathSetup(std::shared_ptr<Config> config, ConfigVal fileListOption, ConfigVal dirListOption);
+    explicit ContentPathSetup(std::shared_ptr<Config> config, const std::shared_ptr<ConfigDefinition>& definition, ConfigVal fileListOption, ConfigVal dirListOption);
     std::vector<fs::path> getContentPath(const std::shared_ptr<CdsObject>& obj, const std::string& setting, fs::path folder = "") const;
 
 private:
     std::shared_ptr<Config> config;
     std::vector<std::string> names;
-    std::map<std::string, std::string> patterns;
+    std::vector<std::vector<std::pair<std::string, std::string>>> patterns;
     std::shared_ptr<DirectoryConfigList> allTweaks;
-    static std::string expandName(std::string_view name, const std::shared_ptr<CdsObject>& obj);
+    static std::string expandName(const std::string& name, const std::shared_ptr<CdsObject>& obj);
     bool caseSensitive;
+    std::shared_ptr<ConfigDefinition> definition;
 };
 
 /// \brief This class is responsible for populating filesystem based metadata
 class MetacontentHandler : public MetadataHandler {
-    using MetadataHandler::MetadataHandler;
+public:
+    explicit MetacontentHandler(const std::shared_ptr<Context>& context);
+
+protected:
+    const std::shared_ptr<StringConverter> f2i;
+    std::shared_ptr<ConfigDefinition> definition;
 };
 
 /// \brief This class is responsible for populating filesystem based album and fan art
