@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -37,12 +37,15 @@
 
 #ifdef HAVE_TAGLIB
 
+#include "metadata_handler.h"
+
+#include <map>
 #include <tbytevector.h>
 #include <tfile.h>
 #include <tiostream.h>
 
-#include "metadata_handler.h"
-
+// forward declarations
+class CdsItem;
 class StringConverter;
 
 /// \brief This class is responsible for reading id3 or ogg tags metadata
@@ -58,19 +61,39 @@ public:
     /// \param obj Object to stream
     /// \param resource the resource
     /// \return iohandler to stream to client
-    std::unique_ptr<IOHandler> serveContent(const std::shared_ptr<CdsObject>& obj, const std::shared_ptr<CdsResource>& resource) override;
+    std::unique_ptr<IOHandler> serveContent(
+        const std::shared_ptr<CdsObject>& obj,
+        const std::shared_ptr<CdsResource>& resource) override;
 
 private:
     std::string entrySeparator;
     std::string legacyEntrySeparator;
 
-    void addField(MetadataFields field, const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const;
-    void addSpecialFields(const TagLib::File& file, const TagLib::Tag* tag, const std::shared_ptr<CdsItem>& item) const;
+    void addField(
+        MetadataFields field,
+        const TagLib::File& file,
+        const TagLib::Tag* tag,
+        const std::shared_ptr<CdsItem>& item) const;
+    void addSpecialFields(
+        const TagLib::File& file,
+        const TagLib::Tag* tag,
+        const std::shared_ptr<CdsItem>& item) const;
 
     static std::unique_ptr<TagLib::File> getOggFile(TagLib::IOStream& ioStream);
 
-    void populateGenericTags(const std::shared_ptr<CdsItem>& item, const TagLib::File& file) const;
-    void populateAuxTags(const std::shared_ptr<CdsItem>& item, const TagLib::PropertyMap& propertyMap, const std::shared_ptr<StringConverter>& sc) const;
+    void populateGenericTags(
+        const std::shared_ptr<CdsItem>& item,
+        const TagLib::File& file,
+        const TagLib::PropertyMap& propertyMap,
+        const std::shared_ptr<StringConverter>& sc) const;
+    void populateAuxTags(
+        const std::shared_ptr<CdsItem>& item,
+        const TagLib::PropertyMap& propertyMap,
+        const std::shared_ptr<StringConverter>& sc) const;
+    void makeComment(
+        const std::shared_ptr<CdsItem>& item,
+        const TagLib::PropertyMap& propertyMap,
+        const std::shared_ptr<StringConverter>& sc) const;
     static bool isValidArtworkContentType(std::string_view artMimetype);
     std::string getContentTypeFromByteVector(const TagLib::ByteVector& data) const;
     static void addArtworkResource(const std::shared_ptr<CdsItem>& item, const std::string& artMimetype);
@@ -82,6 +105,9 @@ private:
     void extractWavPack(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const;
     void extractMP4(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const;
     void extractAiff(TagLib::IOStream& roStream, const std::shared_ptr<CdsItem>& item) const;
+
+    template <class Media>
+    void setBitsPerSample(const std::shared_ptr<CdsItem>& item, Media& media) const;
 };
 
 #endif
