@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -66,8 +66,8 @@
 #include "util/tools.h"
 #include "util/url_utils.h"
 #include "util/xml_to_json.h"
-#include "web/pages.h"
 #include "web/session_manager.h"
+#include "web/web_request_handler.h"
 
 #ifdef HAVE_CURL
 #include "request_handler/url_request_handler.h"
@@ -100,7 +100,7 @@ Server::Server(std::shared_ptr<Config> config)
 {
 }
 
-void Server::init(bool offln)
+void Server::init(const std::shared_ptr<ConfigDefinition>& definition, bool offln)
 {
     offline = offln;
 
@@ -120,16 +120,13 @@ void Server::init(bool offln)
 
     serverUDN = config->getOption(ConfigVal::SERVER_UDN);
     if (serverUDN == GRB_UDN_AUTO) {
-        serverUDN = fmt::format("uuid:{}", generateRandomId());
-        auto cs = ConfigDefinition::findConfigSetup(ConfigVal::SERVER_UDN);
-        database->updateConfigValue(cs->getUniquePath(), cs->getItemPath({}, {}), serverUDN, "added");
-        log_info("Generated UDN '{}' and saved in database", serverUDN);
+        serverUDN = config->generateUDN(database);
     }
     aliveAdvertisementInterval = config->getIntOption(ConfigVal::SERVER_ALIVE_INTERVAL);
 
     clientManager = std::make_shared<ClientManager>(config, database);
     sessionManager = std::make_shared<Web::SessionManager>(config, timer);
-    context = std::make_shared<Context>(config, clientManager, mime, database, sessionManager, converterManager);
+    context = std::make_shared<Context>(definition, config, clientManager, mime, database, sessionManager, converterManager);
 
     content = std::make_shared<ContentManager>(context, self, timer);
     metadataService = std::make_shared<MetadataService>(context, content);
@@ -244,9 +241,11 @@ void Server::run()
         throw UpnpException(ret, fmt::format("run: UpnpSendAdvertisement {} failed", aliveAdvertisementInterval));
     }
 
-    if (config->getBoolOption(ConfigVal::UPNP_LITERAL_HOST_REDIRECTION))
+    if (config->getBoolOption(ConfigVal::UPNP_LITERAL_HOST_REDIRECTION)) {
         UpnpSetAllowLiteralHostRedirection(1);
+    }
 
+    std::string ipv6 = UpnpGetServerIp6Address();
     {
         std::string url = getVirtualUrl();
         writeBookmark(url);
@@ -254,9 +253,11 @@ void Server::run()
 
         validHosts = std::vector<std::string> {
             std::string(UpnpGetServerIpAddress()),
-            std::string(UpnpGetServerIp6Address()),
-            std::string(UpnpGetServerUlaGuaIp6Address()),
         };
+        if (!ipv6.empty()) {
+            validHosts.push_back(ipv6);
+            validHosts.push_back(UpnpGetServerUlaGuaIp6Address());
+        }
         if (!url.empty()) {
             validHosts.push_back(url);
         }
@@ -270,9 +271,11 @@ void Server::run()
         corsHosts = {
             "'self'",
             fmt::format("http://{}", UpnpGetServerIpAddress()),
-            fmt::format("http://[{}]", UpnpGetServerIp6Address()),
-            fmt::format("http://[{}]", UpnpGetServerUlaGuaIp6Address()),
         };
+        if (!ipv6.empty()) {
+            corsHosts.push_back(fmt::format("http://[{}]", ipv6));
+            corsHosts.push_back(fmt::format("http://[{}]", UpnpGetServerUlaGuaIp6Address()));
+        }
         std::string url = getVirtualUrl();
         if (!url.empty()) {
             corsHosts.push_back(url);
@@ -294,6 +297,9 @@ int Server::startupInterface(const std::string& iface, in_port_t inPort)
     const char* ifName = iface.empty() ? nullptr : iface.c_str();
 
     int ret = UPNP_E_INIT_FAILED;
+#ifdef UPNP_HAVE_TOOLS
+    UpnpSetMaxJobsTotal(config->getUIntOption(ConfigVal::SERVER_UPNP_MAXJOBS));
+#endif
     for (std::size_t attempt = 0; attempt < 4; attempt++) {
         ret = UpnpInit2(ifName, inPort);
         if (ret == UPNP_E_SUCCESS)
@@ -674,8 +680,8 @@ int Server::GetInfoCallback(const char* filename, UpnpFileInfo* info, const void
         return 0;
     } catch (const ServerShutdownException&) {
         return -1;
-    } catch (const SubtitlesNotFoundException& stex) {
-        log_warning("SubtitlesNotFoundException: {}", stex.what());
+    } catch (const ResourceNotFoundException& stex) {
+        log_warning("ResourceNotFoundException: {}", stex.what());
         return -1;
     } catch (const std::runtime_error& e) {
         log_error("Exception: {}", e.what());
@@ -687,6 +693,9 @@ UpnpWebFileHandle Server::OpenCallback(const char* filename, enum UpnpOpenFileMo
 {
     try {
         log_debug("open({})", filename);
+        auto server = static_cast<const Server*>(cookie);
+        if (server->getShutdownStatus())
+            return nullptr;
         auto client = requestCookie ? static_cast<const ClientObservation*>(requestCookie) : nullptr;
         auto quirks = client ? std::make_shared<Quirks>(client) : nullptr;
         if (quirks && !quirks->isAllowed()) {
@@ -694,7 +703,7 @@ UpnpWebFileHandle Server::OpenCallback(const char* filename, enum UpnpOpenFileMo
             log_debug("Client blocked {}", ip);
             return nullptr;
         }
-        auto reqHandler = static_cast<const Server*>(cookie)->createRequestHandler(filename, quirks);
+        auto reqHandler = server->createRequestHandler(filename, quirks);
         std::string link = URLUtils::urlUnescape(filename);
         auto ioHandler = reqHandler->open(startswith(link, fmt::format("/{}", CONTENT_UI_HANDLER)) ? filename : link.c_str(), quirks, mode);
         if (ioHandler) {
@@ -705,8 +714,8 @@ UpnpWebFileHandle Server::OpenCallback(const char* filename, enum UpnpOpenFileMo
         return nullptr;
     } catch (const ServerShutdownException&) {
         return nullptr;
-    } catch (const SubtitlesNotFoundException& stex) {
-        log_warning("SubtitlesNotFoundException: {}", stex.what());
+    } catch (const ResourceNotFoundException& stex) {
+        log_warning("ResourceNotFoundException: {}", stex.what());
         return nullptr;
     } catch (const std::runtime_error& ex) {
         log_error("Exception: {}", ex.what());
