@@ -108,20 +108,33 @@
 #define UPNP_DLNA_PROFILE_PNG_SM_ICO "PNG_TN" // "PNG_SM_ICO"
 #define UPNP_DLNA_PROFILE_PNG_LRG_ICO "JPEG_TN" // "PNG_LRG_ICO"
 
-UpnpXMLBuilder::UpnpXMLBuilder(const std::shared_ptr<Context>& context,
+UpnpXMLBuilder::UpnpXMLBuilder(
+    const std::shared_ptr<Context>& context,
     std::string virtualUrl)
     : config(context->getConfig())
     , database(context->getDatabase())
+    , definition(context->getDefinition())
     , virtualURL(std::move(virtualUrl))
 {
     for (auto&& entry : this->config->getArrayOption(ConfigVal::IMPORT_RESOURCES_ORDER)) {
         orderedHandler.push_back(EnumMapper::remapContentHandler(entry));
     }
+
     entrySeparator = config->getOption(ConfigVal::IMPORT_LIBOPTS_ENTRY_SEP);
     multiValue = config->getBoolOption(ConfigVal::UPNP_MULTI_VALUES_ENABLED);
     ctMappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
     profMappings = config->getVectorOption(ConfigVal::IMPORT_MAPPINGS_CONTENTTYPE_TO_DLNAPROFILE_LIST);
     transferMappings = config->getDictionaryOption(ConfigVal::IMPORT_MAPPINGS_CONTENTTYPE_TO_DLNATRANSFER_LIST);
+    objectNamespaces = {
+        { ConfigVal::UPNP_TITLE_NAMESPACES, config->getDictionaryOption(ConfigVal::UPNP_TITLE_NAMESPACES) },
+        { ConfigVal::UPNP_ALBUM_NAMESPACES, config->getDictionaryOption(ConfigVal::UPNP_ALBUM_NAMESPACES) },
+        { ConfigVal::UPNP_ARTIST_NAMESPACES, config->getDictionaryOption(ConfigVal::UPNP_ARTIST_NAMESPACES) },
+        { ConfigVal::UPNP_GENRE_NAMESPACES, config->getDictionaryOption(ConfigVal::UPNP_GENRE_NAMESPACES) },
+        { ConfigVal::UPNP_PLAYLIST_NAMESPACES, config->getDictionaryOption(ConfigVal::UPNP_PLAYLIST_NAMESPACES) },
+    };
+    resourcePropertyDefaults = config->getDictionaryOption(ConfigVal::UPNP_RESOURCE_PROPERTY_DEFAULTS);
+    objectPropertyDefaults = config->getDictionaryOption(ConfigVal::UPNP_OBJECT_PROPERTY_DEFAULTS);
+    containerPropertyDefaults = config->getDictionaryOption(ConfigVal::UPNP_CONTAINER_PROPERTY_DEFAULTS);
 }
 
 std::unique_ptr<pugi::xml_document> UpnpXMLBuilder::createResponse(const std::string& actionName, const std::string& serviceType) const
@@ -167,30 +180,39 @@ static std::string formatXmlString(bool strictXml, std::size_t stringLimit, cons
     return s;
 }
 
-void UpnpXMLBuilder::addPropertyList(bool strictXml, std::size_t stringLimit, pugi::xml_node& result, const std::vector<std::pair<std::string, std::string>>& meta, const std::map<std::string, std::string>& auxData,
-    ConfigVal itemProps, ConfigVal nsProp) const
+std::vector<std::string> UpnpXMLBuilder::addPropertyList(
+    bool strictXml,
+    std::size_t stringLimit,
+    pugi::xml_node& result,
+    const std::vector<std::string>& filter,
+    const std::vector<std::pair<std::string, std::string>>& meta,
+    const std::map<std::string, std::string>& auxData,
+    ConfigVal itemProps,
+    ConfigVal nsProp) const
 {
-    auto namespaceMap = config->getDictionaryOption(nsProp);
+    auto namespaceMap = objectNamespaces.at(nsProp);
     for (auto&& [xmlns, uri] : namespaceMap) {
         result.append_attribute(fmt::format("xmlns:{}", xmlns).c_str()) = uri.c_str();
     }
     auto propertyMap = config->getDictionaryOption(itemProps);
+    std::vector<std::string> propNames;
     for (auto&& [tag, field] : propertyMap) {
         auto metaField = MetaEnumMapper::remapMetaDataField(field);
         bool wasMeta = false;
         for (auto&& [mkey, mvalue] : meta) {
             if ((metaField != MetadataFields::M_MAX && mkey == MetaEnumMapper::getMetaFieldName(metaField)) || mkey == field) {
-                addField(result, tag, formatXmlString(strictXml, stringLimit, mvalue));
+                propNames.push_back(addField(result, filter, tag, formatXmlString(strictXml, stringLimit, mvalue)));
                 wasMeta = true;
             }
         }
         if (!wasMeta) {
             auto avalue = getValueOrDefault(auxData, field);
             if (!avalue.empty()) {
-                addField(result, tag, formatXmlString(strictXml, stringLimit, avalue));
+                propNames.push_back(addField(result, filter, tag, formatXmlString(strictXml, stringLimit, avalue)));
             }
         }
     }
+    return propNames;
 }
 
 std::string UpnpXMLBuilder::printXml(const pugi::xml_node& entry, const char* indent, int flags)
@@ -200,36 +222,163 @@ std::string UpnpXMLBuilder::printXml(const pugi::xml_node& entry, const char* in
     return buf.str();
 }
 
-void UpnpXMLBuilder::addField(pugi::xml_node& entry, const std::string& key, const std::string& val) const
+std::string UpnpXMLBuilder::addField(
+    pugi::xml_node& entry,
+    const std::vector<std::string>& filter,
+    const std::string& key,
+    const std::string& val) const
 {
     auto i = key.find('@');
     auto j = key.find('[', i + 1);
+    auto filterActive = !(filter.size() == 1 && filter[0] == "*");
     if (i != std::string::npos && j != std::string::npos && key[key.length() - 1] == ']') {
         // e.g. used for MetadataFields::M_ALBUMARTIST
         // name@attr[val] => <name attr="val">
         std::string attrName = key.substr(i + 1, j - i - 1);
         std::string attrValue = key.substr(j + 1, key.length() - j - 2);
         std::string name = key.substr(0, i);
+        auto upnpElement = fmt::format("{}@{}", name, attrName);
+        if (filterActive && std::find(filter.begin(), filter.end(), upnpElement) == filter.end())
+            return "";
         auto node = entry.append_child(name.c_str());
         node.append_attribute(attrName.c_str()) = attrValue.c_str();
         node.append_child(pugi::node_pcdata).set_value(val.c_str());
+        return upnpElement;
     } else if (i != std::string::npos) {
         // name@attr val => <name attr="val">
         std::string name = key.substr(0, i);
         std::string attrName = key.substr(i + 1);
+        auto upnpElement = fmt::format("{}@{}", name, attrName);
+        if (filterActive && std::find(filter.begin(), filter.end(), upnpElement) == filter.end())
+            return "";
         auto child = entry.child(name.c_str());
         if (child) {
             child.append_attribute(attrName.c_str()) = val.c_str();
         } else {
-            entry.append_child(key.c_str()).append_attribute(attrName.c_str()) = val.c_str();
+            entry.append_child(name.c_str()).append_attribute(attrName.c_str()) = val.c_str();
         }
+        return upnpElement;
     } else {
+        if (filterActive && std::find(filter.begin(), filter.end(), key) == filter.end())
+            return "";
         entry.append_child(key.c_str()).append_child(pugi::node_pcdata).set_value(val.c_str());
+        return key;
     }
 }
 
-void UpnpXMLBuilder::renderObject(const std::shared_ptr<CdsObject>& obj, std::size_t stringLimit, pugi::xml_node& parent, const std::shared_ptr<Quirks>& quirks) const
+bool UpnpXMLBuilder::checkFilterNamespace(const std::string& f, ConfigVal nsProp) const
 {
+    /*
+     * request Filter =
+     *    Sony-Bluray:
+     *    @id,upnp:class,res,res@protocolInfo,res@av:authenticationUri,res@size,dc:title,upnp:albumArtURI,res@dlna:ifoFileURI,res@protection,res@bitrate,res@duration,res@sampleFrequency,res@bitsPerSample,res@nrAudioChannels,res@resolution,res@colorDepth,dc:date,av:dateTime,upnp:artist,upnp:album,upnp:genre,dc:contributer,upnp:storageFree,upnp:storageUsed,upnp:originalTrackNumber,dc:publisher,dc:language,dc:region,dc:description,upnp:toc,@childCount,upnp:albumArtURI@dlna:profileID,res@dlna:cleartextSize,@restricted,@dlna:dlnaManaged
+     *
+     *    KODI:
+     *    dc:date,dc:description,upnp:longDescription,upnp:genre,res,res@duration,res@size,upnp:albumArtURI,upnp:rating,upnp:lastPlaybackPosition,upnp:lastPlaybackTime,upnp:playbackCount,upnp:originalTrackNumber,upnp:episodeNumber,upnp:programTitle,upnp:seriesTitle,upnp:album,upnp:artist,upnp:author,upnp:director,dc:publisher,searchable,childCount,dc:title,dc:creator,upnp:actor,res@resolution,upnp:episodeCount,upnp:episodeSeason,xbmc:lastPlayerState,xbmc:dateadded,xbmc:rating,xbmc:votes,xbmc:artwork,xbmc:uniqueidentifier,xbmc:country,xbmc:userrating
+     */
+    auto parts = splitString(f, ':');
+    auto namespaceMap = objectNamespaces.at(nsProp);
+    if (parts.size() > 1) {
+        auto nsp = parts.at(0);
+        if (nsp != "dc" && nsp != "upnp" && namespaceMap.find(nsp) == namespaceMap.end())
+            return false;
+    }
+    return true;
+}
+
+void UpnpXMLBuilder::addDefaultProperty(
+    pugi::xml_node& result,
+    const std::vector<std::string>& propNames,
+    const std::vector<std::string>& filter,
+    const std::map<std::string, std::string>& defaults)
+{
+    for (auto&& tag : filter) {
+        if (std::find(propNames.begin(), propNames.end(), tag) == propNames.end()) {
+            std::string attributeTag = fmt::format("@{}", tag);
+            if (defaults.find(tag) != defaults.end()) {
+                result.append_child(tag.c_str()).append_child(pugi::node_pcdata).set_value(defaults.at(tag).c_str());
+            } else if (defaults.find(attributeTag) != defaults.end()) {
+                result.append_attribute(tag.c_str()) = defaults.at(attributeTag).c_str();
+            }
+        }
+    }
+}
+
+void UpnpXMLBuilder::renderObject(
+    const std::shared_ptr<CdsObject>& obj,
+    const std::vector<std::string>& filter,
+    std::size_t stringLimit,
+    pugi::xml_node& parent,
+    const std::shared_ptr<Quirks>& quirks) const
+{
+    ConfigVal itemProps = ConfigVal::UPNP_TITLE_PROPERTIES;
+    ConfigVal nsProp = ConfigVal::UPNP_TITLE_NAMESPACES;
+    const std::string upnpClass = obj->getClass();
+    if (startswith(upnpClass, UPNP_CLASS_MUSIC_ALBUM)) {
+        itemProps = ConfigVal::UPNP_ALBUM_PROPERTIES;
+        nsProp = ConfigVal::UPNP_ALBUM_NAMESPACES;
+    } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_ARTIST)) {
+        itemProps = ConfigVal::UPNP_ARTIST_PROPERTIES;
+        nsProp = ConfigVal::UPNP_ARTIST_NAMESPACES;
+    } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_GENRE)) {
+        itemProps = ConfigVal::UPNP_GENRE_PROPERTIES;
+        nsProp = ConfigVal::UPNP_GENRE_NAMESPACES;
+    } else if (startswith(upnpClass, UPNP_CLASS_PLAYLIST_CONTAINER)) {
+        itemProps = ConfigVal::UPNP_PLAYLIST_PROPERTIES;
+        nsProp = ConfigVal::UPNP_PLAYLIST_NAMESPACES;
+    }
+
+    std::vector<std::string> cntFilter;
+    std::vector<std::string> objFilter;
+    std::vector<std::string> resFilter;
+    bool allObjProps = false;
+    bool allCntProps = false;
+    bool allResProps = false;
+    for (auto&& f : filter) {
+        if (f == "*") {
+            allObjProps = true;
+            allResProps = true;
+            allCntProps = true;
+        } else if (f == "res") {
+            // we always send resources
+        } else if (f == "res#") {
+            allResProps = true;
+        } else if (f == "container#") {
+            allCntProps = true;
+        } else if (startswith(f, "res@")) {
+            std::string resFlt = f.substr(4); // 4 == sizeof(res@)
+            if (checkFilterNamespace(resFlt, nsProp))
+                resFilter.push_back(resFlt);
+        } else if (startswith(f, "container@")) {
+            std::string contFlt = f.substr(10); // 10 == sizeof(container@)
+            if (checkFilterNamespace(contFlt, nsProp))
+                cntFilter.push_back(contFlt);
+        } else if (startswith(f, "@")) {
+            std::string objFlt = f.substr(1);
+            if (checkFilterNamespace(objFlt, nsProp))
+                cntFilter.push_back(objFlt);
+        } else {
+            if (checkFilterNamespace(f, nsProp))
+                objFilter.push_back(f);
+        }
+    }
+    if (allObjProps || objFilter.empty()) {
+        objFilter = { "*" };
+    } else if (std::find(objFilter.begin(), objFilter.end(), MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE)) == objFilter.end()) {
+        // date is required
+        objFilter.push_back(MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE));
+    }
+    if (allCntProps || cntFilter.empty()) {
+        cntFilter = { "*" };
+    }
+    if (allResProps || resFilter.empty()) {
+        resFilter = { "*" };
+    } else {
+        resFilter.push_back("protocolInfo");
+    }
+    log_debug("Object Filters {}", fmt::join(objFilter, ", "));
+    log_debug("Container Filters {}", fmt::join(cntFilter, ", "));
+    log_debug("Resource Filters {}", fmt::join(resFilter, ", "));
     auto result = parent.append_child("");
 
     result.append_attribute("id") = obj->getID();
@@ -238,15 +387,15 @@ void UpnpXMLBuilder::renderObject(const std::shared_ptr<CdsObject>& obj, std::si
 
     auto strictXml = quirks && quirks->needsStrictXml();
     const std::string title = obj->getTitle();
-    const std::string upnpClass = obj->getClass();
 
-    result.append_child("dc:title").append_child(pugi::node_pcdata).set_value(formatXmlString(strictXml, stringLimit, title).c_str());
-    result.append_child("upnp:class").append_child(pugi::node_pcdata).set_value(upnpClass.c_str());
+    result.append_child(DC_TITLE).append_child(pugi::node_pcdata).set_value(formatXmlString(strictXml, stringLimit, title).c_str());
+    result.append_child(UPNP_SEARCH_CLASS).append_child(pugi::node_pcdata).set_value(upnpClass.c_str());
 
     auto auxData = obj->getAuxData();
     auto mvMeta = multiValue;
     auto simpleDate = false;
 
+    std::vector<std::string> propNames;
     if (obj->isItem()) {
         auto item = std::static_pointer_cast<CdsItem>(obj);
 
@@ -265,26 +414,32 @@ void UpnpXMLBuilder::renderObject(const std::shared_ptr<CdsObject>& obj, std::si
                 for (auto&& val : group) {
                     // Trim metadata value as needed
                     auto str = formatXmlString(strictXml, stringLimit, val);
-                    if (key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DESCRIPTION))
+                    if (key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DESCRIPTION)) {
                         result.append_child(key.c_str()).append_child(pugi::node_pcdata).set_value(str.c_str());
-                    else if (startswith(upnpClass, UPNP_CLASS_MUSIC_TRACK) && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER))
+                        propNames.push_back(key);
+                    } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_TRACK) && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER)) {
                         result.append_child(key.c_str()).append_child(pugi::node_pcdata).set_value(str.c_str());
-                    else if (simpleDate && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE))
-                        addField(result, key, makeSimpleDate(str));
-                    else if (key != MetaEnumMapper::getMetaFieldName(MetadataFields::M_TITLE))
-                        addField(result, key, str);
+                        propNames.push_back(key);
+                    } else if (simpleDate && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE)) {
+                        propNames.push_back(addField(result, objFilter, key, makeSimpleDate(str)));
+                    } else if (key != MetaEnumMapper::getMetaFieldName(MetadataFields::M_TITLE)) {
+                        propNames.push_back(addField(result, objFilter, key, str));
+                    }
                 }
             } else {
                 // Trim metadata value as needed
                 auto str = formatXmlString(strictXml, stringLimit, fmt::format("{}", fmt::join(group, entrySeparator)));
-                if (key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DESCRIPTION))
+                if (key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DESCRIPTION)) {
                     result.append_child(key.c_str()).append_child(pugi::node_pcdata).set_value(str.c_str());
-                else if (startswith(upnpClass, UPNP_CLASS_MUSIC_TRACK) && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER))
+                    propNames.push_back(key);
+                } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_TRACK) && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_TRACKNUMBER)) {
                     result.append_child(key.c_str()).append_child(pugi::node_pcdata).set_value(str.c_str());
-                else if (simpleDate && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE))
-                    addField(result, key, makeSimpleDate(str));
-                else if (key != MetaEnumMapper::getMetaFieldName(MetadataFields::M_TITLE))
-                    addField(result, key, str);
+                    propNames.push_back(key);
+                } else if (simpleDate && key == MetaEnumMapper::getMetaFieldName(MetadataFields::M_DATE)) {
+                    propNames.push_back(addField(result, objFilter, key, makeSimpleDate(str)));
+                } else if (key != MetaEnumMapper::getMetaFieldName(MetadataFields::M_TITLE)) {
+                    propNames.push_back(addField(result, objFilter, key, str));
+                }
             }
         }
         auto meta = obj->getMetaData();
@@ -298,16 +453,17 @@ void UpnpXMLBuilder::renderObject(const std::shared_ptr<CdsObject>& obj, std::si
         // add playback statistics
         auto playStatus = item->getPlayStatus();
         if (playStatus) {
-            auxData["upnp:playbackCount"] = fmt::format("{}", playStatus->getPlayCount());
-            auxData["upnp:lastPlaybackTime"] = fmt::format("{:%Y-%m-%d T %H:%M:%S}", fmt::localtime(playStatus->getLastPlayed().count()));
+            auxData[UPNP_SEARCH_PLAY_COUNT] = fmt::format("{}", playStatus->getPlayCount());
+            auxData[UPNP_SEARCH_LAST_PLAYED] = fmt::format("{:%Y-%m-%d T %H:%M:%S}", fmt::localtime(playStatus->getLastPlayed().count()));
             auxData["upnp:lastPlaybackPosition"] = fmt::format("{}", millisecondsToHMSF(playStatus->getLastPlayedPosition().count()));
-            addField(result, "upnp:playbackCount", auxData["upnp:playbackCount"]);
-            addField(result, "upnp:lastPlaybackTime", auxData["upnp:lastPlaybackTime"]);
-            addField(result, "upnp:lastPlaybackPosition", auxData["upnp:lastPlaybackPosition"]);
+            propNames.push_back(addField(result, objFilter, UPNP_SEARCH_PLAY_COUNT, auxData[UPNP_SEARCH_PLAY_COUNT]));
+            propNames.push_back(addField(result, objFilter, UPNP_SEARCH_LAST_PLAYED, auxData[UPNP_SEARCH_LAST_PLAYED]));
+            propNames.push_back(addField(result, objFilter, "upnp:lastPlaybackPosition", auxData["upnp:lastPlaybackPosition"]));
         }
 
-        addPropertyList(strictXml, stringLimit, result, meta, auxData, ConfigVal::UPNP_TITLE_PROPERTIES, ConfigVal::UPNP_TITLE_NAMESPACES);
-        addResources(item, result, quirks);
+        auto propNamesMeta = addPropertyList(strictXml, stringLimit, result, objFilter, meta, auxData, itemProps, nsProp);
+        propNames.insert(propNames.end(), propNamesMeta.begin(), propNamesMeta.end());
+        addResources(item, result, resFilter, quirks);
 
         result.set_name("item");
     } else if (obj->isContainer()) {
@@ -321,28 +477,34 @@ void UpnpXMLBuilder::renderObject(const std::shared_ptr<CdsObject>& obj, std::si
         // add metadata
         log_debug("container is class: {}", upnpClass.c_str());
         auto&& meta = obj->getMetaData();
-        if (startswith(upnpClass, UPNP_CLASS_MUSIC_ALBUM)) {
-            addPropertyList(strictXml, stringLimit, result, meta, auxData, ConfigVal::UPNP_ALBUM_PROPERTIES, ConfigVal::UPNP_ALBUM_NAMESPACES);
-        } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_ARTIST)) {
-            addPropertyList(strictXml, stringLimit, result, meta, auxData, ConfigVal::UPNP_ARTIST_PROPERTIES, ConfigVal::UPNP_ARTIST_NAMESPACES);
-        } else if (startswith(upnpClass, UPNP_CLASS_MUSIC_GENRE)) {
-            addPropertyList(strictXml, stringLimit, result, meta, auxData, ConfigVal::UPNP_GENRE_PROPERTIES, ConfigVal::UPNP_GENRE_NAMESPACES);
-        } else if (startswith(upnpClass, UPNP_CLASS_PLAYLIST_CONTAINER)) {
-            addPropertyList(strictXml, stringLimit, result, meta, auxData, ConfigVal::UPNP_PLAYLIST_PROPERTIES, ConfigVal::UPNP_PLAYLIST_NAMESPACES);
-        }
+        propNames = addPropertyList(strictXml, stringLimit, result, cntFilter, meta, auxData, itemProps, nsProp);
         if (startswith(upnpClass, UPNP_CLASS_MUSIC_ALBUM) || startswith(upnpClass, UPNP_CLASS_MUSIC_ARTIST) || startswith(upnpClass, UPNP_CLASS_CONTAINER) || startswith(upnpClass, UPNP_CLASS_PLAYLIST_CONTAINER)) {
             auto url = renderContainerImageURL(cont);
             if (url) {
                 result.append_child(MetaEnumMapper::getMetaFieldName(MetadataFields::M_ALBUMARTURI).data()).append_child(pugi::node_pcdata).set_value(url.value().c_str());
             }
         }
+        addResources(cont, result, resFilter, quirks);
     }
 
     // make sure a date is set
-    auto dateNode = result.child("dc:date");
+    auto dateNode = result.child(DC_DATE);
     if (!dateNode) {
         auto fDate = fmt::format("{:%FT%T%z}", fmt::localtime(obj->getMTime().count()));
-        result.append_child("dc:date").append_child(pugi::node_pcdata).set_value(fDate.c_str());
+        if (simpleDate)
+            fDate = makeSimpleDate(fDate);
+        result.append_child(DC_DATE).append_child(pugi::node_pcdata).set_value(fDate.c_str());
+        propNames.emplace_back(DC_DATE);
+    }
+    for (auto&& fixTag : { "id", "parentID", "restricted", DC_TITLE, UPNP_SEARCH_CLASS })
+        propNames.emplace_back(fixTag);
+
+    if (quirks && quirks->getFullFilter()) {
+        if (obj->isItem() && !(objFilter.size() == 1 && objFilter[0] == "*")) {
+            addDefaultProperty(result, propNames, objFilter, objectPropertyDefaults);
+        } else if (obj->isContainer() && !(cntFilter.size() == 1 && cntFilter[0] == "*")) {
+            addDefaultProperty(result, propNames, cntFilter, containerPropertyDefaults);
+        }
     }
     log_debug("Rendered DIDL: {}", printXml(result, "  "));
 }
@@ -359,6 +521,7 @@ std::unique_ptr<pugi::xml_document> UpnpXMLBuilder::createEventPropertySet() con
     return doc;
 }
 
+/// \brief: gerbera uses these resource attributes for internal mechanisms
 static bool isPrivateAttribute(ResourceAttribute attribute)
 {
     switch (attribute) {
@@ -373,23 +536,43 @@ static bool isPrivateAttribute(ResourceAttribute attribute)
     }
 }
 
-void UpnpXMLBuilder::renderResource(const CdsObject& object, const CdsResource& resource, pugi::xml_node& parent, const std::map<std::string, std::string>& clientSpecificAttrs, const std::string& clientGroup, const std::map<std::string, std::string>& mimeMappings) const
+void UpnpXMLBuilder::renderResource(const CdsObject& object,
+    const CdsResource& resource,
+    pugi::xml_node& parent,
+    const std::vector<std::string>& filter,
+    const std::shared_ptr<Quirks>& quirks,
+    const std::map<std::string, std::string>& clientSpecificAttrs,
+    const std::string& clientGroup,
+    const std::map<std::string, std::string>& mimeMappings) const
 {
     auto res = parent.append_child("res");
 
+    res.append_attribute("id") = fmt::format("{}.{}", object.getID(), resource.getResId()).c_str();
+
     auto url = renderResourceURL(object, resource, mimeMappings, clientGroup);
 
     res.append_child(pugi::node_pcdata).set_value(url.c_str());
 
+    auto filterActive = !(filter.size() == 1 && filter[0] == "*");
+    std::vector<std::string> propNames = { "id" };
     for (auto&& [attr, val] : resource.getAttributes()) {
         if (isPrivateAttribute(attr)) {
             continue;
         }
+        if (filterActive && std::find(filter.begin(), filter.end(), EnumMapper::getAttributeName(attr)) == filter.end())
+            continue;
         res.append_attribute(EnumMapper::getAttributeName(attr).c_str()) = val.c_str();
+        propNames.push_back(EnumMapper::getAttributeName(attr));
     }
 
     for (auto&& [k, v] : clientSpecificAttrs) {
+        if (filterActive && std::find(filter.begin(), filter.end(), k) == filter.end())
+            continue;
         res.append_attribute(k.c_str()) = v.c_str();
+        propNames.push_back(k);
+    }
+    if (filterActive && quirks && quirks->getFullFilter()) {
+        addDefaultProperty(res, propNames, filter, resourcePropertyDefaults);
     }
 }
 
@@ -442,16 +625,18 @@ std::string UpnpXMLBuilder::renderResourceURL(const CdsObject& item, const CdsRe
         } else if (res.getPurpose() == ResourcePurpose::Transcode) {
             // Transcoded resources dont set a resId, uses pr_name from params instead.
             url = virtualURL + URLUtils::joinUrl({ CONTENT_ONLINE_HANDLER, URL_OBJECT_ID, fmt::to_string(item.getID()), URL_RESOURCE_ID, URL_VALUE_TRANSCODE_NO_RES_ID });
+        } else if (res.getPurpose() == ResourcePurpose::Thumbnail && res.getHandlerType() == ContentHandler::EXTURL && !item.getFlag(OBJECT_FLAG_PROXY_URL)) {
+            url = res.getAttribute(ResourceAttribute::RESOURCE_FILE);
+            if (url.empty())
+                throw_std_runtime_error("Missing attribute thumbnail URL");
+            if (!startswith(url, "http")) {
+                url = "";
+            } else {
+                return url;
+            }
         }
     }
 
-    // Externally hosted thumnbnails?
-    if (res.getPurpose() == ResourcePurpose::Thumbnail && res.getHandlerType() == ContentHandler::EXTURL) {
-        url = res.getOption(RESOURCE_OPTION_URL);
-        if (url.empty())
-            throw_std_runtime_error("missing thumbnail URL");
-    }
-
     // Standard Resource
     if (url.empty()) {
         if (res.getPurpose() == ResourcePurpose::Transcode) {
@@ -545,7 +730,10 @@ std::string UpnpXMLBuilder::renderExtension(const std::string& contentType, cons
     return {};
 }
 
-std::string UpnpXMLBuilder::getDLNAContentHeader(const std::string& contentType, const std::shared_ptr<CdsResource>& res, const std::shared_ptr<Quirks>& quirks) const
+std::string UpnpXMLBuilder::getDLNAContentHeader(
+    const std::string& contentType,
+    const std::shared_ptr<CdsResource>& res,
+    const std::shared_ptr<Quirks>& quirks) const
 {
     std::string contentParameter = dlnaProfileString(*res, contentType, quirks);
     return fmt::format("{}{}={};{}={};{}={}", contentParameter, //
@@ -567,6 +755,7 @@ static const std::map<std::string_view, std::map<std::string_view, std::string_v
                             { RESOURCE_IMAGE_STEP_SD, UPNP_DLNA_PROFILE_JPEG_SM },
                             { RESOURCE_IMAGE_STEP_HD, UPNP_DLNA_PROFILE_JPEG_MED },
                             { RESOURCE_IMAGE_STEP_UHD, UPNP_DLNA_PROFILE_JPEG_LRG },
+                            { RESOURCE_IMAGE_STEP_XHD, UPNP_DLNA_PROFILE_JPEG_LRG },
                             { RESOURCE_IMAGE_STEP_DEF, UPNP_DLNA_PROFILE_JPEG_TN },
                         } },
     { CONTENT_TYPE_PNG, {
@@ -576,11 +765,16 @@ static const std::map<std::string_view, std::map<std::string_view, std::string_v
                             { RESOURCE_IMAGE_STEP_SD, UPNP_DLNA_PROFILE_PNG_SM },
                             { RESOURCE_IMAGE_STEP_HD, UPNP_DLNA_PROFILE_PNG_MED },
                             { RESOURCE_IMAGE_STEP_UHD, UPNP_DLNA_PROFILE_PNG_LRG },
+                            { RESOURCE_IMAGE_STEP_XHD, UPNP_DLNA_PROFILE_PNG_LRG },
                             { RESOURCE_IMAGE_STEP_DEF, UPNP_DLNA_PROFILE_PNG_TN },
                         } },
 };
 
-std::string UpnpXMLBuilder::dlnaProfileString(const CdsResource& res, const std::string& contentType, const std::shared_ptr<Quirks>& quirks, bool formatted) const
+std::string UpnpXMLBuilder::dlnaProfileString(
+    const CdsResource& res,
+    const std::string& contentType,
+    const std::shared_ptr<Quirks>& quirks,
+    bool formatted) const
 {
     std::string dlnaProfile = res.getOption("dlnaProfile");
     if (profileList.find(contentType) != profileList.end()) {
@@ -602,11 +796,14 @@ std::string UpnpXMLBuilder::dlnaProfileString(const CdsResource& res, const std:
     return dlnaProfile;
 }
 
-std::string UpnpXMLBuilder::findDlnaProfile(const CdsResource& res, const std::string& contentType, const std::shared_ptr<Quirks>& quirks) const
+std::string UpnpXMLBuilder::findDlnaProfile(
+    const CdsResource& res,
+    const std::string& contentType,
+    const std::shared_ptr<Quirks>& quirks) const
 {
     std::string dlnaProfile;
-    static auto fromKey = ConfigDefinition::removeAttribute(ConfigVal::A_IMPORT_MAPPINGS_MIMETYPE_FROM);
-    static auto toKey = ConfigDefinition::removeAttribute(ConfigVal::A_IMPORT_MAPPINGS_MIMETYPE_TO);
+    static auto fromKey = definition->removeAttribute(ConfigVal::A_IMPORT_MAPPINGS_MIMETYPE_FROM);
+    static auto toKey = definition->removeAttribute(ConfigVal::A_IMPORT_MAPPINGS_MIMETYPE_TO);
     auto legacyKey = fmt::format("{}-{}-{}", contentType, res.getAttribute(ResourceAttribute::VIDEOCODEC), res.getAttribute(ResourceAttribute::AUDIOCODEC));
     std::size_t matchLength = 0;
     auto clientMappings = quirks ? quirks->getDlnaMappings() : std::vector<std::vector<std::pair<std::string, std::string>>>();
@@ -656,7 +853,11 @@ std::deque<std::shared_ptr<CdsResource>> UpnpXMLBuilder::getOrderedResources(con
     return orderedResources;
 }
 
-std::pair<bool, int> UpnpXMLBuilder::insertTempTranscodingResource(const std::shared_ptr<CdsItem>& item, const std::shared_ptr<Quirks>& quirks, std::deque<std::shared_ptr<CdsResource>>& orderedResources, bool skipURL) const
+std::pair<bool, int> UpnpXMLBuilder::insertTempTranscodingResource(
+    const std::shared_ptr<CdsItem>& item,
+    const std::shared_ptr<Quirks>& quirks,
+    std::deque<std::shared_ptr<CdsResource>>& orderedResources,
+    bool skipURL) const
 {
     bool hideOriginalResource = false;
     int originalResource = -1;
@@ -815,7 +1016,11 @@ std::pair<bool, int> UpnpXMLBuilder::insertTempTranscodingResource(const std::sh
     return { hideOriginalResource, originalResource };
 }
 
-void UpnpXMLBuilder::addResources(const std::shared_ptr<CdsItem>& item, pugi::xml_node& parent, const std::shared_ptr<Quirks>& quirks) const
+void UpnpXMLBuilder::addResources(
+    const std::shared_ptr<CdsItem>& item,
+    pugi::xml_node& parent,
+    const std::vector<std::string>& filter,
+    const std::shared_ptr<Quirks>& quirks) const
 {
     bool isExternalURL = (item->isExternalItem() && !item->getFlag(OBJECT_FLAG_PROXY_URL));
 
@@ -883,6 +1088,7 @@ void UpnpXMLBuilder::addResources(const std::shared_ptr<CdsItem>& item, pugi::xm
 
     auto [hideOriginalResource, originalResource] = insertTempTranscodingResource(item, quirks, orderedResources, isExternalURL);
 
+    auto clientGroup = quirks ? quirks->getGroup() : DEFAULT_CLIENT_GROUP;
     for (auto&& res : orderedResources) {
         auto purpose = res->getPurpose();
         if (quirks && !quirks->supportsResource(purpose)) {
@@ -895,13 +1101,30 @@ void UpnpXMLBuilder::addResources(const std::shared_ptr<CdsItem>& item, pugi::xm
             clientSpecficAttrs["pv:subtitleFileUri"] = captionInfo[""];
         }
 
-        bool transcoded = purpose == ResourcePurpose::Transcode;
-        auto clientGroup = quirks ? quirks->getGroup() : DEFAULT_CLIENT_GROUP;
-
         buildProtocolInfo(*res, mimeMappings, quirks);
 
-        if (!hideOriginalResource || transcoded || originalResource != res->getResId())
-            renderResource(*item, *res, parent, clientSpecficAttrs, clientGroup, mimeMappings);
+        if (!hideOriginalResource || purpose == ResourcePurpose::Transcode || originalResource != res->getResId())
+            renderResource(*item, *res, parent, filter, quirks, clientSpecficAttrs, clientGroup, mimeMappings);
+    }
+}
+
+void UpnpXMLBuilder::addResources(
+    const std::shared_ptr<CdsContainer>& cont,
+    pugi::xml_node& parent,
+    const std::vector<std::string>& filter,
+    const std::shared_ptr<Quirks>& quirks) const
+{
+    auto orderedResources = getOrderedResources(*cont);
+    auto mimeMappings = quirks ? quirks->getMimeMappings() : std::map<std::string, std::string>();
+    auto clientGroup = quirks ? quirks->getGroup() : DEFAULT_CLIENT_GROUP;
+
+    for (auto&& res : orderedResources) {
+        auto purpose = res->getPurpose();
+        if (quirks && !quirks->supportsResource(purpose)) {
+            continue;
+        }
+        buildProtocolInfo(*res, mimeMappings, quirks);
+        renderResource(*cont, *res, parent, filter, quirks, {}, clientGroup, mimeMappings);
     }
 }
 
@@ -921,7 +1144,10 @@ std::string UpnpXMLBuilder::getMimeType(const CdsResource& resource, const std::
     return mimeType;
 }
 
-std::string UpnpXMLBuilder::buildProtocolInfo(CdsResource& resource, const std::map<std::string, std::string>& mimeMappings, const std::shared_ptr<Quirks>& quirks) const
+std::string UpnpXMLBuilder::buildProtocolInfo(
+    CdsResource& resource,
+    const std::map<std::string, std::string>& mimeMappings,
+    const std::shared_ptr<Quirks>& quirks) const
 {
     // Why is this here? Just for transcoding maybe?
 
