@@ -4,7 +4,7 @@ Gerbera - https://gerbera.io/
 
     url_utils.cc - this file is part of Gerbera.
 
-    Copyright (C) 2022-2024 Gerbera Contributors
+    Copyright (C) 2022-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -72,6 +72,11 @@ std::string_view getQuery(std::string_view url)
     return splitUrl(url, URL_UI_PARAM_SEPARATOR).second;
 }
 
+std::string_view getFile(std::string_view url)
+{
+    return splitUrl(url, URL_UI_PARAM_SEPARATOR).first;
+}
+
 std::string joinUrl(const std::vector<std::string>& components, bool addToEnd, std::string_view separator)
 {
     if (components.empty())
