@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    action.cc - this file is part of MediaTomb.
+    web/action.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2025 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,22 +29,23 @@
     $Id$
 */
 
-/// \file action.cc
+/// \file web/action.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "config/config_manager.h"
-#include "content/content_manager.h"
+#include "common.h"
+#include "config/config.h"
+#include "content/content.h"
+#include "exceptions.h"
+#include "util/xml_to_json.h"
 
-void Web::Action::process()
+const std::string_view Web::Action::PAGE = "action";
+
+bool Web::Action::processPageAction(pugi::xml_node& element, const std::string& action)
 {
-    log_debug("action: start");
-    checkRequest();
-
-    std::string action = param("action");
     if (action.empty())
         throw_std_runtime_error("No action given");
-    log_debug("action: {}", action);
 
-    log_debug("action: returning");
+    return true;
 }
