@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    add.cc - this file is part of MediaTomb.
+    web/add.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,7 +29,7 @@
     $Id$
 */
 
-/// \file add.cc
+/// \file web/add.cc
 #define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
@@ -42,12 +42,10 @@
 #include "util/tools.h"
 #include "util/xml_to_json.h"
 
-void Web::Add::process()
+const std::string_view Web::Add::PAGE = "add";
+
+bool Web::Add::processPageAction(pugi::xml_node& element, const std::string& action)
 {
-    log_debug("add: start");
-
-    checkRequest();
-
     std::string objID = param("object_id");
     auto path = fs::path((objID == "0") ? FS_ROOT_DIRECTORY : hexDecodeString(objID));
     if (path.empty())
@@ -66,5 +64,6 @@ void Web::Add::process()
     } else {
         log_error("Failed to read {}: {}", path.c_str(), ec.message());
     }
-    log_debug("add: returning");
+
+    return true;
 }
