@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    add_object.cc - this file is part of MediaTomb.
+    web/add_object.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,7 +29,7 @@
     $Id$
 */
 
-/// \file add_object.cc
+/// \file web/add_object.cc
 #define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
@@ -43,7 +43,12 @@
 #include "util/tools.h"
 #include "util/xml_to_json.h"
 
-void Web::AddObject::addContainer(int parentID, const std::string& title, const std::string& upnp_class)
+const std::string_view Web::AddObject::PAGE = "add_object";
+
+void Web::AddObject::addContainer(
+    int parentID,
+    const std::string& title,
+    const std::string& upnp_class)
 {
     auto cont = content->addContainer(parentID, title, upnp_class);
 
@@ -54,7 +59,11 @@ void Web::AddObject::addContainer(int parentID, const std::string& title, const
     }
 }
 
-std::shared_ptr<CdsItem> Web::AddObject::addItem(int parentID, const std::string& title, const std::string& upnp_class, const fs::path& location)
+std::shared_ptr<CdsItem> Web::AddObject::addItem(
+    int parentID,
+    const std::string& title,
+    const std::string& upnp_class,
+    const fs::path& location)
 {
     auto item = std::make_shared<CdsItem>();
     item->setParentID(parentID);
@@ -67,15 +76,15 @@ std::shared_ptr<CdsItem> Web::AddObject::addItem(int parentID, const std::string
         log_debug("Hidden file '{}' cannot be added", item->getLocation().c_str());
         return {};
     }
-    std::string tmp = param("description");
-    if (!tmp.empty()) {
-        item->addMetaData(MetadataFields::M_DESCRIPTION, tmp);
+    std::string desc = param("description");
+    if (!desc.empty()) {
+        item->addMetaData(MetadataFields::M_DESCRIPTION, desc);
     }
 
-    tmp = param("mime-type");
-    if (tmp.empty())
-        tmp = MIMETYPE_DEFAULT;
-    item->setMimeType(tmp);
+    std::string mime = param("mime-type");
+    if (mime.empty())
+        mime = MIMETYPE_DEFAULT;
+    item->setMimeType(mime);
 
     std::string flags = param("flags");
     if (!flags.empty())
@@ -86,7 +95,12 @@ std::shared_ptr<CdsItem> Web::AddObject::addItem(int parentID, const std::string
     return item;
 }
 
-std::shared_ptr<CdsItemExternalURL> Web::AddObject::addUrl(int parentID, const std::string& title, const std::string& upnp_class, bool addProtocol, const fs::path& location)
+std::shared_ptr<CdsItemExternalURL> Web::AddObject::addUrl(
+    int parentID,
+    const std::string& title,
+    const std::string& upnp_class,
+    bool addProtocol,
+    const fs::path& location)
 {
     auto item = std::make_shared<CdsItemExternalURL>();
     std::string protocolInfo;
@@ -102,27 +116,31 @@ std::shared_ptr<CdsItemExternalURL> Web::AddObject::addUrl(int parentID, const s
         item->addMetaData(MetadataFields::M_DESCRIPTION, desc);
     }
 
-    std::string mime = param("mime-type");
-    if (mime.empty())
-        mime = MIMETYPE_DEFAULT;
-    item->setMimeType(mime);
-
     std::string flags = param("flags");
     if (!flags.empty())
         item->setFlags(CdsObject::makeFlag(flags));
 
-    if (addProtocol) {
-        std::string protocol = param("protocol");
-        if (!protocol.empty())
-            protocolInfo = renderProtocolInfo(mime, protocol);
-        else
-            protocolInfo = renderProtocolInfo(mime);
-    } else
-        protocolInfo = renderProtocolInfo(mime);
+    {
+        std::string mime = param("mime-type");
+        if (mime.empty())
+            mime = MIMETYPE_DEFAULT;
+        item->setMimeType(mime);
 
-    auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
-    resource->addAttribute(ResourceAttribute::PROTOCOLINFO, protocolInfo);
-    item->addResource(resource);
+        if (addProtocol) {
+            std::string protocol = param("protocol");
+            if (!protocol.empty())
+                protocolInfo = renderProtocolInfo(mime, protocol);
+            else
+                protocolInfo = renderProtocolInfo(mime);
+        } else
+            protocolInfo = renderProtocolInfo(mime);
+
+        auto resource = std::make_shared<CdsResource>(ContentHandler::DEFAULT, ResourcePurpose::Content);
+        resource->addAttribute(ResourceAttribute::PROTOCOLINFO, protocolInfo);
+        item->addResource(resource);
+    }
+
+    readResources(item);
 
     return item;
 }
@@ -137,10 +155,8 @@ bool Web::AddObject::isHiddenFile(const std::shared_ptr<CdsObject>& cdsObj)
     return content->isHiddenFile(fs::directory_entry(cdsObj->getLocation()), false, asSetting);
 }
 
-void Web::AddObject::process()
+bool Web::AddObject::processPageAction(pugi::xml_node& element, const std::string& action)
 {
-    checkRequest();
-
     auto title = std::string(param("title"));
     if (title.empty())
         throw_std_runtime_error("Empty 'title'");
@@ -186,4 +202,6 @@ void Web::AddObject::process()
             content->addObject(obj, true);
         }
     }
+
+    return true;
 }
