@@ -4,7 +4,7 @@
 
     web/clients.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2024 Gerbera Contributors
+    Copyright (C) 2020-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -43,12 +43,23 @@ static std::string secondsToString(const std::chrono::seconds& t)
     return fmt::format("{:%a %b %d %H:%M:%S %Y}", fmt::localtime(t.count()));
 }
 
-void Web::Clients::process()
-{
-    checkRequest();
-    auto root = xmlDoc->document_element();
+const std::string_view Web::Clients::PAGE = "clients";
 
-    auto clients = root.append_child("clients");
+bool Web::Clients::processPageAction(pugi::xml_node& element, const std::string& action)
+{
+    // Delete client
+    if (action == "delete") {
+        std::string clientIp = param("client_id");
+        if (!clientIp.empty()) {
+            log_debug("Deleting client with {} IP address", clientIp);
+            content->getContext()->getClients()->removeClient(clientIp);
+        } else {
+            log_warning("Cannot delete client without IP address");
+        }
+    }
+
+    // Return current list of clients
+    auto clients = element.append_child("clients");
     xml2Json->setArrayName(clients, "client");
 
     auto&& clientArr = content->getContext()->getClients()->getClientList();
@@ -71,7 +82,8 @@ void Web::Clients::process()
         item.append_attribute("clientType") = ClientConfig::mapClientType(obj.pInfo->type).data();
     }
 
-    auto groups = root.append_child("groups");
+    // Return current list of groups
+    auto groups = element.append_child("groups");
     xml2Json->setArrayName(groups, "group");
     auto&& groupArr = content->getContext()->getDatabase()->getClientGroupStats();
     for (auto&& obj : groupArr) {
@@ -82,4 +94,6 @@ void Web::Clients::process()
         item.append_attribute("bookmarks") = obj.at("bookmarks").c_str();
         item.append_attribute("last") = obj.at("last").c_str();
     }
+
+    return true;
 }
