@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -34,22 +34,23 @@
 
 #include "web_request_handler.h" // API
 
+#include "cds/cds_enums.h"
 #include "common.h"
 #include "config/config.h"
+#include "config/config_setup.h"
 #include "config/config_val.h"
 #include "content/content.h"
 #include "context.h"
 #include "exceptions.h"
 #include "iohandler/mem_io_handler.h"
 #include "server.h"
-#include "session_manager.h"
 #include "upnp/compat.h"
 #include "upnp/headers.h"
 #include "upnp/quirks.h"
-#include "util/generic_task.h"
 #include "util/url_utils.h"
 #include "util/xml_to_json.h"
 #include "web/pages.h"
+#include "web/session_manager.h"
 
 namespace Web {
 
@@ -63,37 +64,9 @@ WebRequestHandler::WebRequestHandler(const std::shared_ptr<Content>& content,
 {
 }
 
-int WebRequestHandler::intParam(const std::string& name, int invalid) const
+std::string WebRequestHandler::getGroup() const
 {
-    std::string value = param(name);
-    return !value.empty() ? std::stoi(value) : invalid;
-}
-
-bool WebRequestHandler::boolParam(const std::string& name) const
-{
-    std::string value = param(name);
-    return !value.empty() && (value == "1" || value == "true");
-}
-
-void WebRequestHandler::checkRequest(bool checkLogin)
-{
-    // we have a minimum set of parameters that are "must have"
-
-    // check if the session parameter was supplied and if we have
-    // a session with that id
-
-    checkRequestCalled = true;
-
-    std::string sid = param(SID);
-    if (sid.empty())
-        throw SessionException("no session id given");
-
-    if (!(session = sessionManager->getSession(sid)))
-        throw SessionException("invalid session id");
-
-    if (checkLogin && !session->isLoggedIn())
-        throw LoginException("not logged in");
-    session->access();
+    return quirks ? quirks->getGroup() : DEFAULT_CLIENT_GROUP;
 }
 
 bool WebRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
@@ -129,7 +102,10 @@ bool WebRequestHandler::getInfo(const char* filename, UpnpFileInfo* info)
     return quirks && quirks->getClient();
 }
 
-std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const std::shared_ptr<Quirks>& quirks, enum UpnpOpenFileMode mode)
+std::unique_ptr<IOHandler> WebRequestHandler::open(
+    const char* filename,
+    const std::shared_ptr<Quirks>& quirks,
+    enum UpnpOpenFileMode mode)
 {
     this->filename = filename;
     auto&& parameters = URLUtils::getQuery(this->filename);
@@ -150,6 +126,7 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const s
 
     std::string error;
     int errorCode = 0;
+    log_debug("start");
 
     std::string output;
     // processing page, creating output
@@ -159,14 +136,7 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const s
             error = "The UI is disabled in the configuration file. See README.";
             errorCode = 900;
         } else {
-            process();
-
-            if (checkRequestCalled) {
-                // add current task
-                appendTask(content->getCurrentTask(), root);
-
-                handleUpdateIDs();
-            }
+            process(root);
         }
     } catch (const LoginException& e) {
         error = e.what();
@@ -183,13 +153,13 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const s
     } catch (const std::runtime_error& e) {
         error = fmt::format("Error: {}", e.what());
         errorCode = 800;
+    } catch (const std::exception& e) {
+        error = fmt::format("Exception: {}", e.what());
+        errorCode = 800;
     }
 
-    if (error.empty()) {
-        root.append_attribute("success") = true;
-    } else {
-        root.append_attribute("success") = false;
-
+    root.append_attribute("success") = error.empty();
+    if (!error.empty()) {
         auto errorEl = root.append_child("error");
         errorEl.append_attribute("text") = error.c_str();
 
@@ -204,6 +174,8 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const s
         output = xml2Json->getJson(root);
     } catch (const std::runtime_error& e) {
         log_error("Web marshalling error: {}", e.what());
+    } catch (const std::exception& e) {
+        log_error("Web marshalling exception: {}", e.what());
     }
 
     log_debug("output-----------------------{}", output);
@@ -213,52 +185,61 @@ std::unique_ptr<IOHandler> WebRequestHandler::open(const char* filename, const s
     return ioHandler;
 }
 
-void WebRequestHandler::handleUpdateIDs()
-{
-    // session will be filled by check_request
-    std::string updates = param("updates");
-    if (!updates.empty()) {
-        auto root = xmlDoc->document_element();
-        auto updateIDs = root.append_child("update_ids");
+static std::map<AutoscanType, std::string_view> asTypeMap = {
+    { AutoscanType::Ui, "ui" },
+    { AutoscanType::Config, "persistent" },
+    { AutoscanType::None, "none" },
+};
 
-        if (updates == "check") {
-            updateIDs.append_attribute("pending") = session->hasUIUpdateIDs();
-        } else if (updates == "get") {
-            addUpdateIDs(session, updateIDs);
-        }
-    }
+std::string_view WebRequestHandler::mapAutoscanType(AutoscanType type)
+{
+    return asTypeMap.at(type);
 }
 
-void WebRequestHandler::addUpdateIDs(const std::shared_ptr<Session>& session, pugi::xml_node& updateIDsEl)
+std::unique_ptr<WebRequestHandler> createWebRequestHandler(
+    const std::shared_ptr<Context>& context,
+    const std::shared_ptr<Content>& content,
+    const std::shared_ptr<Server>& server,
+    const std::shared_ptr<UpnpXMLBuilder>& xmlBuilder,
+    const std::shared_ptr<Quirks>& quirks,
+    const std::string& page)
 {
-    std::string updateIDs = session->getUIUpdateIDs();
-    if (!updateIDs.empty()) {
-        log_debug("UI: sending update ids: {}", updateIDs);
-        updateIDsEl.append_attribute("ids") = updateIDs.c_str();
-        updateIDsEl.append_attribute("updates") = true;
-    }
-}
+    if (page == Web::Add::PAGE)
+        return std::make_unique<Web::Add>(content, server, xmlBuilder, quirks);
+    if (page == Web::Remove::PAGE)
+        return std::make_unique<Web::Remove>(content, server, xmlBuilder, quirks);
+    if (page == Web::AddObject::PAGE)
+        return std::make_unique<Web::AddObject>(content, server, xmlBuilder, quirks);
+    if (page == Web::Auth::PAGE)
+        return std::make_unique<Web::Auth>(content, server, xmlBuilder, quirks);
+    if (page == Web::Containers::PAGE)
+        return std::make_unique<Web::Containers>(content, server, xmlBuilder, quirks);
+    if (page == Web::Directories::PAGE)
+        return std::make_unique<Web::Directories>(content, context->getConverterManager(), server, xmlBuilder, quirks);
+    if (page == Web::Files::PAGE)
+        return std::make_unique<Web::Files>(content, context->getConverterManager(), server, xmlBuilder, quirks);
+    if (page == Web::Items::PAGE)
+        return std::make_unique<Web::Items>(content, server, xmlBuilder, quirks);
+    if (page == Web::EditLoad::PAGE)
+        return std::make_unique<Web::EditLoad>(content, server, xmlBuilder, quirks);
+    if (page == Web::EditSave::PAGE)
+        return std::make_unique<Web::EditSave>(content, server, xmlBuilder, quirks);
+    if (page == Web::Autoscan::PAGE)
+        return std::make_unique<Web::Autoscan>(content, server, xmlBuilder, quirks);
+    if (page == Web::VoidType::PAGE)
+        return std::make_unique<Web::VoidType>(content, server, xmlBuilder, quirks);
+    if (page == Web::Tasks::PAGE)
+        return std::make_unique<Web::Tasks>(content, server, xmlBuilder, quirks);
+    if (page == Web::Action::PAGE)
+        return std::make_unique<Web::Action>(content, server, xmlBuilder, quirks);
+    if (page == Web::Clients::PAGE)
+        return std::make_unique<Web::Clients>(content, server, xmlBuilder, quirks);
+    if (page == Web::ConfigLoad::PAGE)
+        return std::make_unique<Web::ConfigLoad>(content, server, xmlBuilder, quirks);
+    if (page == Web::ConfigSave::PAGE)
+        return std::make_unique<Web::ConfigSave>(context, content, server, xmlBuilder, quirks);
 
-void WebRequestHandler::appendTask(const std::shared_ptr<GenericTask>& task, pugi::xml_node& parent)
-{
-    if (!task || !parent)
-        return;
-    auto taskEl = parent.append_child("task");
-    taskEl.append_attribute("id") = task->getID();
-    taskEl.append_attribute("cancellable") = task->isCancellable();
-    taskEl.append_attribute("text") = task->getDescription().c_str();
-}
-
-std::string_view WebRequestHandler::mapAutoscanType(int type)
-{
-    switch (type) {
-    case 1:
-        return "ui";
-    case 2:
-        return "persistent";
-    default:
-        return "none";
-    }
+    throw_std_runtime_error("Unknown page: {}", page);
 }
 
 } // namespace Web
