@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    web_update.cc - this file is part of MediaTomb.
+    web/web_update.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,6 +11,8 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
+    Copyright (C) 2016-2025 Gerbera Contributors
+
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
     as published by the Free Software Foundation.
@@ -27,14 +29,17 @@
     $Id$
 */
 
-/// \file web_update.cc
+/// \file web/web_update.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
-#include "database/database.h"
+#include "content/content.h"
+#include "util/xml_to_json.h"
 
-void Web::VoidType::process()
+const std::string_view Web::VoidType::PAGE = "void";
+
+bool Web::VoidType::processPageAction(pugi::xml_node& element, const std::string& action)
 {
-    checkRequest();
+    return true;
 }
