@@ -3,7 +3,7 @@
 
     config_mock.h - this file is part of Gerbera.
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -45,7 +45,18 @@ public:
     std::uint32_t getUIntOption(ConfigVal option) const override { return 0; }
     std::int64_t getLongOption(ConfigVal option) const override { return 0; }
     std::shared_ptr<ConfigOption> getConfigOption(ConfigVal option) const override { return {}; }
-    bool getBoolOption(ConfigVal option) const override { return false; }
+    bool getBoolOption(ConfigVal option) const override
+    {
+        switch (option) {
+#ifdef HAVE_FFMPEGTHUMBNAILER
+        case ConfigVal::SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED:
+            return true;
+#endif
+        default:
+            return false;
+        }
+    }
+    std::string generateUDN(const std::shared_ptr<Database>& database) override { return "uuid:12345678-1234-1234-1234-123456789abc"; };
     std::map<std::string, std::string> getDictionaryOption(ConfigVal option) const override
     {
         switch (option) {
