@@ -3,7 +3,7 @@
 
     test_tools.cc - this file is part of Gerbera.
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -121,6 +121,14 @@ TEST(ToolsTest, subdirTest)
     EXPECT_EQ(isSubDir(test2, test), true);
 }
 
+TEST(ToolsTest, subdirFileTest)
+{
+    std::string test = "/regular/path";
+    std::string test2 = "/regular/path-sub";
+    EXPECT_EQ(isSubDir(test, test2), false);
+    EXPECT_EQ(isSubDir(test2, test), false);
+}
+
 TEST(ToolsTest, splitStringTest)
 {
     auto parts = splitString("", ',');
@@ -181,6 +189,15 @@ TEST(ToolsTest, trimStringTest)
     EXPECT_EQ(trimString("  AB  "), "AB");
 }
 
+TEST(ToolsTest, camelCaseStringTest)
+{
+    EXPECT_EQ(camelCaseString(""), "");
+    EXPECT_EQ(camelCaseString("AB"), "AB");
+    EXPECT_EQ(camelCaseString("upnp-test"), "upnpTest");
+    EXPECT_EQ(camelCaseString("upnptest-"), "upnptest");
+    EXPECT_EQ(camelCaseString("-upnptest"), "Upnptest");
+}
+
 TEST(ToolsTest, startswithTest)
 {
     EXPECT_EQ(startswith("AB", "AB"), true);
