@@ -4,7 +4,7 @@
 
     gerbera-tree.module.js - this file is part of Gerbera.
 
-    Copyright (C) 2016-2021 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -20,12 +20,13 @@
 
     $Id$
 */
-import {Autoscan} from "./gerbera-autoscan.module.js";
-import {Auth} from "./gerbera-auth.module.js";
-import {GerberaApp} from "./gerbera-app.module.js";
-import {Trail} from "./gerbera-trail.module.js";
-import {Items} from "./gerbera-items.module.js";
-import {Updates} from "./gerbera-updates.module.js";
+import { Autoscan } from "./gerbera-autoscan.module.js";
+import { Auth } from "./gerbera-auth.module.js";
+import { GerberaApp } from "./gerbera-app.module.js";
+import { Trail } from "./gerbera-trail.module.js";
+import { Tweaks } from "./gerbera-tweak.module.js";
+import { Items } from "./gerbera-items.module.js";
+import { Updates } from "./gerbera-updates.module.js";
 
 const treeViewCss = {
   titleClass: 'folder-title',
@@ -35,11 +36,13 @@ const treeViewCss = {
 
 let currentTree = [];
 let currentType = 'db';
+let currentView = undefined;
 let pendingItems = [];
 
 const destroy = () => {
   currentTree = [];
   currentType = 'db';
+  currentView = undefined;
   const tree = $('#tree');
   if (tree.hasClass('grb-tree')) {
     tree.tree('destroy');
@@ -47,27 +50,95 @@ const destroy = () => {
   return Promise.resolve();
 };
 
+class TreeView {
+  parentId = -1;
+  constructor(pId) {
+    this.parentId = pId;
+  }
+  get requestData() {
+    let result = {
+      parent_id: this.parentId,
+      select_it: 0
+    };
+    result[Auth.SID] = Auth.getSessionId();
+    return result;
+  }
+  checkForUpdates() {
+    return Promise.resolve();
+  }
+  hasClass() { return true; }
+}
+
+class BrowseTreeView extends TreeView {
+  constructor(pId) {
+    super(pId);
+  }
+  get requestData() {
+    let result = super.requestData;
+    result.req_type = 'containers';
+    result.action = 'browse';
+    return result;
+  }
+  checkForUpdates() {
+    return Updates.getUpdates(true);
+  }
+}
+
+class SearchTreeView extends TreeView {
+  constructor(pId) {
+    super(pId);
+  }
+  get requestData() {
+    let result = super.requestData;
+    result.req_type = 'containers';
+    result.action = 'search';
+    return result;
+  }
+  hasClass(cls) { return (cls === undefined || !cls.endsWith('dynamicFolder')); }
+}
+
+class FileTreeView extends TreeView {
+  constructor(pId) {
+    super(pId);
+  }
+  get requestData() {
+    let result = super.requestData;
+    result.req_type = 'directories';
+    result.action = 'browse';
+    return result;
+  }
+}
+
 const initialize = () => {
   $('#tree').html('');
   return Promise.resolve();
 };
 
-const selectType = (type, parentId) => {
+const viewFactory = (type, parentId) => {
   currentType = type;
-  const linkType = (currentType === 'db') ? 'containers' : 'directories';
+  switch (currentType) {
+    case 'db':
+      currentView = new BrowseTreeView(parentId);
+      break;
+    case 'search':
+      currentView = new SearchTreeView(parentId);
+      break;
+    default:
+      currentView = new FileTreeView(parentId);
+      break;
+  }
+  return currentView;
+};
 
+const selectType = (type, parentId) => {
+  viewFactory(type, parentId);
   return $.ajax({
     url: GerberaApp.clientConfig.api,
     type: 'get',
-    data: {
-      req_type: linkType,
-      sid: Auth.getSessionId(),
-      parent_id: parentId,
-      select_it: 0
-    }
+    data: currentView.requestData
   })
     .then((response) => loadTree(response))
-    .then(() => checkForUpdates())
+    .then(() => currentView.checkForUpdates())
     .catch((err) => GerberaApp.error(err))
 };
 
@@ -91,12 +162,15 @@ const onItemExpanded = (event) => {
 
   if (item.gerbera && item.gerbera.id) {
     if (tree.tree('closed', folderName) && item.gerbera.childCount > 0) {
-      fetchChildren(item.gerbera).then(function (response) {
-        var childTree = transformContainers(response, false);
-        tree.tree('append', $(folderList), childTree);
-
-        initSelection(pendingItems);
-      })
+      fetchChildren(item.gerbera)
+        .then(function (response) {
+          if (response) {
+            var childTree = transformContainers(response, false);
+            tree.tree('append', $(folderList), childTree);
+            initSelection(pendingItems);
+          }
+        })
+        .catch((err) => GerberaApp.error(err));
     } else {
       tree.tree('collapse', $(folderList));
     }
@@ -107,6 +181,10 @@ const onAutoscanEdit = (event) => {
   Autoscan.addAutoscan(event);
 };
 
+const onTweakEdit = (event) => {
+  Tweaks.addDirTweak(event);
+};
+
 const selectTreeItem = (folderList) => {
   const tree = $('#tree');
   tree.tree('select', folderList);
@@ -117,7 +195,8 @@ const loadTree = (response, config) => {
   const treeViewEvents = {
     onSelection: onItemSelected,
     onExpand: onItemExpanded,
-    onAutoscanEdit: onAutoscanEdit
+    onAutoscanEdit: onAutoscanEdit,
+    onTweakEdit: onTweakEdit,
   };
 
   config = $.extend({}, treeViewCss, treeViewEvents, config);
@@ -138,7 +217,7 @@ const loadTree = (response, config) => {
 
     pendingItems = GerberaApp.currentItem();
 
-    if (pendingItems){
+    if (pendingItems) {
       initSelection(pendingItems);
     }
   }
@@ -165,21 +244,15 @@ const initSelection = (currentItems) => {
 
 const fetchChildren = (gerberaData) => {
   currentType = GerberaApp.getType();
-  const linkType = (currentType === 'db') ? 'containers' : 'directories';
+  currentView.parentId = gerberaData.id;
   return $.ajax({
     url: GerberaApp.clientConfig.api,
     type: 'get',
-    data: {
-      req_type: linkType,
-      sid: Auth.getSessionId(),
-      parent_id: gerberaData.id,
-      select_it: 0
-    }
+    data: currentView.requestData
   })
-  .catch((err) => {
-    console.log(err);
-    GerberaApp.error(err);
-  });
+    .catch((err) => {
+      GerberaApp.error(err);
+    });
 };
 
 const transformContainers = (response, createParent) => {
@@ -207,6 +280,8 @@ const transformContainers = (response, createParent) => {
 
   for (let i = 0; i < items.length; i++) {
     const item = items[i];
+    if (currentView && !currentView.hasClass(item.upnp_class))
+      continue;
     const node = {};
     node.title = item.title;
     node.badge = generateBadges(item);
@@ -216,10 +291,16 @@ const transformContainers = (response, createParent) => {
     node.gerbera = {
       id: item.id,
       childCount: item.child_count,
+      path: item.location ?? null,
+      upnpClass: item.upnp_class ?? null,
+      upnpShortcut: item.upnp_shortcut ?? null,
       autoScanMode: item.autoscan_mode,
       autoScanType: item.autoscan_type,
       image: ('image' in item) ? item.image : null
     };
+    if (!GerberaApp.serverConfig.enableThumbnail) {
+      node.gerbera.image = null;
+    }
     if (createParent) {
       parent.nodes.push(node);
     } else {
@@ -238,8 +319,12 @@ const generateBadges = (item) => {
 
   if (item.autoscan_type === 'ui' || item.autoscan_type === 'persistent') {
     badges.push('a');
+  } else if (item.autoscan_type === 'parent') {
+    badges.push('p');
+  }
+  if ('tweak' in item && (item.tweak || item.tweak === 'true')) {
+    badges.push('t');
   }
-
   return badges;
 };
 
@@ -247,12 +332,14 @@ const reloadTreeItem = (folderList) => {
   const tree = $('#tree');
   const item = folderList.data('grb-item');
   tree.tree('collapse', folderList);
-  return fetchChildren(item.gerbera).then(function (response) {
-    var childTree = transformContainers(response, false);
-    tree.tree('append', folderList, childTree);
-    selectTreeItem(folderList);
-    Items.treeItemSelected(item);
-  });
+  return fetchChildren(item.gerbera)
+    .then(function (response) {
+      var childTree = transformContainers(response, false);
+      tree.tree('append', folderList, childTree);
+      selectTreeItem(folderList);
+      Items.treeItemSelected(item);
+    })
+    .catch((err) => GerberaApp.error(err));
 };
 
 const reloadTreeItemById = (id) => {
@@ -268,14 +355,6 @@ const reloadParentTreeItem = (id) => {
   }
 };
 
-const checkForUpdates = () => {
-  if (GerberaApp.getType() === 'db') {
-    return Updates.getUpdates(true);
-  } else {
-    return Promise.resolve();
-  }
-};
-
 const getTreeElementById = (id) => {
   return $('#tree').tree('getElement', id);
 };
@@ -285,11 +364,13 @@ export const Tree = {
   getTreeElementById,
   initialize,
   loadTree,
+  onTweakEdit,
   onAutoscanEdit,
   reloadParentTreeItem,
   reloadTreeItem,
   reloadTreeItemById,
   selectType,
+  viewFactory,
   transformContainers,
   currentTree,
 };
